@@ -155,7 +155,8 @@ public class ConfigurationDefaults {
     def.put("Listen.Port.Randomize.Range", RandomUtils.LISTEN_PORT_MIN + "-" + RandomUtils.LISTEN_PORT_MAX );
     def.put(ConfigKeys.Connection.BCFG_PEERCONTROL_TCP_PUBLIC_ENABLE, TRUE );
     def.put(ConfigKeys.Connection.BCFG_PEERCONTROL_UDP_PUBLIC_ENABLE, TRUE );
-    def.put("webseed.activation.uses.availability", TRUE );
+    def.put(ConfigKeys.Connection.BCFG_WEBSEED_ACTIVATION_USES_AVAILABILITY, TRUE );
+    def.put(ConfigKeys.Connection.BCFG_WEBSEED_ACTIVATION_MIN_SPEED_KBPS, 5 );
     def.put("IPV6 Enable Support", FALSE );
     def.put(ConfigKeys.Connection.BCFG_IPV_6_CHECK_MULTIPLE_ADDRESS_CHECKS, TRUE );
     def.put("IPV6 Prefer Addresses",FALSE );
@@ -230,6 +231,7 @@ public class ConfigurationDefaults {
     def.put("Merge Same Size Files Tolerance", ZERO );
     def.put(ConfigKeys.File.ICFG_MERGE_SAME_SIZE_FILES_MIN_PIECES, 5 );
     def.put(ConfigKeys.File.BCFG_SKIP_COMP_DL_FILE_CHECKS, FALSE );
+    def.put(ConfigKeys.File.BCFG_FILE_MOVE_ORIGIN_DELETE_FAIL_IS_WARNING, FALSE );
     def.put("Stop Ratio", new Float(0));
     def.put("Stop Peers Ratio", ZERO);
     def.put("Disconnect Seed", TRUE);
@@ -255,6 +257,7 @@ public class ConfigurationDefaults {
     def.put("Ip Filter Ban Discard Ratio", "5.0" );
     def.put("Ip Filter Ban Discard Min KB", new Long(128));
     def.put("Ip Filter Banning Persistent", TRUE);
+    def.put(ConfigKeys.IPFilter.BCFG_IP_FILTER_DONT_BAN_LAN, FALSE );
     def.put("Ip Filter Enable Description Cache", TRUE);
     def.put(ConfigKeys.IPFilter.SCFG_IP_FILTER_AUTOLOAD_FILE, "" );
     def.put(ConfigKeys.IPFilter.SCFG_IP_FILTER_V6_AUTOLOAD_FILE, "");
@@ -265,6 +268,7 @@ public class ConfigurationDefaults {
     def.put("Allow Same IP Peers",FALSE);
     def.put(ConfigKeys.Transfer.ICFG_IPv4_IPv6_CONN_ACTION, ZERO );
     def.put("Use Super Seeding",FALSE);
+    def.put("Enable Light Seeding", TRUE);
     def.put("StartStopManager_bAddForDownloadingSR1", TRUE);
     
     def.put("Start On Login", FALSE );
@@ -408,6 +412,7 @@ public class ConfigurationDefaults {
     def.put( "Watch Torrent Folder Path", "" );
     def.put( "Watch Torrent Folder Path Count", ONE );
     def.put( ConfigKeys.Transfer.BCFG_PRIORITIZE_FIRST_PIECE, FALSE );
+    def.put( ConfigKeys.Transfer.ICFG_PRIORITIZE_FIRST_MB, 0 );
     def.put( ConfigKeys.Transfer.BCFG_PRIORITIZE_FIRST_PIECE_FORCE, FALSE );
     def.put( "Prioritize Most Completed Files", FALSE );
     def.put( "Piece Picker Request Hint Enabled", TRUE );
@@ -425,6 +430,7 @@ public class ConfigurationDefaults {
     def.put( "Popup Download Finished", FALSE);
     def.put( "Popup File Finished", FALSE);
     def.put( "Popup Download Added", FALSE);
+    def.put( "Popup Check Complete", FALSE );
     def.put( "Notify Download Finished", FALSE);
     def.put( "Show Timestamp For Alerts", FALSE);
     def.put( "Request Attention On New Download", TRUE );
@@ -445,13 +451,14 @@ public class ConfigurationDefaults {
     def.put( "Play Download Error Announcement", FALSE);
     def.put( "Play Download Error Announcement Text", "Download Error");
     def.put( "Popup Download Error", FALSE);
-
+    
     def.put( "Play Notification Added Announcement", FALSE );
     def.put( "Play Notification Added Announcement Text", "Notification Added" );
     def.put( "Play Notification Added", FALSE );
     def.put( "Play Notification Added File", "" );
 
     def.put( Tag.BCFG_TRACKER_AUTO_TAG_INTERESTING_TRACKERS, TRUE );
+    def.put( Tag.ICFG_TAG_AUTO_FULL_REAPPLY_PERIOD_SECS, 30 );
     
     //default torrent directory option
     def.put( "Save Torrent Files", TRUE );
@@ -464,7 +471,10 @@ public class ConfigurationDefaults {
     def.put( "Enforce Bind IP", FALSE);
     def.put( "Enforce Bind IP Pause", FALSE);
     def.put( "Show IP Bindings Icon", TRUE );
-
+    def.put( ConfigKeys.Connection.SCFG_NETWORK_ADDITIONAL_SERVICE_BINDS, "" );
+    def.put( ConfigKeys.Connection.BCFG_NETWORK_IGNORE_BIND_FOR_LAN, FALSE );
+    
+    
     def.put( "Stats Period", Long.valueOf(30) );
     def.put( "Stats Export Peer Details", FALSE );
     def.put( "Stats Export File Details", FALSE );
@@ -606,6 +616,7 @@ public class ConfigurationDefaults {
     def.put( "Alert on close", FALSE );
     def.put( "diskmanager.hashchecking.strategy", ONE );
     def.put( "diskmanager.hashchecking.smallestfirst", TRUE );
+    def.put( ConfigKeys.File.BCFG_DISKMANAGER_ONE_OP_PER_FS, FALSE );
     def.put( "diskmanager.hashchecking.maxactive", ONE );
     def.put( "Default Start Torrents Stopped", FALSE);
     def.put( "Default Start Torrents Stopped Auto Pause", FALSE );
@@ -619,9 +630,9 @@ public class ConfigurationDefaults {
     def.put( "diskmanager.perf.cache.size", new Long(4));		// 4 MB
     def.put( "diskmanager.perf.cache.notsmallerthan", new Long(1024));	// 1024 K
     def.put( "diskmanager.perf.read.maxthreads", new Long(32));
-    def.put( "diskmanager.perf.read.maxmb", new Long(5));
+    def.put( "diskmanager.perf.read.maxmb", new Long(10));
     def.put( "diskmanager.perf.write.maxthreads", new Long(32));
-    def.put( "diskmanager.perf.write.maxmb", new Long(5));
+    def.put( "diskmanager.perf.write.maxmb", new Long(10));
     def.put( "diskmanager.perf.cache.trace", FALSE);
     def.put( "diskmanager.perf.cache.flushpieces", TRUE);
     def.put( "diskmanager.perf.read.aggregate.enable", FALSE );
@@ -745,7 +756,8 @@ public class ConfigurationDefaults {
     def.put("FileBrowse.usePathFinder", FALSE);
 
     def.put( "Beta Programme Enabled", FALSE );
-    def.put( "def.deletetorrent", TRUE );
+    def.put( ConfigKeys.File.BCFG_DEF_DELETETORRENT, TRUE );
+    def.put( ConfigKeys.File.BCFG_DEF_DELETEALLSELECTED, FALSE );
     def.put( "tb.confirm.delete.content", Long.valueOf(0) ); // 0=ask; 1=delete content; 2=delete torrent
 
     def.put( "br.backup.auto.enable", FALSE );
