@@ -28,11 +28,12 @@ import com.biglybt.core.config.ParameterListener;
 import com.biglybt.core.disk.*;
 import com.biglybt.core.disk.impl.DiskManagerFileInfoImpl;
 import com.biglybt.core.disk.impl.DiskManagerHelper;
-import com.biglybt.core.disk.impl.DiskManagerRecheckInstance;
+import com.biglybt.core.disk.impl.DiskManagerRecheckScheduler.DiskManagerRecheckInstance;
 import com.biglybt.core.disk.impl.access.DMChecker;
 import com.biglybt.core.disk.impl.piecemapper.DMPieceList;
 import com.biglybt.core.disk.impl.piecemapper.DMPieceMapEntry;
 import com.biglybt.core.diskmanager.cache.CacheFile;
+import com.biglybt.core.global.GlobalManagerEvent;
 import com.biglybt.core.logging.LogEvent;
 import com.biglybt.core.logging.LogIDs;
 import com.biglybt.core.logging.Logger;
@@ -407,6 +408,13 @@ DMCheckerImpl
 					  	       					complete();
 					  	       				}			  	       			}
 
+					  	       			@Override
+					  	       			public boolean 
+					  	       			isFailureInteresting()
+					  	       			{	
+					  	       				return( listener.isFailureInteresting());
+					  	       			}
+					  	       			
 					  	       			protected void
 					  	       			complete()
 					  	       			{
@@ -432,6 +440,13 @@ DMCheckerImpl
 		  	       		complete_recheck_in_progress	= false;
 
 		  	       		recheck_inst.unregister();
+		  	       		
+		  	       		if ( !stopped ){
+		  	       			
+		  	       			disk_manager.getDownload().fireGlobalManagerEvent(
+		  	       				GlobalManagerEvent.ET_RECHECK_COMPLETE,
+		  	       				request.isExplicit());
+		  	       		}
 		  	       }
 		        }
 		 	}.start();
@@ -555,8 +570,12 @@ DMCheckerImpl
 									Logger.log(new LogEvent(disk_manager, LOGID, LogEvent.LT_INFORMATION,
 												"Piece " + request.getPieceNumber() + " passed hash check."));
 								}else{
-									Logger.log(new LogEvent(disk_manager, LOGID, LogEvent.LT_WARNING,
-												"Piece " + request.getPieceNumber() + " failed hash check."));
+									Logger.log(
+										new LogEvent(
+											disk_manager, 
+											LOGID, 
+											listener.isFailureInteresting()?LogEvent.LT_WARNING:LogEvent.LT_INFORMATION,
+											"Piece " + request.getPieceNumber() + " failed hash check."));
 								}
 							}
 						}
@@ -605,6 +624,13 @@ DMCheckerImpl
 						}
 					}
 					
+					@Override
+					public boolean 
+					isFailureInteresting()
+					{
+						return( listener.isFailureInteresting());
+					}
+					
 					@Override
 					public boolean 
 					hashRequest(
@@ -767,6 +793,8 @@ DMCheckerImpl
 
 		   	read_request.setUseCache( !request.isAdHoc());
 
+		   	read_request.setErrorIsFatal( request.getErrorIsFatal());
+		   	
 			disk_manager.enqueueReadRequest(
 				read_request,
 				new DiskManagerReadRequestListener()
