@@ -20,8 +20,10 @@
 package com.biglybt.core.disk.impl.access.impl;
 
 import java.util.*;
+import java.util.concurrent.ConcurrentHashMap;
 
 import com.biglybt.core.disk.*;
+import com.biglybt.core.disk.impl.DiskManagerAllocationScheduler;
 import com.biglybt.core.disk.impl.DiskManagerFileInfoImpl;
 import com.biglybt.core.disk.impl.DiskManagerHelper;
 import com.biglybt.core.disk.impl.access.DMWriter;
@@ -50,12 +52,16 @@ DMWriterImpl
 
 	private static final int		MIN_ZERO_BLOCK	= 1*1024*1024;	// must be mult of 1024 (see init below)
 
-	final DiskManagerHelper		disk_manager;
-	final DiskAccessController	disk_access;
+	private final DiskManagerHelper		disk_manager;
+	private final DiskAccessController	disk_access;
 
-	int				async_writes;
-	final Set				write_requests		= new HashSet();
-	final AESemaphore		async_write_sem 	= new AESemaphore("DMWriter::asyncWrite");
+	private final ConcurrentHashMap<DiskAccessRequest,String>		active_requests = new ConcurrentHashMap<>();
+
+	private int						async_writes;
+	private long					async_writes_bytes;
+	
+	private final Set<DiskManagerWriteRequest>	write_requests		= new HashSet<>();
+	private final AESemaphore					async_write_sem 	= new AESemaphore("DMWriter::asyncWrite");
 
 	private boolean	started;
 
@@ -67,6 +73,11 @@ DMWriterImpl
 
 	final AEMonitor		this_mon	= new AEMonitor( "DMWriter" );
 
+	private long					total_write_ops;
+	private long					total_write_bytes;
+
+	private volatile long			latency;
+	
 	public
 	DMWriterImpl(
 		DiskManagerHelper	_disk_manager )
@@ -163,10 +174,11 @@ DMWriterImpl
 	@Override
 	public boolean
 	zeroFile(
-		DiskManagerFileInfoImpl file,
-		long					start_from,
-		long 					overall_length,
-		ProgressListener		listener )
+		DiskManagerAllocationScheduler.AllocationInstance		allocation_instance,
+		DiskManagerFileInfoImpl 								file,
+		long													start_from,
+		long 													overall_length,
+		ProgressListener										listener )
 				
 		throws DiskManagerException
 	{
@@ -207,8 +219,30 @@ DMWriterImpl
 
 					buffer.position(DirectByteBuffer.SS_DW, 0);
 
+					long	time = SystemTime.getMonotonousTime();
+					
 					while ( remainder > 0 && !stopped ){
 
+						long	now = SystemTime.getMonotonousTime();
+
+						if ( now - time >= 250 ){
+							
+							time = now;
+							
+							while( !stopped ){
+	
+								if ( allocation_instance.getPermission()){
+	
+									break;
+								}
+							}
+						
+							if ( stopped ){
+							
+								break;
+							}
+						}
+					       
 						int	write_size = buffer_size;
 
 						if ( remainder < write_size ){
@@ -228,11 +262,23 @@ DMWriterImpl
 								false,
 								new DiskAccessRequestListener()
 								{
+									@Override
+									public void 
+									requestQueued(
+										DiskAccessRequest request)
+									{
+										active_requests.put( request, "" );
+									}	
+									
 									@Override
 									public void
 									requestComplete(
 										DiskAccessRequest	request )
 									{
+										latency = SystemTime.getMonotonousTime() - request.getCreateMonoTime();
+
+										active_requests.remove( request );
+								
 										sem.release();
 									}
 
@@ -335,11 +381,7 @@ DMWriterImpl
 		try{
 			this_mon.enter();
 
-			Iterator	it = write_requests.iterator();
-
-			while( it.hasNext()){
-
-				DiskManagerWriteRequest	request = (DiskManagerWriteRequest)it.next();
+			for ( DiskManagerWriteRequest request: write_requests ){
 
 				if ( request.getPieceNumber() == piece_number ){
 
@@ -411,7 +453,7 @@ DMWriterImpl
 				int	buffer_position = buffer.position(DirectByteBuffer.SS_DW);
 				int buffer_limit	= buffer.limit(DirectByteBuffer.SS_DW);
 
-				// final long	write_length = buffer_limit - buffer_position;
+				final long	write_length = buffer_limit - buffer_position;
 
 				int previousFilesLength = 0;
 
@@ -538,10 +580,15 @@ DMWriterImpl
 
 								async_writes--;
 
+								async_writes_bytes -= write_length;
+								
 								if ( !write_requests.remove( request )){
 
 									Debug.out( "request not found" );
 								}
+								
+								total_write_ops++;
+								total_write_bytes	+= write_length;
 
 								if ( stopped ){
 
@@ -569,6 +616,8 @@ DMWriterImpl
 
 						async_writes++;
 
+						async_writes_bytes += write_length;
+						
 						write_requests.add( request );
 					}
 
@@ -591,6 +640,29 @@ DMWriterImpl
 		}
 	}
 
+	@Override
+	public long[]
+	getStats()
+	{
+		return( new long[]{ total_write_ops, total_write_bytes, async_writes, async_writes_bytes });
+	}
+	
+	@Override
+	public long
+	getLatency()
+	{
+		long	result = latency;
+		
+		long now = SystemTime.getMonotonousTime();
+		
+		for ( DiskAccessRequest req: active_requests.keySet()){
+			
+			result = Math.max( result, now - req.getCreateMonoTime());
+		}
+		
+		return( result );
+	}
+	
 	protected class
 	requestDispatcher
 		implements DiskAccessRequestListener
@@ -657,6 +729,14 @@ DMWriterImpl
 							doRequest(
 								new DiskAccessRequestListener()
 								{
+									@Override
+									public void 
+									requestQueued(
+										DiskAccessRequest request)
+									{
+										// don't need to track this, done in doRequest
+									}	
+
 									@Override
 									public void
 									requestComplete(
@@ -755,11 +835,23 @@ DMWriterImpl
 			DiskAccessRequestListener	delegate_listener =
 				new DiskAccessRequestListener()
 				{
+					@Override
+					public void 
+					requestQueued(
+						DiskAccessRequest request)
+					{
+						active_requests.put( request, "" );
+					}	
+					
 					@Override
 					public void
 					requestComplete(
 						DiskAccessRequest	request )
 					{
+						latency = SystemTime.getMonotonousTime() - request.getCreateMonoTime();
+
+						active_requests.remove( request );
+						
 						l.requestComplete( request );
 
 						file.dataWritten( request.getOffset(), request.getSize(), request.getUserData());
@@ -811,6 +903,14 @@ DMWriterImpl
 				delegate_listener );
 		}
 
+		@Override
+		public void 
+		requestQueued(
+			DiskAccessRequest request)
+		{
+			// don't need to track latency here as done in doRequest
+		}	
+
 		@Override
 		public void
 		requestComplete(
