@@ -20,6 +20,7 @@
 package com.biglybt.core.disk.impl.resume;
 
 import java.util.*;
+import java.util.function.Consumer;
 
 import com.biglybt.core.config.COConfigurationManager;
 import com.biglybt.core.config.ConfigKeys;
@@ -27,13 +28,14 @@ import com.biglybt.core.config.ParameterListener;
 import com.biglybt.core.disk.*;
 import com.biglybt.core.disk.impl.DiskManagerFileInfoImpl;
 import com.biglybt.core.disk.impl.DiskManagerImpl;
-import com.biglybt.core.disk.impl.DiskManagerRecheckInstance;
+import com.biglybt.core.disk.impl.DiskManagerRecheckScheduler.DiskManagerRecheckInstance;
 import com.biglybt.core.disk.impl.access.DMChecker;
 import com.biglybt.core.disk.impl.piecemapper.DMPieceList;
 import com.biglybt.core.disk.impl.piecemapper.DMPieceMapEntry;
 import com.biglybt.core.diskmanager.cache.CacheFileManagerException;
 import com.biglybt.core.download.DownloadManager;
 import com.biglybt.core.download.DownloadManagerState;
+import com.biglybt.core.logging.LogAlert;
 import com.biglybt.core.logging.LogEvent;
 import com.biglybt.core.logging.LogIDs;
 import com.biglybt.core.logging.Logger;
@@ -136,6 +138,7 @@ RDResumeHandler
 	public void
 	checkAllPieces(
 		boolean 			newfiles,
+		boolean				forceRecheck,
 		ProgressListener	listener )
 	{
 		//long	start = System.currentTimeMillis();
@@ -144,6 +147,32 @@ RDResumeHandler
 			 
         final List<DiskManagerCheckRequest>	failed_pieces = new ArrayList<>();
 
+        Consumer<Throwable> forceRecheckErrorReporter =
+        	new Consumer<Throwable>(){
+        		private boolean reported;
+        		
+				@Override
+				public void 
+				accept(
+					Throwable cause)
+				{
+					synchronized( this ){
+						
+						if ( reported ){
+							
+							return;
+						}
+						
+						reported = true;
+					}
+					
+					String msg = disk_manager.getDisplayName() + ": One or more piece checks failed: " + Debug.getNestedExceptionMessage( cause );
+					
+            		Logger.log(new LogAlert( disk_manager, LogAlert.REPEATABLE, LogAlert.AT_ERROR, msg));
+
+				}
+        	};
+        
         listener.percentDone( 0 );
         
 		try{
@@ -181,15 +210,6 @@ RDResumeHandler
 					DiskManagerFileInfo[]	files = disk_manager.getFiles();
 					
 					file_sizes = new HashMap<>();
-	
-					for (int i=0;i<files.length;i++){
-						try{
-							Long	len = new Long(((DiskManagerFileInfoImpl)files[i]).getCacheFile().getLength());
-							file_sizes.put( files[i], len );
-						}catch( CacheFileManagerException e ){
-							Debug.printStackTrace(e);
-						}
-					}
 				}
 
 				if ( resumeEnabled ){
@@ -347,9 +367,26 @@ RDResumeHandler
 
 									DMPieceMapEntry	entry = list.get(j);
 									
-									Long	file_size 		= (Long)file_sizes.get(entry.getFile());
-	
+									DiskManagerFileInfo file = entry.getFile();
+									
+									Long file_size = file_sizes.get( file );
+									
 									if ( file_size == null ){
+									
+										try{
+											file_size = new Long(((DiskManagerFileInfoImpl)file).getCacheFile().getLength());
+																						
+										}catch( CacheFileManagerException e ){
+											
+											Debug.printStackTrace(e);
+											
+											file_size = -1L;
+										}
+										
+										file_sizes.put( file, file_size );
+									}
+
+									if ( file_size == -1 ){
 	
 										piece_state	= PIECE_NOT_DONE;
 										pieceCannotExist = true;
@@ -398,8 +435,9 @@ RDResumeHandler
 
 								// We only need to recheck pieces that are marked as not-ok
 								// if the resume data is invalid or explicit recheck needed
-							if(pieceCannotExist)
-							{
+							
+							if (pieceCannotExist){
+								
 								dm_piece.setDone( false );
 								
 							} else if ( piece_state == PIECE_RECHECK_REQUIRED || !resumeValid ){
@@ -435,6 +473,11 @@ RDResumeHandler
 
 										request.setLowPriority( true );
 
+										if ( forceRecheck ){
+										
+											request.setErrorIsFatal( false );
+										}
+										
 										checker.enqueueCheckRequest(
 											request,
 											new DiskManagerCheckRequestListener()
@@ -477,6 +520,11 @@ RDResumeHandler
 													DiskManagerCheckRequest 	request,
 													Throwable		 			cause )
 												{
+													if ( forceRecheck ){
+														
+														forceRecheckErrorReporter.accept( cause );
+													}
+													
 													complete();
 												}
 
@@ -487,6 +535,13 @@ RDResumeHandler
 
 													pending_checks_sem.release();
 												}
+												
+												@Override
+												public boolean 
+												isFailureInteresting()
+												{
+													return( false );
+												}
 											});
 
 										pending_check_num++;
@@ -604,6 +659,11 @@ RDResumeHandler
 
 							request.setLowPriority( true );
 
+							if ( forceRecheck ){
+								
+								request.setErrorIsFatal( false );
+							}
+							
 							checker.enqueueCheckRequest(
 									request,
 									new DiskManagerCheckRequestListener()
@@ -646,6 +706,11 @@ RDResumeHandler
 											DiskManagerCheckRequest 	request,
 											Throwable		 			cause )
 										{
+											if ( forceRecheck ){
+											
+												forceRecheckErrorReporter.accept( cause );
+											}
+											
 											complete();
 										}
 
@@ -656,6 +721,13 @@ RDResumeHandler
 											
 											pending_checks_sem.release();
 										}
+										
+										@Override
+										public boolean 
+										isFailureInteresting()
+										{
+											return( false );
+										}
 									});
 
 							pending_check_num++;
@@ -783,6 +855,13 @@ RDResumeHandler
 																			{
 																				sem.release();
 																			}
+																			
+																			@Override
+																			public boolean 
+																			isFailureInteresting()
+																			{
+																				return( false );
+																			}
 																		});
 															}catch( Throwable e ){
 
@@ -1260,7 +1339,23 @@ RDResumeHandler
 		return pivot;
 	}
 
-	public static boolean fileMustExist(DownloadManager download_manager, DiskManagerFileInfo file) {
+	/**
+	 * @deprecated Kept for xmwebui 
+	 */
+	public static boolean
+	fileMustExist(
+		DownloadManager 			download_manager,
+		DiskManagerFileInfo 		file)
+	{
+		return fileMustExist(download_manager, download_manager.getDiskManagerFileInfoSet(), file);
+	}
+
+	public static boolean 
+	fileMustExist(
+		DownloadManager 			download_manager,
+		DiskManagerFileInfoSet		fileSet,
+		DiskManagerFileInfo 		file) 
+	{
 
 		Map resumeData = getResumeData( download_manager );
 
@@ -1268,7 +1363,7 @@ RDResumeHandler
 
 		boolean sharesAnyNeededPieces = false;
 
-		DiskManagerFileInfo[] files = download_manager.getDiskManagerFileInfo();
+		DiskManagerFileInfo[] files = fileSet.getFiles();
 		int firstPiece = file.getFirstPieceNumber();
 		int lastPiece = file.getLastPieceNumber();
 
