@@ -101,6 +101,13 @@ DiskManagerFileInfoStream
 		throw( new RuntimeException( "Not supported" ));
 	}
 
+	@Override
+	public Boolean 
+	isSkipping()
+	{
+		return( null );
+	}
+	
 	@Override
 	public int
 	getNumericPriority()
