@@ -262,6 +262,13 @@ DiskManagerFileInfoURL
 		throw( new RuntimeException( "Not supported" ));
 	}
 
+	@Override
+	public Boolean 
+	isSkipping()
+	{
+		return( null );
+	}
+	
 	@Override
 	public int
 	getNumericPriority()
