@@ -23,6 +23,7 @@ package com.biglybt.core.download;
 import java.io.File;
 import java.util.List;
 
+import com.biglybt.core.CoreOperationTask.ProgressCallback;
 import com.biglybt.core.disk.DiskManager;
 import com.biglybt.core.disk.DiskManagerFileInfo;
 import com.biglybt.core.disk.DiskManagerFileInfoSet;
@@ -40,6 +41,7 @@ import com.biglybt.core.tracker.client.TRTrackerScraperResponse;
 import com.biglybt.core.util.FileUtil;
 import com.biglybt.core.util.IndentWriter;
 import com.biglybt.pif.download.Download;
+import com.biglybt.pif.download.Download.SeedingRank;
 import com.biglybt.pif.download.DownloadAnnounceResult;
 import com.biglybt.pif.download.DownloadScrapeResult;
 
@@ -95,6 +97,9 @@ DownloadManager
 	
 	public static final Object UD_KEY_STOP_REASON = Download.UD_KEY_STOP_REASON;
 	
+	public boolean
+	isConstructed();
+	
     public void
     initialize();
 
@@ -118,16 +123,24 @@ DownloadManager
     public void
     startDownload();
 
-    public boolean canForceRecheck();
+    public boolean 
+    canForceRecheck();
 
-    public void forceRecheck();
+    public boolean
+    isForceRechecking();
+    
+    public void 
+    forceRecheck();
 
 		/**
 		 * @param l
 		 *
 		 * @since 3.0.0.7
 		 */
-		void forceRecheck(ForceRecheckListener l);
+    
+	public void 
+	forceRecheck(
+		ForceRecheckListener l);
 
         /**
          * Reset the file download state to totally undownloaded. Download must be stopped
@@ -147,6 +160,16 @@ DownloadManager
     recheckFile(
         DiskManagerFileInfo     file );
 
+    	/**
+    	 * Instructions that file given files need allocation - used when switching file storage types/skipped state  
+    	 * Will pause and resume the download if it is active
+    	 * @param files
+    	 */
+    
+    public void
+    requestAllocation(
+    	List<DiskManagerFileInfo>		files );
+    
     	/**
     	 * Use with care - introduced to support speed tests whereby we want to avoid checking the
     	 * virtual torrent used for the test
@@ -354,6 +377,10 @@ DownloadManager
     public boolean
     isDownloadComplete(boolean bIncludingDND);
 
+    public void
+    checkLightSeeding(
+    	boolean	full_sync );
+    
     public String
     getTrackerStatus();
 
@@ -508,9 +535,9 @@ DownloadManager
     public void setDataAlreadyAllocated( boolean already_allocated );
 
 
-    public void setSeedingRank(int rank);
+    public void setSeedingRank(SeedingRank rank);
 
-    public int getSeedingRank();
+    public SeedingRank getSeedingRank();
 
     public boolean isSwarmMerging();
     
@@ -584,7 +611,8 @@ DownloadManager
 
     public void
     copyDataFiles(
-    	File	parent_dir )
+    	File				parent_dir,
+    	ProgressCallback	progress )
 
     	throws DownloadManagerException;
 
@@ -859,9 +887,17 @@ DownloadManager
     public void
     exportDownload( File parent_dir ) throws DownloadManagerException;
     
+    public default void
+    fireGlobalManagerEvent(
+    	int		event_type )
+    {
+    	fireGlobalManagerEvent( event_type, null );
+    }
+    
     public void
     fireGlobalManagerEvent(
-    	int		event_type );
+    	int		event_type,
+    	Object	event_data );
 
 		/**
 		 * Sets the priority for an array for files
