@@ -32,6 +32,7 @@ import com.biglybt.core.disk.*;
 import com.biglybt.core.disk.impl.DiskManagerUtil;
 import com.biglybt.core.download.*;
 import com.biglybt.core.global.GlobalManager;
+import com.biglybt.core.global.GlobalManagerEvent;
 import com.biglybt.core.global.GlobalManagerStats;
 import com.biglybt.core.internat.MessageText;
 import com.biglybt.core.logging.LogEvent;
@@ -172,15 +173,17 @@ DownloadManagerController
 
 	private volatile int		state_set_by_method = DownloadManager.STATE_START_OF_DAY;
 	private volatile int		substate;
-	volatile boolean 	force_start;
-
+	private volatile boolean 	force_start;
+	private volatile boolean	is_force_rechecking;
+	
 		// to try and ensure people don't start using disk_manager without properly considering its
 		// access implications we've given it a silly name
 
 	private volatile DiskManager 			disk_manager_use_accessors;
 	private DiskManagerListener				disk_manager_listener_use_accessors;
 
-	private DiskManagerPiece[]				disk_manager_pieces_snapshot;
+	private Object							disk_manager_pieces_snapshot_lock = new Object();
+	private volatile DiskManagerPiece[]		disk_manager_pieces_snapshot;
 	
 	final FileInfoFacadeSet		fileFacadeSet = new FileInfoFacadeSet();
 	boolean					files_facade_destroyed;
@@ -197,6 +200,8 @@ DownloadManagerController
 
 	private DownloadManagerStateAttributeListener	dm_attribute_listener;
 	
+	private Object			external_rate_limiters_cow_lock = new Object();
+	
 	private List<Object[]>	external_rate_limiters_cow;
 
 	private String 	errorDetail;
@@ -212,7 +217,7 @@ DownloadManagerController
 	private static final int			ACTIVATION_REBUILD_TIME		= 10*60*1000;
 	private static final int			BLOOM_SIZE					= 64;
 	private volatile BloomFilter		activation_bloom;
-	private volatile long				activation_bloom_create_time	= SystemTime.getCurrentTime();
+	private volatile long				activation_bloom_create_time	= SystemTime.getMonotonousTime();
 	private volatile int				activation_count;
 	private volatile long				activation_count_time;
 
@@ -316,6 +321,14 @@ DownloadManagerController
 
 		try{
 			control_mon.enter();
+			
+			if (download_manager.isDestroyed()) {
+				if (Logger.isEnabled()) {
+					Logger.log(new LogEvent(this, LogIDs.CORE, LogEvent.LT_ERROR,
+							"startDownload() after manager is destroyed"));
+				}
+				return;
+			}
 
 			if ( getState() != DownloadManager.STATE_READY ){
 
@@ -699,7 +712,10 @@ DownloadManagerController
 
 			SimpleTimer.addTickReceiver( this );
 
-			limiters = external_rate_limiters_cow;
+			synchronized( external_rate_limiters_cow_lock ){
+			
+				limiters = external_rate_limiters_cow;
+			}
 
 		}finally{
 
@@ -808,8 +824,15 @@ DownloadManagerController
 	  	           	(state == DownloadManager.STATE_ERROR && getDiskManager() == null));
 	}
 
+	protected boolean
+	isForceRechecking()
+	{
+		return( is_force_rechecking );
+	}
+	
 	public void
-	forceRecheck(final ForceRecheckListener l)
+	forceRecheck(
+		final ForceRecheckListener l)
 	{
 		try{
 			control_mon.enter();
@@ -821,7 +844,16 @@ DownloadManagerController
 				return;
 			}
 
-			final int start_state = DownloadManagerController.this.getState();
+			if ( is_force_rechecking ){
+				
+				Debug.out( "DownloadManagerController::forceRecheck: already active" );
+				
+				return;
+			}
+			
+			is_force_rechecking = true;
+			
+			int start_state = DownloadManagerController.this.getState();
 
 				// remove resume data
 
@@ -830,7 +862,7 @@ DownloadManagerController
 	  			// For extra protection from a plugin stopping a checking torrent,
 	  			// fake a forced start.
 
-	  		final boolean wasForceStarted = force_start;
+	  		boolean wasForceStarted = force_start;
 
 	  		force_start = true;
 
@@ -842,7 +874,25 @@ DownloadManagerController
 
 	  		initializeDiskManagerSupport(
 	  			DownloadManager.STATE_CHECKING,
-	  			new forceRecheckDiskManagerListener(wasForceStarted, start_state, l));
+	  			new ForceRecheckDiskManagerListener(
+	  				wasForceStarted, 
+	  				start_state, 
+	  				new ForceRecheckListener()
+	  				{
+	  					public void 
+	  					forceRecheckComplete(
+	  						DownloadManager dm)
+	  					{
+	  						is_force_rechecking = false;
+	  						
+	  						download_manager.fireGlobalManagerEvent( GlobalManagerEvent.ET_RECHECK_COMPLETE,  true );
+	  						
+	  						if ( l != null ){
+	  							
+	  							l.forceRecheckComplete(dm);
+	  						}
+	  					}
+	  				}));
 
 		}finally{
 
@@ -923,6 +973,8 @@ DownloadManagerController
 					download_manager.deleteTorrentFile();
 				}
 
+				download_manager.informStopped( null, stateAfterStopping==DownloadManager.STATE_QUEUED );
+
 				setState( _stateAfterStopping, false );
 
 				return;
@@ -1121,17 +1173,29 @@ DownloadManagerController
 
 
   	@Override
-	  public void
+	public void
   	setStateSeeding(
   		boolean	never_downloaded )
 	{
 		// should already be finishing, but make sure (if it already is, there
 		// won't be a trigger)
+  		
 		setStateFinishing();
 
 		download_manager.downloadEnded(never_downloaded);
 
-		setState(DownloadManager.STATE_SEEDING, true);
+		setState( DownloadManager.STATE_SEEDING, true);
+		
+		if ( !never_downloaded ){
+			
+	    	if ( !COConfigurationManager.getBooleanParameter( "StartStopManager_bRetainForceStartWhenComplete" )){
+	
+	    		if (isForceStart()){
+	
+		    		setForceStart(false);
+	    		}
+	    	}
+		}
 	}
 
   	public boolean
@@ -1252,8 +1316,12 @@ DownloadManagerController
 
 	  				activation_bloom = null;
 
-	  				if ( state_set_by_method == DownloadManager.STATE_STOPPED ){
+	  				if ( 	state_set_by_method == DownloadManager.STATE_STOPPED ||
+	  						state_set_by_method == DownloadManager.STATE_DOWNLOADING ||
+	  						state_set_by_method == DownloadManager.STATE_SEEDING ){
 
+	  						// don't need this anymore once stopped or active
+	  					
 	  					activation_count = 0;
 	  				}
 	  			}
@@ -1508,9 +1576,23 @@ DownloadManagerController
 		return( null );
 	}
 	
+	@Override
+	public byte[] 
+	getPeerID()
+	{
+		return( download_manager.getTrackerClient().getPeerId());
+	}
+	
 	@Override
 	public int 
-	getLocalPort(
+	getNbPieces()
+	{
+		return( download_manager.getTorrent().getNumberOfPieces());
+	}
+	
+	@Override
+	public int 
+	getHashOverrideLocalPort(
 		boolean	only_if_allocated )
 	{
 		return( download_manager.getTCPPortOverride( only_if_allocated ));
@@ -1620,30 +1702,41 @@ DownloadManagerController
 		LimitedRateGroup	group,
 		boolean				upload )
 	{
+		final boolean not_stopping = getState() != DownloadManager.STATE_STOPPING;	// locking issue here on stop :(
+		
 		PEPeerManager	pm;
 
 		try{
-			control_mon.enter();
-
-			ArrayList<Object[]>	new_limiters = new ArrayList<>(external_rate_limiters_cow == null ? 1 : external_rate_limiters_cow.size() + 1);
-
-			if ( external_rate_limiters_cow != null ){
-
-				new_limiters.addAll( external_rate_limiters_cow );
+			if ( not_stopping ){
+				
+				control_mon.enter();
 			}
 
-			new_limiters.add( new Object[]{ group, Boolean.valueOf(upload)});
-
-			external_rate_limiters_cow = new_limiters;
-
+			synchronized( external_rate_limiters_cow_lock ){
+				
+				ArrayList<Object[]>	new_limiters = new ArrayList<>(external_rate_limiters_cow == null ? 1 : external_rate_limiters_cow.size() + 1);
+	
+				if ( external_rate_limiters_cow != null ){
+	
+					new_limiters.addAll( external_rate_limiters_cow );
+				}
+	
+				new_limiters.add( new Object[]{ group, Boolean.valueOf(upload)});
+	
+				external_rate_limiters_cow = new_limiters;
+			}
+			
 			pm	= peer_manager;
 
 		}finally{
 
-			control_mon.exit();
+			if ( not_stopping ){
+				
+				control_mon.exit();
+			}
 		}
 
-		if ( pm != null ){
+		if ( not_stopping && pm != null ){
 
 			pm.addRateLimiter(group, upload);
 		}
@@ -1653,8 +1746,7 @@ DownloadManagerController
 	getRateLimiters(
 		boolean	upload )
 	{
-		try{
-			control_mon.enter();
+		synchronized( external_rate_limiters_cow_lock ){
 
 			if ( external_rate_limiters_cow == null ){
 
@@ -1674,9 +1766,6 @@ DownloadManagerController
 
 				return( result.toArray( new LimitedRateGroup[ result.size() ]));
 			}
-		}finally{
-
-			control_mon.exit();
 		}
 	}
 
@@ -1685,11 +1774,16 @@ DownloadManagerController
 		LimitedRateGroup	group,
 		boolean				upload )
 	{
+		final boolean not_stopping = getState() != DownloadManager.STATE_STOPPING;	// locking issue here on stop :(
+
 		PEPeerManager	pm;
 
 		try{
-			control_mon.enter();
-
+			if ( not_stopping ){
+			
+				control_mon.enter();
+			}
+			
 			if ( external_rate_limiters_cow != null ){
 
 				ArrayList<Object[]>	new_limiters = new ArrayList<>(external_rate_limiters_cow.size() - 1);
@@ -1718,10 +1812,13 @@ DownloadManagerController
 
 		}finally{
 
-			control_mon.exit();
+			if ( not_stopping ){
+			
+				control_mon.exit();
+			}
 		}
 
-		if ( pm != null ){
+		if ( not_stopping && pm != null ){
 
 			pm.removeRateLimiter(group, upload);
 		}
@@ -1738,17 +1835,26 @@ DownloadManagerController
 	}
 
 	@Override
-	public boolean
+	public int
 	activateRequest(
 		InetSocketAddress	address )
 	{
 		if ( getState() == DownloadManager.STATE_QUEUED ){
 
+			if ( download_manager.isLightSeedTracker( address )){
+				
+				return( AT_ACCEPTED_PROBE );
+			}
+			
+			long	now = SystemTime.getMonotonousTime();
+
 			BloomFilter	bloom = activation_bloom;
 
 			if ( bloom == null ){
 
 				activation_bloom = bloom = BloomFilterFactory.createAddRemove4Bit( BLOOM_SIZE );
+				
+				activation_bloom_create_time = now;
 			}
 
 			byte[]	address_bytes = AddressUtils.getAddressBytes(address);
@@ -1764,17 +1870,16 @@ DownloadManagerController
 							LogEvent.LT_WARNING,
 							"Activate request for " + getDisplayName() + " from " + address + " denied as too many recently received" ));
 
-				return( false );
+				return( AT_DECLINED );
 			}
 
 			Logger.log(new LogEvent(this, LogIDs.CORE, "Activate request for " + getDisplayName() + " from " + address ));
 
-			long	now = SystemTime.getCurrentTime();
 
 				// we don't really care about the bloom filter filling up and giving false positives
 				// as activation events should be fairly rare
 
-			if ( now < activation_bloom_create_time || now - activation_bloom_create_time > ACTIVATION_REBUILD_TIME ){
+			if ( now - activation_bloom_create_time > ACTIVATION_REBUILD_TIME ){
 
 				activation_bloom = BloomFilterFactory.createAddRemove4Bit( BLOOM_SIZE );
 
@@ -1785,10 +1890,13 @@ DownloadManagerController
 
 			activation_count_time = now;
 
-			return( download_manager.activateRequest( activation_count ));
+			if ( download_manager.activateRequest( activation_count )){
+				
+				return( AT_ACCEPTED );
+			}
 		}
 
-		return( false );
+		return( AT_DECLINED );
 	}
 
 	@Override
@@ -1852,13 +1960,11 @@ DownloadManagerController
 			// in the absence of any new activations we persist the last count for the activation rebuild
 			// period
 
-		long	now = SystemTime.getCurrentTime();
+		long	now = SystemTime.getMonotonousTime();
 
-		if ( now < activation_count_time ){
-
-			activation_count_time = now;
-
-		}else if ( now - activation_count_time > ACTIVATION_REBUILD_TIME ){
+		if ( 	activation_count > 0 && 
+				activation_count_time != 0 &&
+				now - activation_count_time > ACTIVATION_REBUILD_TIME ){
 
 			activation_count = 0;
 		}
@@ -1938,6 +2044,8 @@ DownloadManagerController
 			
 		}else{
 			
+			Debug.out( cause );
+			
 			setFailed( DownloadManager.ET_OTHER, reason + ": " + Debug.getNestedExceptionMessage( cause ));
 		}
 	}
@@ -2129,6 +2237,8 @@ DownloadManagerController
 		// if it's more than one file just do the scan anyway
 		fileFacadeSet.makeSureFilesFacadeFilled(false);
 		calculateCompleteness( fileFacadeSet.facadeFiles );
+		
+		disk_manager_pieces_snapshot = null;
 	}
 
 	protected void
@@ -2227,8 +2337,7 @@ DownloadManagerController
 	public DiskManagerPiece[] 
 	getDiskManagerPiecesSnapshot()
 	{
-		try{
-			control_mon.enter();
+		synchronized( disk_manager_pieces_snapshot_lock ){
 		
 			if ( disk_manager_pieces_snapshot == null ){
 				
@@ -2236,10 +2345,6 @@ DownloadManagerController
 			}
 			
 			return( disk_manager_pieces_snapshot );
-			
-		}finally{
-			
-			control_mon.exit();
 		}
 	}
 
@@ -2894,6 +2999,11 @@ DownloadManagerController
 		DiskManagerFileInfoSet delegate;
 		fileInfoFacade[] facadeFiles = new fileInfoFacade[0];	// default before torrent avail
 
+		@Override
+		public void load(int[] priorities, boolean[] skipped){
+			delegate.load(priorities, skipped);
+		}
+		
 		@Override
 		public DiskManagerFileInfo[] getFiles() {
 			return facadeFiles;
@@ -3102,6 +3212,12 @@ DownloadManagerController
 			delegate.setSkipped(b);
 		}
 
+		@Override
+		public Boolean 
+		isSkipping()
+		{
+			return( delegate.isSkipping());
+		}
 
 		@Override
 		public boolean
@@ -3415,7 +3531,7 @@ DownloadManagerController
 		}
 	}
 
-	public class forceRecheckDiskManagerListener
+	public class ForceRecheckDiskManagerListener
 		implements DiskManagerListener
 	{
 		private final boolean wasForceStarted;
@@ -3424,7 +3540,7 @@ DownloadManagerController
 
 		private final ForceRecheckListener l;
 
-		public forceRecheckDiskManagerListener(boolean wasForceStarted,
+		public ForceRecheckDiskManagerListener(boolean wasForceStarted,
 				int start_state, ForceRecheckListener l) {
 			this.wasForceStarted = wasForceStarted;
 			this.start_state = start_state;
@@ -3442,7 +3558,8 @@ DownloadManagerController
 
 					download_manager.setAssumedComplete(false);
 
-					if (l != null) {
+					if ( l != null ){
+						
 						l.forceRecheckComplete(download_manager);
 					}
 
@@ -3535,9 +3652,10 @@ DownloadManagerController
 					}
 
 					download_manager.setAssumedComplete(false);
-
 				}
-				if (l != null) {
+				
+				if ( l != null ){
+					
 					l.forceRecheckComplete(download_manager);
 				}
 			}
