@@ -45,7 +45,6 @@ import com.biglybt.core.peer.PEPeerSource;
 import com.biglybt.core.torrent.*;
 import com.biglybt.core.tracker.client.TRTrackerAnnouncer;
 import com.biglybt.core.util.*;
-import com.biglybt.pif.logging.LoggerChannel;
 
 /**
  * @author parg
@@ -1597,6 +1596,16 @@ DownloadManagerStateImpl
 		String		name,
 		String		value )
 	{
+		setAttribute( name,value, true );
+	}
+	
+	@Override
+	public void
+	setAttribute(
+		String		name,
+		String		value,
+		boolean		set_dirty )
+	{
 
 		if ( name.equals( AT_CATEGORY )){
 
@@ -1626,7 +1635,7 @@ DownloadManagerStateImpl
 			}
 		}
 
-		setStringAttribute( name, value );
+		setStringAttribute( name, value, set_dirty );
 	}
 
 	@Override
@@ -1706,11 +1715,11 @@ DownloadManagerStateImpl
 
 		if ( category != null ){
 
-			setStringAttribute( AT_CATEGORY, category.getName());
+			setStringAttribute( AT_CATEGORY, category.getName(), true );
 
 		}else{
 
-			setStringAttribute( AT_CATEGORY, null );
+			setStringAttribute( AT_CATEGORY, null, true );
 		}
 	}
 
@@ -1726,7 +1735,7 @@ DownloadManagerStateImpl
 	setTrackerClientExtensions(
 		String		value )
 	{
-		setStringAttribute( AT_TRACKER_CLIENT_EXTENSIONS, value );
+		setStringAttribute( AT_TRACKER_CLIENT_EXTENSIONS, value, true );
 	}
 
     @Override
@@ -1736,7 +1745,7 @@ DownloadManagerStateImpl
 
     @Override
     public void setDisplayName(String value) {
-    	this.setStringAttribute(AT_DISPLAY_NAME, value);
+    	this.setStringAttribute(AT_DISPLAY_NAME, value, true);
     }
 
     @Override
@@ -1746,7 +1755,7 @@ DownloadManagerStateImpl
 
     @Override
     public void setUserComment(String value) {
-    	this.setStringAttribute(AT_USER_COMMENT, value);
+    	this.setStringAttribute(AT_USER_COMMENT, value, true);
     }
 
     @Override
@@ -2445,7 +2454,8 @@ DownloadManagerStateImpl
 	protected void
 	setStringAttribute(
 		final String	attribute_name,
-		final String	attribute_value )
+		final String	attribute_value,
+		boolean			set_dirty )
 	{
 		boolean	changed	= false;
 
@@ -2460,7 +2470,10 @@ DownloadManagerStateImpl
 
 					changed = true;
 					
-					setDirty( attribute_name == DownloadManagerState.AT_AGGREGATE_SCRAPE_CACHE );
+					if ( set_dirty ){
+					
+						setDirty( attribute_name == DownloadManagerState.AT_AGGREGATE_SCRAPE_CACHE );
+					}
 				}
 			}else{
 
@@ -2470,7 +2483,9 @@ DownloadManagerStateImpl
 				if (existing_bytes == null || !Arrays.equals(existing_bytes, new_bytes)) {
 					attributes.put(attribute_name, new_bytes);
 					changed = true;
-					setDirty( attribute_name == DownloadManagerState.AT_AGGREGATE_SCRAPE_CACHE );
+					if ( set_dirty ){
+						setDirty( attribute_name == DownloadManagerState.AT_AGGREGATE_SCRAPE_CACHE );
+					}
 				}
 			}
 		}finally{
@@ -3274,6 +3289,15 @@ DownloadManagerStateImpl
 		{
 		}
 
+		@Override
+		public void
+		setAttribute(
+			String		name,
+			String		value,
+			boolean		setDirty )
+		{
+		}
+		
 		@Override
 		public String
 		getAttribute(
@@ -4705,6 +4729,31 @@ DownloadManagerStateImpl
 			}
     	}
 
+    	@Override
+	    public String
+    	getSource()
+       	{
+	   		if ( fixup()){
+	
+				return( delegate.getSource());
+			}
+	   		
+	   		return( null );
+       	}
+
+    	@Override
+	    public void
+    	setSource(
+    		String	str )
+    	
+    		throws TOTorrentException
+	   	{
+  	   		if ( fixup()){
+
+				delegate.setSource( str );
+			}
+    	}
+    	
     	@Override
 	    public void
     	setAdditionalStringProperty(
