@@ -977,7 +977,7 @@ DownloadManagerStatsImpl
 			// manipulate by updating downloaded to be one full copy and then uploaded as required
 
 		long	target_downloaded 	= total_size;
-		long	target_uploaded 	= ( ratio * total_size ) / 1000;
+		long	target_uploaded 	= ( ratio * total_size + 999) / 1000;
 
 		saved_data_bytes_downloaded	= target_downloaded - downloaded;
 		saved_data_bytes_uploaded	= target_uploaded - uploaded;
@@ -1282,7 +1282,7 @@ DownloadManagerStatsImpl
 					
 					if ( consolidated == null ){
 						
-						consolidated = vals;
+						consolidated = vals.clone();
 						
 					}else{
 						
@@ -1591,9 +1591,14 @@ DownloadManagerStatsImpl
   {
 	  DiskManager	dm = download_manager.getDiskManager();
 
-	  if (dm != null) {
-	  	return dm.getRemainingExcludingDND();
-	  }
+		if (dm != null) {
+			int state = dm.getState();
+			// INITIALIZING and ALLOCATING states returns sizes that include DND.
+			if (state != DiskManager.INITIALIZING
+					&& state != DiskManager.ALLOCATING) {
+				return dm.getRemainingExcludingDND();
+			}
+		}
 
 	  long remaining = getRemaining();
 	  long rem = ( remaining - ( getSkippedFileSetSize() - saved_skipped_but_downloaded ));
@@ -1612,9 +1617,14 @@ DownloadManagerStatsImpl
 	{
 	  DiskManager	dm = download_manager.getDiskManager();
 
-	  if (dm != null) {
-	  	return dm.getSizeExcludingDND();
-	  }
+		if (dm != null) {
+			int state = dm.getState();
+			// INITIALIZING and ALLOCATING states returns sizes that include DND.
+			if (state != DiskManager.INITIALIZING
+					&& state != DiskManager.ALLOCATING) {
+				return dm.getSizeExcludingDND();
+			}
+		}
 
   	long totalLength = download_manager.getSize();
   	return totalLength - getSkippedFileSetSize();
