@@ -45,10 +45,12 @@ import com.biglybt.core.logging.LogEvent;
 import com.biglybt.core.logging.LogIDs;
 import com.biglybt.core.logging.Logger;
 import com.biglybt.core.tag.Tag;
+import com.biglybt.core.tag.TagFeatureFileLocation;
 import com.biglybt.core.tag.TagManager;
 import com.biglybt.core.tag.TagManagerFactory;
 import com.biglybt.core.tag.TagType;
 import com.biglybt.core.torrent.TOTorrent;
+import com.biglybt.core.torrent.TOTorrentFile;
 import com.biglybt.core.torrent.impl.TorrentOpenOptions;
 import com.biglybt.core.util.*;
 import com.biglybt.core.util.protocol.magnet.MagnetConnection2;
@@ -249,7 +251,7 @@ public class TorrentFolderWatcher {
 				File folder = null;
 
 				if (folder_path != null && folder_path.length() > 0) {
-					folder = new File(folder_path);
+					folder = FileUtil.newFile(folder_path);
 					if (!folder.isDirectory()) {
 						if (!folder.exists()) {
 							FileUtil.mkdirs(folder);
@@ -287,11 +289,11 @@ public class TorrentFolderWatcher {
 				return;
 			}
 
-			String data_save_path = COConfigurationManager.getStringParameter("Default save path");
+			String default_data_save_path = COConfigurationManager.getStringParameter("Default save path");
 
 			File f = null;
-			if (data_save_path != null && data_save_path.length() > 0) {
-				f = new File(data_save_path);
+			if (default_data_save_path != null && default_data_save_path.length() > 0) {
+				f = FileUtil.newFile(default_data_save_path);
 
 				// Path is not an existing directory.
 				if (!f.isDirectory()) {
@@ -350,8 +352,8 @@ public class TorrentFolderWatcher {
 				boolean	save_torrents = save_torrents_default;
 
 				if (torrent_save_path.length() == 0
-						|| new File( torrent_save_path ).getAbsolutePath().equals(folder.getAbsolutePath())
-						|| !new File(torrent_save_path).isDirectory()) {
+						|| FileUtil.newFile( torrent_save_path ).getAbsolutePath().equals(folder.getAbsolutePath())
+						|| !FileUtil.newFile(torrent_save_path).isDirectory()) {
 
 					save_torrents = false;
 				}
@@ -371,7 +373,7 @@ public class TorrentFolderWatcher {
 							return;
 						}
 
-						File file = new File(folder, currentFileList[i]).getAbsoluteFile();
+						File file = FileUtil.newFile(folder, currentFileList[i]).getAbsoluteFile();
 						
 						if ( file.getName().toLowerCase( Locale.US ).endsWith( ".magnet" )) {
 							
@@ -398,9 +400,9 @@ public class TorrentFolderWatcher {
 
 									if ( always_rename ){
 									
-										if ( !file.equals( new File( dm.getTorrentFileName()).getAbsoluteFile())){
+										if ( !file.equals( FileUtil.newFile( dm.getTorrentFileName()).getAbsoluteFile())){
 											
-											File imported = new File(folder, file.getName() + ".imported");
+											File imported = FileUtil.newFile(folder, file.getName() + ".imported");
 											
 											TorrentUtils.move(file, imported);
 										}
@@ -418,7 +420,7 @@ public class TorrentFolderWatcher {
 	
 									if ( always_rename || !save_torrents ){
 	
-										File imported = new File(folder, file.getName() + ".imported");
+										File imported = FileUtil.newFile(folder, file.getName() + ".imported");
 	
 										TorrentUtils.move(file, imported);
 	
@@ -428,7 +430,9 @@ public class TorrentFolderWatcher {
 									}
 	
 								} else {
-	
+
+									boolean[] to_skip = TorrentUtils.getSkipFiles( torrent );
+
 									final DownloadManagerInitialisationAdapter dmia = new DownloadManagerInitialisationAdapter() {
 	
 										@Override
@@ -452,8 +456,6 @@ public class TorrentFolderWatcher {
 											
 											boolean reorder_mode = COConfigurationManager.getBooleanParameter("Enable reorder storage mode");
 											int reorder_mode_min_mb = COConfigurationManager.getIntParameter("Reorder storage mode min MB");
-
-											boolean[] to_skip = TorrentUtils.getSkipFiles( torrent );
 											
 											if ( to_skip != null ){
 												
@@ -528,23 +530,93 @@ public class TorrentFolderWatcher {
 										hash = torrent.getHash();
 									} catch (Exception e) { }
 	
+									String data_save_path = default_data_save_path;
+									
+									boolean for_seeding = false;
+									
+									if ( tag_name != null ){
+										
+											// if we have a tag with a move-on-complete destination and the files are already
+											// there then fix things to it gets added-for-seeding
+										
+										TagManager tm = TagManagerFactory.getTagManager();
+
+										TagType tt = tm.getTagType( TagType.TT_DOWNLOAD_MANUAL );
+
+										Tag	tag = tt.getTag( tag_name, true );
+										
+										if ( tag instanceof TagFeatureFileLocation ){
+											
+											TagFeatureFileLocation tag_save_location = (TagFeatureFileLocation)tag;
+										
+											if ( tag_save_location.supportsTagMoveOnComplete()){
+												
+												File move_loc = tag_save_location.getTagMoveOnCompleteFolder();
+												
+												if ( move_loc != null && move_loc.exists()){
+												
+													File root;
+													
+													if ( torrent.isSimpleTorrent()){
+													
+														root = move_loc;
+														
+													}else{
+														
+														root = FileUtil.newFile( move_loc, FileUtil.convertOSSpecificChars( TorrentUtils.getLocalisedName(torrent), true ));
+													}
+													
+													if (( tag_save_location.getTagMoveOnCompleteOptions() & TagFeatureFileLocation.FL_DATA ) != 0 ){
+																																										
+														TOTorrentFile[] files = torrent.getFiles();
+														
+														boolean all_exist = true;
+														
+														for ( int j=0;j<files.length;j++){
+															
+															if ( to_skip != null && to_skip[j] ){
+																
+																continue;
+															}
+															
+															File file_loc = FileUtil.newFile( root, files[j].getRelativePath());
+															
+															if ( !file_loc.exists()){
+																
+																all_exist = false;
+																
+																break;
+															}
+														}
+														
+														if ( all_exist ){
+															
+															data_save_path = move_loc.getAbsolutePath();
+															
+															for_seeding = true;
+														}
+													}
+												}
+											}
+										}
+									}
 									
 									int start_state = TorrentOpenOptions.addModePreCreate(start_mode);
 									
 									if ( always_rename || !save_torrents) {
 	
-										File imported = new File(folder, file.getName() + ".imported");
+										File imported = FileUtil.newFile(folder, file.getName() + ".imported");
 	
 										TorrentUtils.move(file, imported);
 	
 										dm = global_manager.addDownloadManager(imported.getAbsolutePath(), hash,
-												data_save_path, start_state, true,false,dmia);
+												data_save_path, start_state, true, for_seeding, dmia);
 										
 	
 									} else {
 	
 										dm = global_manager.addDownloadManager(file.getAbsolutePath(), hash,
-												data_save_path, start_state, true, false, dmia);
+												data_save_path, start_state, true, for_seeding, dmia);
 	
 										// add torrent for deletion, since there will be a
 										// saved copy elsewhere
@@ -597,8 +669,10 @@ public class TorrentFolderWatcher {
 					tag = tt.createTag( tag_name, true );
 				}
 
-				tag.addTaggable( dm );
-
+				if ( !tag.hasTaggable( dm )){
+				
+					tag.addTaggable( dm );
+				}
 			}catch( Throwable e ){
 
 				Debug.out( e );
@@ -829,7 +903,7 @@ public class TorrentFolderWatcher {
 						
 						bad_magnet = false;
 						
-						File output_file = new File( active.getAbsolutePath() + ".torrent" );
+						File output_file = FileUtil.newFile( active.getAbsolutePath() + ".torrent" );
 						
 						if ( output_file.exists()){
 							
@@ -877,7 +951,7 @@ public class TorrentFolderWatcher {
 								
 								if ( bad_magnet ){
 								
-									active.renameTo( new File( active.getAbsolutePath() + ".failed" ));
+									active.renameTo( FileUtil.newFile( active.getAbsolutePath() + ".failed" ));
 								}
 								
 								failed_magnets.add( active );
@@ -907,4 +981,4 @@ public class TorrentFolderWatcher {
 			}
 		}.start();
 	}
-}
\ No newline at end of file
+}
