@@ -38,6 +38,7 @@ import com.biglybt.core.dht.DHT;
 import com.biglybt.core.dht.DHTListener;
 import com.biglybt.core.dht.speed.DHTSpeedTester;
 import com.biglybt.core.disk.DiskManager;
+import com.biglybt.core.disk.DiskManagerFactory;
 import com.biglybt.core.download.DownloadManager;
 import com.biglybt.core.download.DownloadManagerState;
 import com.biglybt.core.global.GlobalManager;
@@ -70,6 +71,7 @@ import com.biglybt.core.peer.PEPeerManager;
 import com.biglybt.core.peer.PEPeerSource;
 import com.biglybt.core.peermanager.PeerManager;
 import com.biglybt.core.peermanager.nat.PeerNATTraverser;
+import com.biglybt.core.proxy.AEProxyFactory;
 import com.biglybt.core.proxy.AEProxySelectorFactory;
 import com.biglybt.core.security.BGSpongy;
 import com.biglybt.core.security.CryptoManager;
@@ -337,8 +339,9 @@ CoreImpl
 			if (DEBUG_STARTUPTIME) {
 				logTime("Init1");
 			}
-	
-	
+		
+			DiskManagerFactory.initialise( this );
+			
 			NetworkManager.getSingleton();
 	
 			if (DEBUG_STARTUPTIME) {
@@ -359,6 +362,8 @@ CoreImpl
 				logTime("Init PluginInitializer");
 			}
 	
+			AEProxyFactory.initialise( this );
+			
 			BGSpongy.initialize( this );
 
 			instance_manager =
