@@ -23,6 +23,7 @@ package com.biglybt.core.lws;
 import java.io.File;
 
 import com.biglybt.core.disk.*;
+import com.biglybt.core.disk.DiskManager.DownloadEndedProgress;
 import com.biglybt.core.disk.impl.*;
 import com.biglybt.core.disk.impl.access.DMAccessFactory;
 import com.biglybt.core.disk.impl.access.DMChecker;
@@ -136,6 +137,13 @@ LWSDiskManager
 		return( reader.getStats());
 	}
 
+	@Override
+	public long[]
+	getWriteStats()
+	{
+		return( new long[]{ 0, 0, 0, 0 } );
+	}
+	
 	@Override
 	public void
 	start()
@@ -572,17 +580,22 @@ LWSDiskManager
 	}
 
 	@Override
-	public void
-	downloadEnded( OperationStatus op_status )
+	public DownloadEndedProgress
+	downloadEnded()
 	{
+		return( new DownloadEndedProgress(){
+			@Override
+			public boolean isComplete(){
+				return( true );
+			}
+		});
 	}
 
 	@Override
 	public void
 	moveDataFiles(
 		File 				new_parent_dir,
-		String 				new_name,
-		OperationStatus		op_status )
+		String 				new_name )
 	{
 		throw( new RuntimeException( "moveDataFiles not implemented" ));
 	}
@@ -602,7 +615,7 @@ LWSDiskManager
     }
 
     @Override
-    public File 
+    public String 
     getMoveSubTask()
     {
     	return( null );
@@ -797,21 +810,28 @@ LWSDiskManager
 	}
 
   	@Override
-	  public void
+	public void
     skippedFileSetChanged(
   	   	DiskManagerFileInfo	file )
   	{
   	}
 
   	@Override
-	  public void
+	public void
   	priorityChanged(
   		DiskManagerFileInfo	file )
   	{
   	}
 
   	@Override
-	  public File
+  	public void 
+  	storageTypeChanged(
+  			DiskManagerFileInfo file)
+  	{
+  	}
+  	
+  	@Override
+	public File
   	getSaveLocation()
   	{
   		return( save_file );
