@@ -488,6 +488,12 @@ LWSDownload
 		throw( new DownloadRemovalVetoException( "no way" ));
 	}
 
+	@Override
+	public void stopAndRemove(boolean delete_torrent, boolean delete_data)
+			throws DownloadException, DownloadRemovalVetoException {
+		throw (new DownloadRemovalVetoException("no way"));
+	}
+
 	@Override
 	public boolean
 	isRemoved()
@@ -902,16 +908,16 @@ LWSDownload
 
 
 	@Override
-	public int
+	public SeedingRank
 	getSeedingRank()
 	{
-		return( 0 );
+		return( null );
 	}
 
 	@Override
 	public void
 	setSeedingRank(
-		int rank)
+		SeedingRank rank)
 	{
 		notSupported();
 	}
@@ -1068,4 +1074,4 @@ LWSDownload
 	{
 		return new Object[] { lws };
 	}
-}
\ No newline at end of file
+}
