@@ -43,6 +43,15 @@ PEPeerTransport
   public static final int CONNECTION_WAITING_FOR_HANDSHAKE  = 2;
   public static final int CONNECTION_FULLY_ESTABLISHED      = 4;
 
+  public static final int CP_UNKNOWN				= 0;
+  public static final int CP_CONNECTING				= 1;
+  public static final int CP_CONNECT_OK				= 2;
+  public static final int CP_CONNECT_FAILED			= 3;
+  public static final int CP_RECEIVED_DATA			= 4;
+
+	public int
+	getOutboundConnectionProgress();
+
   		/**
   		 * Start message processing for the peer
   		 */
@@ -123,8 +132,7 @@ PEPeerTransport
    * @param reason for closure
    */
 	public void closeConnection( String reason );
-
-
+	
 	@Override
 	public boolean
 	transferAvailable();
