@@ -149,6 +149,13 @@ SubscriptionManager
 	public SubscriptionScheduler
 	getScheduler();
 
+	public int
+	getDefaultCheckFrequencyMins();
+	
+	public void
+	setDefaultCheckFrequencyMins(
+		int		mins );
+	
 	public int
 	getMaxNonDeletedResults();
 
