@@ -61,6 +61,8 @@ TagBase
 	protected static final String	AT_RSS_ENABLE					= "rss.enable";
 	protected static final String	AT_RATELIMIT_UP_PRI				= "rl.uppri";
 	protected static final String	AT_XCODE_TARGET					= "xcode.to";
+	
+	protected static final String	AT_FL_PREVENT_DELETE			= "fl.pd";
 	protected static final String	AT_FL_MOVE_COMP					= "fl.comp";
 	protected static final String	AT_FL_MOVE_COMP_OPT				= "fl.comp.o";
 	protected static final String	AT_FL_MOVE_REM					= "fl.rem";
@@ -72,13 +74,16 @@ TagBase
 	protected static final String	AT_FL_MOVE_ASSIGN				= "fl.ass";
 	protected static final String	AT_FL_MOVE_ASSIGN_OPT			= "fl.ass.o";
 
-	protected static final String	AT_RATELIMIT_MIN_SR				= "rl.minsr";
-	protected static final String	AT_RATELIMIT_MAX_SR				= "rl.maxsr";
-	protected static final String	AT_RATELIMIT_MAX_SR_ACTION		= "rl.maxsr.a";
-	protected static final String	AT_RATELIMIT_MAX_AGGREGATE_SR	= "rl.maxaggsr";
+	protected static final String	AT_RATELIMIT_MIN_SR						= "rl.minsr";
+	protected static final String	AT_RATELIMIT_MAX_SR						= "rl.maxsr";
+	protected static final String	AT_RATELIMIT_MAX_SR_ACTION				= "rl.maxsr.a";
+	protected static final String	AT_RATELIMIT_MAX_AGGREGATE_SR			= "rl.maxaggsr";
 	protected static final String	AT_RATELIMIT_MAX_AGGREGATE_SR_ACTION	= "rl.maxaggsr.a";
 	protected static final String	AT_RATELIMIT_MAX_AGGREGATE_SR_PRIORITY	= "rl.maxaggsr.p";
 	protected static final String	AT_RATELIMIT_FP_SEEDING					= "rl.fps";
+	protected static final String	AT_RATELIMIT_BOOST						= "rl.bst";
+	protected static final String	AT_RATELIMIT_MAX_ACTIVE_DL				= "rl.maxadl";
+	
 	protected static final String	AT_PROPERTY_PREFIX				= "pp.";
 	//protected static final String	AT_EOA_PREFIX					= "eoa.";	// meh, should be used but copy/paste error resulted in AT_PROPERTY_PREFIX being used instead 
 	protected static final String	AT_BYTES_UP						= "b.up";
@@ -93,6 +98,7 @@ TagBase
 	protected static final String	AT_NOTIFICATION_PUBLISH			= "noti.pub";
 	protected static final String	AT_LIMIT_ORDERING				= "max.t.o";
 	protected static final String	AT_EOS_ASSIGN_TAGS				= "eos.at";
+	protected static final String	AT_EOS_REMOVE_TAGS				= "eos.rt";
 
 	private static final String[] EMPTY_STRING_LIST = {};
 
@@ -193,17 +199,20 @@ TagBase
 
 				tag_limits = (TagFeatureLimits)this;
 			}
-			
-			if ( group != null ){
-				
-				tag_type.setTagGroup( this, null, group );
-			}
 		}
 	}
 
 	protected void
 	initialized()
 	{
+			// need to defer group setting until this point as the tag-group can have listeners and we 
+			// don't want the Tag to 'escape' to the world before it is initialized
+		
+		if ( group != null ){
+			
+			tag_type.setTagGroup( this, null, group );
+		}
+		
 		loadPersistentStuff();
 
 		loadTransientStuff();
@@ -678,7 +687,19 @@ TagBase
 	setColor(
 		int[]		rgb )
 	{
-
+		if ( rgb == null ){
+			
+			if ( group != null ){
+			
+				TagGroup tg = getGroupContainer();
+				
+				if ( tg != null ){
+					
+					rgb = tg.getColor();
+				}
+			}
+		}
+		
 		boolean changed = writeStringAttribute( AT_COLOR_ID, encodeRGB( rgb ));
 
 		colour = null;
@@ -765,7 +786,85 @@ TagBase
 			}
 		}
 	}
+	
+	public boolean
+	getPreventDelete()
+	{
+		if ( tag_fl != null ){
+		
+			return( readBooleanAttribute( AT_FL_PREVENT_DELETE, false ));
+		}
+		
+		return( false );
+	}
+	
+	public void
+	setPreventDelete(
+		boolean		b )
+	{
+		if ( tag_fl != null ){
+		
+			if ( getPreventDelete() != b ){
+			
+				writeBooleanAttribute( AT_FL_PREVENT_DELETE, b );
+		
+				tag_type.fireMetadataChanged( this );
+			}
+		}
+	}
 
+	public boolean
+	getTagBoost()
+	{
+		if ( tag_rl != null ){
+			
+			return( readBooleanAttribute( AT_RATELIMIT_BOOST, false ));
+		}
+		
+		return( false );
+	}
+
+	public void
+	setTagBoost(
+		boolean		boost )
+	{
+		if ( tag_rl != null ){
+			
+			if ( getTagBoost() != boost ){
+			
+				writeBooleanAttribute( AT_RATELIMIT_BOOST, boost );
+		
+				tag_type.fireMetadataChanged( this );
+			}
+		}
+	}
+	
+	public int
+	getMaxActiveDownloads()
+	{
+		if ( tag_rl != null ){
+			
+			return( readLongAttribute( AT_RATELIMIT_MAX_ACTIVE_DL, 0L ).intValue());
+		}
+		
+		return( 0 );
+	}
+
+	public void
+	setMaxActiveDownloads(
+		int			max )
+	{
+		if ( tag_rl != null ){
+			
+			if ( getMaxActiveDownloads() != max ){
+			
+				writeLongAttribute( AT_RATELIMIT_MAX_ACTIVE_DL, max );
+		
+				tag_type.fireMetadataChanged( this );
+			}
+		}
+	}
+	
 		// initial save location
 
 	public boolean
@@ -1853,9 +1952,196 @@ TagBase
 		
 		tag_type.fireMetadataChanged( this );
 	}
+	
+		// remove tags
+		
+	public List<Tag>
+	getTagRemoves()
+	{
+		String[] tag_uids = readStringListAttribute( AT_EOS_REMOVE_TAGS, new String[0] );
+	
+		if ( tag_uids == null || tag_uids.length == 0 ){
+	
+			return( Collections.emptyList());
+		}
+		
+		List<Tag> result = new ArrayList<>();
+	
+		for ( String uid: tag_uids ){
+			
+			try{
+				Tag tag = tag_type.getTagManager().lookupTagByUID( Long.parseLong( uid ));
+				
+				if ( tag != null ){
+					
+					result.add( tag );
+				}
+			}catch( Throwable e ){
+				
+				Debug.out( e );
+			}
+		}
+		
+		return( result );
+	}
+	
+	public void
+	setTagRemoves(
+		List<Tag>	tags )
+	{
+		if ( tags == null ){
+	
+			tags = new ArrayList<>();
+		}
+	
+		String[] tag_uids = new String[tags.size()];
+		
+		for ( int i=0;i<tag_uids.length;i++ ){
+			
+			tag_uids[i] = String.valueOf( tags.get(i).getTagUID());
+		}
+		
+		writeStringListAttribute( AT_EOS_REMOVE_TAGS, tag_uids);
+	
+		setActionEnabled( TagFeatureExecOnAssign.ACTION_REMOVE_TAGS, !tags.isEmpty());
+		
+		tag_type.fireMetadataChanged( this );
+	}
 
+	public String
+	getPropertiesString()
+	{
+		TagFeatureProperties.TagProperty[] props = getSupportedProperties();
+
+		String text = "";
+		
+		if ( props.length > 0 ){
+
+			for ( TagFeatureProperties.TagProperty prop: props ){
+
+				String prop_str = prop.getString();
+
+				if ( prop_str.length() > 0 ){
+
+					text += (text.length()==0?"":"; ") + prop_str;
+				}
+			}
+		}
+		
+		return( text );
+	}
+	
+	public String
+	getEOAString()
+	{
+		String text = "";
+		
+		int	actions = getSupportedActions();
+
+		if ( actions != TagFeatureExecOnAssign.ACTION_NONE ){
+
+			String actions_str = "";
+
+			boolean is_peer_set = getTagType().getTagType() == TagType.TT_PEER_IPSET;
+			
+			int[]	action_ids =
+				{	TagFeatureExecOnAssign.ACTION_APPLY_OPTIONS_TEMPLATE,
+				 	TagFeatureExecOnAssign.ACTION_DESTROY,
+					TagFeatureExecOnAssign.ACTION_START,
+					TagFeatureExecOnAssign.ACTION_FORCE_START,
+					TagFeatureExecOnAssign.ACTION_NOT_FORCE_START,
+					TagFeatureExecOnAssign.ACTION_STOP,
+					TagFeatureExecOnAssign.ACTION_QUEUE,
+					TagFeatureExecOnAssign.ACTION_SCRIPT,
+					TagFeatureExecOnAssign.ACTION_PAUSE,
+					TagFeatureExecOnAssign.ACTION_RESUME,
+					TagFeatureExecOnAssign.ACTION_POST_MAGNET_URI,
+					TagFeatureExecOnAssign.ACTION_MOVE_INIT_SAVE_LOC,
+					TagFeatureExecOnAssign.ACTION_ASSIGN_TAGS,
+					TagFeatureExecOnAssign.ACTION_REMOVE_TAGS,
+					TagFeatureExecOnAssign.ACTION_HOST,
+					TagFeatureExecOnAssign.ACTION_PUBLISH };
+
+			String[] action_keys =
+				{ 	"label.apply.options.template",
+					is_peer_set?"azbuddy.ui.menu.disconnect":"v3.MainWindow.button.delete",
+					"v3.MainWindow.button.start",
+					"v3.MainWindow.button.forcestart",
+					"v3.MainWindow.button.notforcestart",
+					"v3.MainWindow.button.stop",
+					"ConfigView.section.queue",
+					"label.script",
+					"v3.MainWindow.button.pause",
+					"v3.MainWindow.button.resume",
+					"label.post.magnet.to.chat",
+					"label.init.save.loc.move",
+					"label.assign.tags",
+					"label.remove.tags",
+					"menu.host.on.tracker",
+					"menu.publish.on.tracker"};
+
+			for ( int i=0; i<action_ids.length;i++ ){
+
+				int	action_id = action_ids[i];
+
+				if ( supportsAction( action_id)){
+
+					boolean enabled = isActionEnabled( action_id );
+
+					if ( enabled ){
+
+						if ( action_id == TagFeatureExecOnAssign.ACTION_SCRIPT ){
+
+							String script = getActionScript();
+
+							if ( script.length() > 63 ){
+								script = script.substring( 0, 60 ) + "...";
+							}
+
+							actions_str += (actions_str.length()==0?"":",") +
+									MessageText.getString( action_keys[i]) + "=" + script;
+							
+						}else if ( 	action_id == TagFeatureExecOnAssign.ACTION_ASSIGN_TAGS ||
+									action_id == TagFeatureExecOnAssign.ACTION_REMOVE_TAGS ){
+							
+							List<Tag> tags = action_id == TagFeatureExecOnAssign.ACTION_ASSIGN_TAGS?getTagAssigns():getTagRemoves();
+							
+							String tag_str = "";
+							
+							for ( Tag t: tags ){
+								
+								tag_str += (tag_str==""?"":";") + t.getTagName( true );
+							}
+							
+							actions_str += (actions_str.length()==0?"":",") +
+									MessageText.getString( action_keys[i]) + "=" + tag_str ;
+							
+						}else{
+
+							actions_str += (actions_str.length()==0?"":",") +
+											MessageText.getString( action_keys[i]) + "=Y";
+						}
+					}
+				}
+			}
+
+			if ( actions_str.length() > 0 ){
+
+				text += actions_str;
+			}
+		}
+		
+		return( text );
+	}
 		// others
 
+	@Override
+	public void
+	addTaggableBatch(
+		boolean		starts )
+	{
+	}
+	
 	@Override
 	public void
 	addTaggable(
@@ -2888,41 +3174,50 @@ TagBase
 
 							value += vals[0];
 
-							if ( vals.length > 1 ){
-
-								String options = vals[1];
-
-								boolean new_dls = options.contains( "am=3;" );
-
-								if ( new_dls ){
+							if ( value.trim().isEmpty()){
 								
-									value += "," + MessageText.getString( "label.scope" );
-									
-									value += "=";
-									
-									value += MessageText.getString( "label.new.downloads" );
-									
-								}else{
-									
-									boolean auto_add 	= !options.contains( "am=2;" );
-									boolean auto_remove = !options.contains( "am=1;" );
+									// no constraint set
+								
+								value = null;
+								
+							}else{
+								
+								if ( vals.length > 1 ){
 	
+									String options = vals[1];
 	
-									if ( auto_add && auto_remove ){
-	
-									}else if ( auto_add || auto_remove ){
+									boolean new_dls = options.contains( "am=3;" );
 	
+									if ( new_dls ){
+									
 										value += "," + MessageText.getString( "label.scope" );
-	
+										
 										value += "=";
-	
-										if ( auto_add ){
-	
-											value += MessageText.getString( "label.addition.only" );
-	
-										}else{
-	
-											value += MessageText.getString( "label.removal.only" );
+										
+										value += MessageText.getString( "label.new.downloads" );
+										
+									}else{
+										
+										boolean auto_add 	= !options.contains( "am=2;" );
+										boolean auto_remove = !options.contains( "am=1;" );
+		
+		
+										if ( auto_add && auto_remove ){
+		
+										}else if ( auto_add || auto_remove ){
+		
+											value += "," + MessageText.getString( "label.scope" );
+		
+											value += "=";
+		
+											if ( auto_add ){
+		
+												value += MessageText.getString( "label.addition.only" );
+		
+											}else{
+		
+												value += MessageText.getString( "label.removal.only" );
+											}
 										}
 									}
 								}
