@@ -46,7 +46,7 @@ import com.biglybt.plugin.net.buddy.BuddyPluginUtils;
 public class
 TagDownloadWithState
 	extends TagWithState
-	implements TagDownload
+	implements TagDownload, TaggableResolver.LifecycleControlListener
 {
 	private static Object	FP_DL_KEY = new Object();
 	
@@ -77,6 +77,8 @@ TagDownloadWithState
 	private boolean	supports_xcode;
 	private boolean	supports_file_location;
 
+	private boolean	prevent_delete;
+	
 	private String	notification_pub = "";
 		
 	final Object	rate_lock = new Object();
@@ -260,6 +262,18 @@ TagDownloadWithState
 
 		notification_pub					= getNotifyMessageChannel();
 		
+		prevent_delete = getPreventDelete();
+		
+		if ( prevent_delete ){
+			
+			TaggableResolver resolver = getTagType().getResolver();
+			
+			if ( resolver != null ){
+			
+				resolver.addLifecycleControlListener( this );
+			}
+		}
+		
 		addTagListener(
 			new TagListener()
 			{
@@ -292,7 +306,7 @@ TagDownloadWithState
 						
 						updateFPSeeding( manager, true );
 					}
-					
+										
 					ncp_pub_list_mutate_index.incrementAndGet();
 				}
 
@@ -478,6 +492,59 @@ TagDownloadWithState
 			});
 	}
 	
+	private List<DownloadManager>	batch 		= new ArrayList<DownloadManager>();
+	private int						batch_depth	= 0;
+	
+	@Override
+	public void
+	addTaggableBatch(
+		boolean		starts )
+	{
+		List<DownloadManager> to_do = new ArrayList<>();
+		
+		synchronized( batch ){
+	
+			if ( starts ){
+				
+				batch_depth++;
+				
+			}else{
+				
+				batch_depth--;
+				
+				if ( batch_depth == 0 ){
+					
+					to_do.addAll( batch );
+					
+					batch.clear();
+				}
+			}
+		}
+		
+		if ( !to_do.isEmpty()){
+			
+			String script = getActionScript();
+
+			if ( script.length() > 0 ){
+				
+				rs_async.dispatch(
+						new AERunnable()
+						{
+							@Override
+							public void
+							runSupport()
+							{
+								TagManagerImpl.getSingleton().evalScript(
+									TagDownloadWithState.this,
+									script,
+									to_do,
+									"execAssign" );
+							}
+						});
+			}
+		}
+	}
+	
 	@Override
 	public void
 	addTaggable(
@@ -532,7 +599,8 @@ TagDownloadWithState
 						}
 
 					}else if ( 	isActionEnabled( TagFeatureExecOnAssign.ACTION_STOP ) ||
-								isActionEnabled( TagFeatureExecOnAssign.ACTION_PAUSE )){
+								isActionEnabled( TagFeatureExecOnAssign.ACTION_PAUSE ) ||
+								isActionEnabled( TagFeatureExecOnAssign.ACTION_QUEUE )){
 
 						int	dm_state = dm.getState();
 
@@ -551,6 +619,10 @@ TagDownloadWithState
 
 											dm.stopIt( DownloadManager.STATE_STOPPED, false, false );
 
+										}else if ( isActionEnabled( TagFeatureExecOnAssign.ACTION_QUEUE )){
+
+											dm.stopIt( DownloadManager.STATE_QUEUED, false, false );
+
 										}else{
 
 											dm.pause( true );
@@ -599,20 +671,30 @@ TagDownloadWithState
 
 						if ( script.length() > 0 ){
 
-							rs_async.dispatch(
-								new AERunnable()
-								{
-									@Override
-									public void
-									runSupport()
-									{
-										TagManagerImpl.getSingleton().evalScript(
-											TagDownloadWithState.this,
-											script,
-											dm,
-											"execAssign" );
-									}
-								});
+							synchronized( batch ){
+								
+								if ( batch_depth > 0 ){
+							
+									batch.add( dm );
+									
+								}else{
+							
+									rs_async.dispatch(
+										new AERunnable()
+										{
+											@Override
+											public void
+											runSupport()
+											{
+												TagManagerImpl.getSingleton().evalScript(
+													TagDownloadWithState.this,
+													script,
+													Arrays.asList( dm ),
+													"execAssign" );
+											}
+										});
+								}
+							}
 						}
 					}
 
@@ -742,6 +824,31 @@ TagDownloadWithState
 						}
 					}
 					
+					if ( isActionEnabled( TagFeatureExecOnAssign.ACTION_REMOVE_TAGS )){
+						
+						List<Tag> tags = getTagRemoves();
+						
+						if ( !tags.isEmpty()){
+
+							rs_async.dispatch(
+								new AERunnable()
+								{
+									@Override
+									public void
+									runSupport()
+									{
+										for ( Tag t: tags ){
+										
+											if ( t.hasTaggable( dm )){
+											
+												t.removeTaggable( dm );
+											}
+										}
+									}
+								});
+						}
+					}
+
 					if ( isActionEnabled( TagFeatureExecOnAssign.ACTION_HOST )){
 
 						rs_async.dispatch(
@@ -1416,6 +1523,42 @@ TagDownloadWithState
 		checkFPSeeding();
 	}
 	
+	public void
+	setPreventDelete(
+		boolean		b )
+	{
+		super.setPreventDelete( b );
+		
+		if ( prevent_delete != b ){
+			
+			prevent_delete = b;
+				
+			TaggableResolver resolver = getTagType().getResolver();
+			
+			if ( prevent_delete && resolver != null ){
+				
+				resolver.addLifecycleControlListener( this );
+				
+			}else{
+				
+				resolver.removeLifecycleControlListener( this );
+			}
+		}
+	}
+	
+	public void
+	canTaggableBeRemoved(
+		Taggable		taggable )
+	
+		throws Exception
+	{
+		if ( prevent_delete && hasTaggable( taggable )){
+			
+			
+			throw( new Exception( "Download removal prevented by Tag '" + getTagName() + "'" ));
+		}
+	}
+	
 	@Override
 	public void
 	setNotifyMessageChannel(
@@ -2294,6 +2437,7 @@ TagDownloadWithState
 					TagFeatureExecOnAssign.ACTION_FORCE_START |
 					TagFeatureExecOnAssign.ACTION_NOT_FORCE_START |
 					TagFeatureExecOnAssign.ACTION_STOP |
+					TagFeatureExecOnAssign.ACTION_QUEUE |
 					TagFeatureExecOnAssign.ACTION_PAUSE |
 					TagFeatureExecOnAssign.ACTION_SCRIPT |
 					TagFeatureExecOnAssign.ACTION_APPLY_OPTIONS_TEMPLATE |
@@ -2301,7 +2445,8 @@ TagDownloadWithState
 					TagFeatureExecOnAssign.ACTION_MOVE_INIT_SAVE_LOC |
 					TagFeatureExecOnAssign.ACTION_ASSIGN_TAGS |
 					TagFeatureExecOnAssign.ACTION_HOST |
-					TagFeatureExecOnAssign.ACTION_PUBLISH );
+					TagFeatureExecOnAssign.ACTION_PUBLISH |
+					TagFeatureExecOnAssign.ACTION_REMOVE_TAGS );
 
 		}else if ( getTagType().getTagType() == TagType.TT_DOWNLOAD_STATE ){
 
