@@ -32,9 +32,12 @@ import com.biglybt.core.Core;
 import com.biglybt.core.CoreComponent;
 import com.biglybt.core.CoreFactory;
 import com.biglybt.core.CoreLifecycleAdapter;
+import com.biglybt.core.CoreOperation;
+import com.biglybt.core.CoreOperationTask;
 import com.biglybt.core.config.COConfigurationManager;
 import com.biglybt.core.disk.DiskManager;
 import com.biglybt.core.download.DownloadManager;
+import com.biglybt.core.download.DownloadManagerException;
 import com.biglybt.core.download.DownloadManagerInitialisationAdapter;
 import com.biglybt.core.download.DownloadManagerState;
 import com.biglybt.core.global.GlobalManager;
@@ -1188,8 +1191,87 @@ TagManagerImpl
 										Thread.sleep( 1000 );
 									}
 
-									manager.copyDataFiles( new_loc );
+									// manager.copyDataFiles( new_loc );
 
+									
+									try{
+										FileUtil.runAsTask(
+												CoreOperation.OP_DOWNLOAD_COPY,
+												new CoreOperationTask()
+												{
+													private ProgressCallback cb = new CoreOperationTask.ProgressCallbackAdapter();
+													
+													@Override
+													public String 
+													getName()
+													{
+														return( manager.getDisplayName());
+													}
+
+													@Override
+													public DownloadManager 
+													getDownload()
+													{
+														return( manager );
+													}
+
+													@Override
+													public String[] 
+													getAffectedFileSystems()
+													{
+														return( FileUtil.getFileStoreNames( manager.getAbsoluteSaveLocation(), new_loc ));
+													}
+
+													@Override
+													public void
+													run(
+															CoreOperation operation)
+													{
+														try{
+															manager.copyDataFiles( new_loc, cb );
+
+														}catch( Throwable e ){
+
+															throw( new RuntimeException( e ));
+														}
+													}
+
+													@Override
+													public ProgressCallback 
+													getProgressCallback()
+													{
+														return( cb );
+													}
+												});
+
+									}catch( Throwable e ){
+
+										Throwable f = e.getCause();
+
+										if ( f instanceof DownloadManagerException ){
+
+											throw((DownloadManagerException)f);
+										}
+
+										throw( new DownloadManagerException( "Copy failed", e ));
+									}
+									
+									
+									
+									
+									
+									
+									
+									
+									
+									
+									
+									
+									
+									
+									
+									
+									
 									Logger.logTextResource(
 										new LogAlert(
 											manager,
@@ -1238,33 +1320,40 @@ TagManagerImpl
 
 	protected Object
 	evalScript(
-		Tag					tag,
-		String				script,
-		DownloadManager		dm,
-		String				intent_key )
+		Tag						tag,
+		String					script,
+		List<DownloadManager>	dms,
+		String					intent_key )
 	{
 		String script_type = "";
 
-		if ( script.length() >=10 && script.substring(0,10).toLowerCase( Locale.US ).startsWith( "javascript" )){
-
-			int	p1 = script.indexOf( '(' );
-
-			int	p2 = script.lastIndexOf( ')' );
-
-			if ( p1 != -1 && p2 != -1 ){
-
-				script = script.substring( p1+1, p2 ).trim();
-
-				if ( script.startsWith( "\"" ) && script.endsWith( "\"" )){
-
-					script = script.substring( 1, script.length()-1 );
+		script = script.trim();
+		
+		if ( script.length() >=10 ){
+			
+			String start = script.substring(0,10).toLowerCase( Locale.US );
+		
+			if ( start.startsWith( "javascript" ) || start.startsWith( "plugin" )){
+				
+				int	p1 = script.indexOf( '(' );
+	
+				int	p2 = script.lastIndexOf( ')' );
+	
+				if ( p1 != -1 && p2 != -1 ){
+	
+					script = script.substring( p1+1, p2 ).trim();
+	
+					if ( script.startsWith( "\"" ) && script.endsWith( "\"" )){
+	
+						script = script.substring( 1, script.length()-1 );
+					}
+	
+						// allow people to escape " if it makes them feel better
+	
+					script = script.replaceAll( "\\\\\"", "\"" );
+	
+					script_type = start.startsWith( "javascript" )?ScriptProvider.ST_JAVASCRIPT:ScriptProvider.ST_PLUGIN;
 				}
-
-					// allow people to escape " if it makes them feel better
-
-				script = script.replaceAll( "\\\\\"", "\"" );
-
-				script_type = ScriptProvider.ST_JAVASCRIPT;
 			}
 		}
 
@@ -1287,46 +1376,116 @@ TagManagerImpl
 
 				provider_found = true;
 
-				Download plugin_dm = PluginCoreUtils.wrap( dm );
+				if ( dms.size() > 1 && p.canEvalBatch( script )){
+													
+					Map<String,Object>	bindings = new HashMap<>();
+	
+					List<String>	intents 	= new ArrayList<>();
+					List<Download>	plugin_dms	= new ArrayList<>();
+						
+					for ( DownloadManager dm: dms ){
 
-				if ( plugin_dm == null ){
+						Download plugin_dm = PluginCoreUtils.wrap( dm );
+						
+						if ( plugin_dm == null ){
+		
+							continue;
+						}
 
-					return( null );	// deleted in the meantime
-				}
-
-				Map<String,Object>	bindings = new HashMap<>();
-
-
-				String dm_name = dm.getDisplayName();
-
-				if ( dm_name.length() > 32 ){
-
-					dm_name = dm_name.substring( 0, 29 ) + "...";
-				}
-
-				String intent = intent_key + "(\"" + tag.getTagName() + "\",\"" + dm_name + "\")";
-
-				bindings.put( "intent", intent );
-
-				bindings.put( "download", plugin_dm );
-
-				bindings.put( "tag", tag );
-
-				try{
-					Object result = p.eval( script, bindings );
-
-					return( result );
-
-				}catch( Throwable e ){
-
-					Debug.out( e );;
+						String dm_name = dm.getDisplayName();
+		
+						if ( dm_name.length() > 32 ){
+		
+							dm_name = dm_name.substring( 0, 29 ) + "...";
+						}
+		
+						String intent = intent_key + "(\"" + tag.getTagName() + "\",\"" + dm_name + "\")";
+						
+						intents.add( intent );
+						
+						plugin_dms.add( plugin_dm );
+					}
 					
-					return( e );
+					if ( intents.isEmpty()){
+						
+						return( null );
+					}
+					
+					bindings.put( "intents", intents );
+	
+					bindings.put( "downloads", plugin_dms );
+	
+					bindings.put( "tag", tag );
+	
+					try{
+						Object result = p.eval( script, bindings );
+	
+						return( result );
+	
+					}catch( Throwable e ){
+	
+						Debug.out( e );
+						
+						return( e );
+					}
+					
+				}else{
+					
+					List<Object>	results = new ArrayList<>();
+					
+					for ( DownloadManager dm: dms ){
+						
+						Download plugin_dm = PluginCoreUtils.wrap( dm );
+		
+						if ( plugin_dm == null ){
+		
+							continue; // deleted in the meantime
+						}
+		
+						Map<String,Object>	bindings = new HashMap<>();
+		
+		
+						String dm_name = dm.getDisplayName();
+		
+						if ( dm_name.length() > 32 ){
+		
+							dm_name = dm_name.substring( 0, 29 ) + "...";
+						}
+		
+						String intent = intent_key + "(\"" + tag.getTagName() + "\",\"" + dm_name + "\")";
+		
+						bindings.put( "intent", intent );
+		
+						bindings.put( "download", plugin_dm );
+		
+						bindings.put( "tag", tag );
+		
+						try{
+							Object result = p.eval( script, bindings );
+		
+							results.add( result );
+		
+						}catch( Throwable e ){
+		
+							Debug.out( e );
+							
+							results.add( e );
+						}
+					}
+					
+					if ( results.size() == 1 && dms.size() == 1 ){
+						
+						return( results.get( 0 ));
+						
+					}else{
+						
+						return( results );
+					}
 				}
 			}
 		}
 
-		if ( !provider_found ){
+		if ( script_type == ScriptProvider.ST_JAVASCRIPT && !provider_found ){
 
 			if ( !js_plugin_install_tried ){
 
@@ -2153,6 +2312,23 @@ TagManagerImpl
 		}
 	}
 
+	public TaggableResolver
+	getResolver(
+		long		taggable_type )
+	{
+		synchronized( lifecycle_handlers ){
+
+			LifecycleHandlerImpl handler = lifecycle_handlers.get( taggable_type );
+			
+			if ( handler != null ){
+				
+				return( handler.resolver );
+			}
+		}
+		
+		return( null );
+	}
+	
 	@Override
 	public void
 	addTaggableLifecycleListener(
@@ -3331,6 +3507,20 @@ TagManagerImpl
 		}
 	}
 	
+	
+	@Override
+	public TagConstraint 
+	compileConstraint(
+		String expression ) 
+	{
+		if ( constraint_handler == null ){
+			
+			return( null );
+		}
+		
+		return( constraint_handler.compileConstraint( expression ));
+	}
+	
 	@Override
 	public VuzeFile 
 	exportTags(
@@ -3363,6 +3553,41 @@ TagManagerImpl
 		return( vf );
 	}
 	
+	@Override
+	public Tag 
+	duplicate(
+		Tag tag)
+	{
+		if ( tag.getTagType().getTagType() == TagType.TT_DOWNLOAD_MANUAL ){
+			
+			TagTypeDownloadManual tt = (TagTypeDownloadManual)tag.getTagType();
+
+			TagWithState tws = (TagWithState)tag;
+
+			Map	map = new HashMap();
+		
+			Map tag_map = new HashMap();
+	
+			VuzeFile	vf = VuzeFileHandler.getSingleton().create();
+			
+			tws.exportDetails( vf, tag_map, false );
+			
+			Map conf = getConf( tt, tws, false );
+				
+			tag_map	= BEncoder.cloneMap(tag_map);
+			conf 	= BEncoder.cloneMap( conf );
+			
+			TagDownloadWithState dup_tag = tt.importTag( tag_map, conf );
+			
+			tt.addTag( dup_tag );
+			
+			return( dup_tag );
+		}else{
+			
+			return( null );
+		}
+	}
+	
 	protected String
 	explain(
 		Tag				tag,
