@@ -27,7 +27,6 @@ import java.util.*;
 import com.biglybt.core.internat.LocaleTorrentUtil;
 import com.biglybt.core.internat.LocaleUtil;
 import com.biglybt.core.internat.LocaleUtilDecoder;
-import com.biglybt.core.torrent.TOTorrent;
 import com.biglybt.core.torrent.TOTorrentException;
 import com.biglybt.core.torrent.TOTorrentFile;
 import com.biglybt.core.util.*;
@@ -36,6 +35,10 @@ public class
 TOTorrentFileImpl
 	implements TOTorrentFile
 {
+	private static final byte FLAG_BEP47_PAD			= 0x01;
+	private static final byte FLAG_OTHER_PAD			= 0x02;
+	private static final byte FLAG_OTHER_PAD_CHECKED	= 0x04;
+	
 	private final TOTorrentImpl	torrent;
 	
 	private final int		index;
@@ -50,7 +53,7 @@ TOTorrentFileImpl
 
 	private final boolean	is_utf8;
 
-	private boolean		attr_pad_file;
+	private byte			flags;
 
 	private final TOTorrentFileHashTreeImpl	hash_tree;
 	private byte[]	root_hash;
@@ -265,7 +268,10 @@ TOTorrentFileImpl
 			
 			String attr_str = new String((byte[])value, Constants.UTF_8 );
 			
-			attr_pad_file = attr_str.contains( "p" );
+			if ( attr_str.contains( "p" )){
+				
+				flags |= FLAG_BEP47_PAD;
+			}
 		}			
 	}
 
@@ -280,7 +286,7 @@ TOTorrentFileImpl
 	public byte[]
 	getRootHash()
 	{
-		if ( root_hash == null && !attr_pad_file ){
+		if ( root_hash == null && ( flags & FLAG_BEP47_PAD ) == 0 ){
 			
 			torrent.fixupRootHashes();
 		}
@@ -326,8 +332,70 @@ TOTorrentFileImpl
 	@Override
 	public boolean 
 	isPadFile()
-	{
-		return( attr_pad_file );
+	{		
+		if (( flags & FLAG_BEP47_PAD ) != 0 ){
+			
+			return( true );
+		}
+		
+		if (( flags & FLAG_OTHER_PAD_CHECKED ) == 0 ){
+
+			byte[][] comps = path_components_utf8 == null ? path_components : path_components_utf8;
+			
+			byte[] last = comps[ comps.length-1];
+	
+				// "_____padding_file_0_" etc
+
+			byte	other_pad = 0;
+			
+			if ( last.length > 20 ){
+				
+				if ( 	last[0]  == '_' &&
+						last[1]  == '_' &&
+						last[2]  == '_' &&
+						last[3]  == '_' &&
+						last[4]  == '_' &&
+						last[5]  == 'p' &&
+						last[6]  == 'a' &&
+						last[7]  == 'd' &&
+						last[8]  == 'd' &&
+						last[9]  == 'i' &&
+						last[10] == 'n' &&
+						last[11] == 'g' &&
+						last[12] == '_' &&
+						last[13] == 'f' &&
+						last[14] == 'i' &&
+						last[15] == 'l' &&
+						last[16] == 'e' &&
+						last[17] == '_' ){
+						
+					int pos = 18;
+											
+					while( pos < last.length ){
+						
+						byte c = last[pos];
+						
+						if ( c < '0' || c > '9' ){
+							
+							break;
+							
+						}else{
+							
+							pos++;
+						}
+					}
+						
+					if ( pos > 18 && pos < last.length && last[pos] == '_' ){
+						
+						other_pad = FLAG_OTHER_PAD;
+					}
+				}
+			}
+			
+			flags |= ( FLAG_OTHER_PAD_CHECKED | other_pad );
+		}
+		
+		return( ( flags & FLAG_OTHER_PAD ) != 0 );
 	}
 	
 	@Override
