@@ -24,6 +24,7 @@ import java.net.URL;
 import java.util.*;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentLinkedDeque;
+import java.util.concurrent.atomic.AtomicLong;
 
 import com.biglybt.core.Core;
 import com.biglybt.core.CoreFactory;
@@ -115,6 +116,8 @@ AllTrackersManagerImpl
 	
 	private Map<HashWrapper,String>		dm_name_cache = new HashMap<>();
 	
+	private AtomicLong	options_mutation_count = new AtomicLong();
+	
 	private
 	AllTrackersManagerImpl()
 	{
@@ -789,6 +792,13 @@ AllTrackersManagerImpl
 		}
 	}
 	
+	@Override
+	public long 
+	getOptionsMutationCount()
+	{
+		return( options_mutation_count.get());
+	}
+	
 	@Override
 	public String
 	ingestURL(
@@ -875,6 +885,31 @@ AllTrackersManagerImpl
 		return( existing_tracker );
 	}
 	
+	private void
+	unregister(
+		String		name )
+	{
+		AllTrackersTrackerImpl existing_tracker = host_map.remove( name );
+		
+		if ( existing_tracker != null ){
+			
+			for ( AllTrackersListener listener: listeners ){
+				
+				List<AllTrackersTracker>	trackers = new ArrayList<>();
+				
+				trackers.add( existing_tracker );
+				
+				try{
+					listener.trackerEventOccurred( new AllTrackersEventImpl( AllTrackersEvent.ET_TRACKER_REMOVED, trackers ));
+	
+				}catch( Throwable e ){
+					
+					Debug.out( e );
+				}
+			}
+		}
+	}
+	
 	@Override
 	public void 
 	updateTracker(
@@ -934,6 +969,19 @@ AllTrackersManagerImpl
 		update_queue.add( new Object[]{ tracker, cmd } );
 	}
 	
+	@Override
+	public AllTrackersTracker 
+	getTracker(
+		String		name )
+	{
+		if ( name == null ){
+			
+			return( null );
+		}
+		
+		return( host_map.get( name ));
+	}
+	
 	@Override
 	public AllTrackersTracker 
 	getTracker(URL url)
@@ -1197,7 +1245,7 @@ AllTrackersManagerImpl
 																		
 					if ( consolidated == null ){
 						
-						consolidated = vals;
+						consolidated = vals.clone();
 						
 					}else{
 						
@@ -1255,6 +1303,20 @@ AllTrackersManagerImpl
 			return( registered );
 		}
 		
+		@Override
+		public boolean 
+		isRemovable()
+		{
+			return( num_private + num_public == 0 );
+		}
+		
+		@Override
+		public void 
+		remove()
+		{
+			unregister( name );
+		}
+		
 		@Override
 		public String 
 		getTrackerName()
@@ -1614,6 +1676,8 @@ AllTrackersManagerImpl
 			Map<String, Object> _options)
 		{
 			options = _options;
+			
+			options_mutation_count.incrementAndGet();
 		}
 		
 		@Override
