@@ -33,24 +33,53 @@ AEJavaManagement
 	public static void
 	initialise()
 	{
-		try{
-			thread_stuff = (ThreadStuff)Class.forName("com.biglybt.core.util.jman.AEThreadMonitor").newInstance();
-
-		}catch( ClassNotFoundException e ){
-
-		}catch( Throwable e ){
-
-			e.printStackTrace();
+		String thread_prop = System.getProperty( SystemProperties.SYSPROP_THREAD_MON_ENABLE, null );
+		
+		if ( thread_prop == null ){
+			
+			// not seeing any progress on https://bugs.java.com/bugdatabase/view_bug.do?bug_id=JDK-8245768 
+			// so setting linux default to disabled for the moment
+			
+			// thread_prop = ( Constants.isWindows || Constants.isOSX )?"1":"0";
+			
+			// Seems that using user-time on Linux (see SYSPROP_THREAD_MON_USERONLY) works around the bug so we switch
+			// from disabling entirely on Linux to defaulting to user-time
+			
+			thread_prop = "1";
 		}
+		
+		if ( thread_prop.equals( "0" )){
+			
+			Debug.outNoStack( "Thread monitoring disabled" );
+			
+		}else{
+			
+			try{
+				thread_stuff = (ThreadStuff)Class.forName("com.biglybt.core.util.jman.AEThreadMonitor").newInstance();
+	
+			}catch( ClassNotFoundException e ){
+	
+			}catch( Throwable e ){
+	
+				e.printStackTrace();
+			}
+		}
+		
+		if ( System.getProperty( SystemProperties.SYSPROP_MEMORY_MON_ENABLE, "1" ).equals( "0" )){
 
-		try{
-			memory_stuff = (MemoryStuff)Class.forName("com.biglybt.core.util.jman.AEMemoryMonitor").newInstance();
+			Debug.outNoStack( "Memory monitoring disabled" );
 
-		}catch( ClassNotFoundException e ){
-
-		}catch( Throwable e ){
-
-			e.printStackTrace();
+		}else{
+			
+			try{
+				memory_stuff = (MemoryStuff)Class.forName("com.biglybt.core.util.jman.AEMemoryMonitor").newInstance();
+	
+			}catch( ClassNotFoundException e ){
+	
+			}catch( Throwable e ){
+	
+				e.printStackTrace();
+			}
 		}
 	}
 
