@@ -43,6 +43,7 @@ import com.biglybt.core.CoreOperation;
 import com.biglybt.core.CoreOperationTask;
 import com.biglybt.core.config.COConfigurationListener;
 import com.biglybt.core.config.COConfigurationManager;
+import com.biglybt.core.config.ConfigKeys;
 import com.biglybt.core.diskmanager.file.impl.FMFileAccess.FileAccessor;
 import com.biglybt.core.download.DownloadManager;
 import com.biglybt.core.logging.LogEvent;
@@ -66,25 +67,67 @@ public class FileUtil {
   private static final List		reserved_file_handles 	= new ArrayList();
   private static final AEMonitor	class_mon				= new AEMonitor( "FileUtil:class" );
 
-  private static Method reflectOnUsableSpace;
+  private static final Method reflectOnUsableSpace;
 
+  	// FileStore is minSDK 26 on Android
+
+	protected static final Method 	mPath_getFileStore;
+	protected static final Method 	mToPath;
+
+	
   private static char[]	char_conversion_mapping = null;
 
   private static final FileHandler fileHandling;
 
   private static AEDiagnosticsLogger file_logger;
   
+  private static final ThreadLocal<IdentityHashSet<CoreOperationTask>>		tls	=
+	new ThreadLocal<IdentityHashSet<CoreOperationTask>>()
+	{
+	  @Override
+	  public IdentityHashSet<CoreOperationTask>
+	  initialValue()
+	  {
+		  return( new IdentityHashSet<CoreOperationTask>());
+	  }
+	};
+
   static {
 
-	  try
-	  {
-		  reflectOnUsableSpace = File.class.getMethod("getUsableSpace", (Class[])null);
-	  } catch (Throwable e)
-	  {
-		  reflectOnUsableSpace = null;
+	  Method _reflectOnUsableSpace;
+	
+	  try{
+		  _reflectOnUsableSpace = File.class.getMethod("getUsableSpace", (Class[])null);
+		  
+	  }catch (Throwable e){
+		  
+		  _reflectOnUsableSpace = null;
 	  }
 
-		String fileHandlingCN = System.getProperty("az.FileHandling.impl",
+	  reflectOnUsableSpace = _reflectOnUsableSpace;
+	  
+	  
+	  Method 	_mPath_getFileStore;
+	  Method 	_mToPath;
+
+	  try{
+		  Class  _claPath = Class.forName("java.nio.file.Path");
+		  Class  _claFiles = Class.forName("java.nio.file.Files");
+		  _mPath_getFileStore = _claFiles.getMethod("getFileStore", _claPath);
+		  _mToPath = File.class.getMethod("toPath");
+		  
+	  }catch( Throwable e ){
+		  
+		  _mPath_getFileStore	= null;
+		  _mToPath				= null;
+	  }
+	 
+	  mPath_getFileStore	= _mPath_getFileStore;
+	  mToPath				= _mToPath;
+
+	  
+	  
+	  String fileHandlingCN = System.getProperty("az.FileHandling.impl",
 			"");
 	  // To test:
 	  // fileHandlingCN = FileHandlerHack.class.getName();
@@ -1294,33 +1337,102 @@ public class FileUtil {
     }
     */
 
-    public static boolean copyFile( final File _source, final File _dest ) {
-      try {
-    	File parent = _dest.getParentFile();
-    	if ( !parent.exists()){
-    		parent.mkdirs();
+    public static boolean 
+    copyFileWithDates( 
+    	File 	from_file, 
+    	File 	to_file ) 
+    {
+		FileTime from_last_modified = null;
+		FileTime from_last_access	= null;
+		FileTime from_created		= null;
+		
+   		try{
+			BasicFileAttributeView from_attributes_view 	= Files.getFileAttributeView( from_file.toPath(), BasicFileAttributeView.class);
+
+			BasicFileAttributes from_attributes = from_attributes_view.readAttributes();
+
+			from_last_modified 	= from_attributes.lastModifiedTime();
+			from_last_access	= from_attributes.lastAccessTime();
+			from_created		= from_attributes.creationTime();
+			
+		}catch( Throwable e ){
+		}
+   		
+    	try{
+    		File parent = to_file.getParentFile();
+    		
+    		if ( !parent.exists()){
+    			
+    			parent.mkdirs();
+    		}
+    		
+    		copyFile( newFileInputStream( from_file ), newFileOutputStream( to_file ) );
+    		
+    		try{
+    			BasicFileAttributeView to_attributes_view 		= Files.getFileAttributeView( to_file.toPath(), BasicFileAttributeView.class);
+
+   				to_attributes_view.setTimes( from_last_modified, from_last_access, from_created );
+    		
+    		}catch( Throwable e ){
+    		}
+    		
+    		return( true );
+    		
+    	}catch( Throwable e ) {
+    		
+    		Debug.out( "Copy failed for " + from_file, e );
+    		
+    		return( false );
+    	}
+    }
+    
+    public static boolean 
+    copyFile( 
+    	File 	_source, 
+    	File 	_dest ) 
+    {
+    	try {
+    		File parent = _dest.getParentFile();
+    		if ( !parent.exists()){
+    			parent.mkdirs();
+    		}
+    		copyFile( newFileInputStream( _source ), newFileOutputStream( _dest ) );
+    		return true;
+    	}
+    	catch( Throwable e ) {
+    		Debug.out( "Copy failed for " + _source, e );
+    		return false;
     	}
-        copyFile( newFileInputStream( _source ), newFileOutputStream( _dest ) );
-        return true;
-      }
-      catch( Throwable e ) {
-      	Debug.out( "Copy failed for " + _source, e );
-        return false;
-      }
     }
 
-    public static void copyFileWithException( final File _source, final File _dest ) throws IOException{
-         copyFile( newFileInputStream( _source ), newFileOutputStream( _dest ) );
+    public static void 
+    copyFileWithException( 
+    	File 				_source, 
+    	File 				_dest,
+    	ProgressListener	pl ) 
+    		
+    	throws IOException
+    {
+         copyFile( newFileInputStream( _source ), newFileOutputStream( _dest ), pl );
     }
 
-    public static boolean copyFile( final File _source, final OutputStream _dest, boolean closeInputStream ) {
-        try {
+    public static boolean 
+    copyFile( 
+    	File 			_source, 
+    	OutputStream 	_dest, 
+    	boolean 		closeInputStream ) 
+    {
+        try{
+        	
           copyFile( newFileInputStream( _source ), _dest, closeInputStream );
+          
           return true;
-        }
-        catch( Throwable e ) {
+          
+        }catch( Throwable e ){
+        	
         	Debug.printStackTrace( e );
-          return false;
+          
+        	return false;
         }
       }
 
@@ -1427,6 +1539,17 @@ public class FileUtil {
       copyFile(is,os,true);
     }
 
+    public static void
+    copyFile(
+      InputStream   	is,
+      OutputStream  	os,
+      ProgressListener	pl )
+
+    	throws IOException
+    {
+    	copyFile(is,os,true,pl);
+    }
+    
     public static void
 	copyFile(
 		InputStream		is,
@@ -1468,6 +1591,77 @@ public class FileUtil {
     	}
 	}
 
+    public static void
+	copyFile(
+		InputStream			is,
+		OutputStream		os,
+		boolean 			closeInputStream,
+		ProgressListener	pl )
+
+		throws IOException
+	{
+    	try{
+
+    		if ( !(is instanceof BufferedInputStream )){
+
+    			is = new BufferedInputStream(is,128*1024);
+    		}
+
+    		byte[]	buffer = new byte[128*1024];
+
+      		while(true){
+
+    			if ( pl != null ){
+    				
+    				while( true ){
+    					
+    					int state =  pl.getState();
+    					
+    					if ( state == ProgressListener.ST_PAUSED ){
+    			
+    						try{
+    							Thread.sleep(250);
+    						}catch( Throwable e ){
+    						}
+    						
+    					}else if ( state == ProgressListener.ST_CANCELLED ){
+    						    						
+    						throw( new IOException( "Cancelled" ));
+    						
+    					}else{
+    						
+    						break;
+    					}
+    				}
+    			}	
+    			
+    			int	len = is.read(buffer);
+
+    			if ( len == -1 ){
+
+    				break;
+    			}
+
+    			os.write( buffer, 0, len );
+    			
+    			if ( pl != null ){
+    				
+    				pl.bytesDone( len );
+    			}
+    		}
+    	}finally{
+    		try{
+    			if(closeInputStream){
+    			  is.close();
+    			}
+    		}catch( IOException e ){
+
+    		}
+
+    		os.close();
+    	}
+	}
+    
     public static void
     copyFileOrDirectory(
     	File	from_file_or_dir,
@@ -1640,7 +1834,7 @@ public class FileUtil {
 		File		from_file,
 		File		to_file )
     {
-        return renameFile(from_file, to_file, true);
+        return( renameFile(from_file, to_file, true) == null );
    	}
 
     public static boolean
@@ -1649,10 +1843,10 @@ public class FileUtil {
  		File				to_file,
  		ProgressListener	pl )
     {
-         return renameFile(from_file, to_file, true, null, pl );
+         return( renameFile(from_file, to_file, true, null, pl ) == null );
     }
     
-    public static boolean
+    private static String
     renameFile(
         File        from_file,
         File        to_file,
@@ -1661,7 +1855,7 @@ public class FileUtil {
     	return renameFile(from_file, to_file, fail_on_existing_directory, null, null);
     }
     
-    public static boolean
+    public static String
     renameFile(
    		File        		from_file,
    		File        		to_file,
@@ -1685,9 +1879,9 @@ public class FileUtil {
 		}catch( Throwable e ){
 		}
    		
-    	boolean result = renameFileSupport( from_file, to_file, fail_on_existing_directory, file_filter, pl );
+    	String result = renameFileSupport( from_file, to_file, fail_on_existing_directory, file_filter, pl );
 
-    	if ( result ){
+    	if ( result == null ){
 
     			// try to maintain the file times if they now differ 
     		
@@ -1710,12 +1904,15 @@ public class FileUtil {
     			}
     		}catch( Throwable e ){
     		}
+    	}else{
+    		
+    		Debug.out( result );
     	}
 
     	return( result );
     }
         
-    private static boolean
+    private static String
     renameFileSupport(
     	File        		from_file,
     	File        		to_file,
@@ -1725,9 +1922,7 @@ public class FileUtil {
     {
     	if ( !from_file.exists()){
 
-    		Debug.out( "renameFile: source file '" + from_file + "' doesn't exist, failing" );
-
-    		return( false );
+    		return( "renameFile: source file '" + from_file + "' doesn't exist, failing" );
     	}
     	
     	boolean to_file_exists = to_file.exists();
@@ -1738,13 +1933,13 @@ public class FileUtil {
     			
     			if ( areFilePathsIdentical( from_file, to_file )){
     				
-    				return( true );	// nothing to do
+    				return( null );	// nothing to do
     				
     			}else{
     				
     				if ( from_file.renameTo( to_file )){
     					
-    					return( true );
+    					return( null );
     				}
     			}
     		}
@@ -1767,9 +1962,7 @@ public class FileUtil {
     	
         if ( to_file_exists && ( fail_on_existing_directory || from_file.isFile() || to_file.isFile())) {
 
-        	Debug.out( "renameFile: target file '" + to_file + "' already exists, failing" );
-
-            return( false );
+        	return( "renameFile: target file '" + to_file + "' already exists, failing" );
         }
         
 		if ( pl != null ){
@@ -1787,10 +1980,8 @@ public class FileUtil {
 					
 				}else if ( state == ProgressListener.ST_CANCELLED ){
 					
-					Debug.out( "renameFile: Cancelled" );
-					
-					return( false );
-					
+					return( "renameFile: Cancelled" );
+										
 				}else{
 					
 					break;
@@ -1822,11 +2013,13 @@ public class FileUtil {
 
     				// empty dir
 
-    			return( true );
+    			return( null );
     		}
 
     		int	last_ok = 0;
 
+    		String last_error = null;
+    		
     		if ( !to_file.exists()){
     			
     			to_file.mkdir();
@@ -1838,12 +2031,16 @@ public class FileUtil {
 				File	tf = newFile( to_file, ff.getName());
 
     			try{
-     				if ( renameFile( ff, tf, fail_on_existing_directory, file_filter, pl )){
+     				String res = renameFile( ff, tf, fail_on_existing_directory, file_filter, pl );
 
+     				if ( res == null ){
+     					
     					last_ok++;
 
     				}else{
 
+    					last_error = res;
+    					
     					break;
     				}
     			}catch( Throwable e ){
@@ -1868,7 +2065,7 @@ public class FileUtil {
     				}
     				else {
     					/* Should we log this? How should we log this? */
-    					return true;
+    					return( null );
     				}
 
     			}else{
@@ -1879,7 +2076,7 @@ public class FileUtil {
     				}
     			}
 
-    			return( true );
+    			return( null );
     		}
 
     			// recover by moving files back
@@ -1896,7 +2093,7 @@ public class FileUtil {
 	    			try{
 	    				// null - We don't want to use the file filter, it only refers to source paths.
 	    				
-	                    if ( !renameFile( 
+	                    if ( renameFile( 
 	                    		tf, 
 	                    		ff, 
 	                    		false, 
@@ -1945,7 +2142,7 @@ public class FileUtil {
 	                    			{
 	                    				
 	                    			}
-	                    		})){
+	                    		}) != null ){
 	                    
 	    					Debug.out( "renameFile: recovery - failed to move file '" + tf.toString()
 											+ "' to '" + ff.toString() + "'" );
@@ -1958,37 +2155,38 @@ public class FileUtil {
 	      		}
       		}
 
-      		return( false );
+    		if ( last_error != null ){
+      		
+    			return( last_error );
+    		}
 
+    		return( "failed to move one or more files" );
     	}else{
 
     		boolean	same_drive = false;
     		
-			try{
-				/* FileStore is minSDK 26 on Android
-				FileStore fs1 = Files.getFileStore( from_file.toPath());
-				FileStore fs2 = Files.getFileStore( to_file_parent.toPath());
-				*/
-
-				Class claPath = Class.forName("java.nio.file.Path");
-				Class claFiles = Class.forName("java.nio.file.Files");
-				Method mPath_getFileStore = claFiles.getMethod("getFileStore", claPath);
-
-				Method mToPath = from_file.getClass().getMethod("toPath");
-				/* Path */ Object from_file_toPath = mToPath.invoke(from_file);
-				/* Path */ Object to_file_parent_toPath = mToPath.invoke(to_file_parent);
-
-				/* FileStore */ Object fs1 = mPath_getFileStore.invoke(null, from_file_toPath);
-				/* FileStore */ Object fs2 = mPath_getFileStore.invoke(null, to_file_parent_toPath);
-
-   				if ( fs1.equals( fs2 )){
-
-   					same_drive = true;
-   				}
-			}catch( Throwable e ){
-			}
+	    	if ( mToPath != null && mPath_getFileStore != null ){
+				try{
+					/* FileStore is minSDK 26 on Android
+					FileStore fs1 = Files.getFileStore( from_file.toPath());
+					FileStore fs2 = Files.getFileStore( to_file_parent.toPath());
+					*/
+	
+					/* Path */ Object from_file_toPath = mToPath.invoke(from_file);
+					/* Path */ Object to_file_parent_toPath = mToPath.invoke(to_file_parent);
+	
+					/* FileStore */ Object fs1 = mPath_getFileStore.invoke(null, from_file_toPath);
+					/* FileStore */ Object fs2 = mPath_getFileStore.invoke(null, to_file_parent_toPath);
+	
+	   				if ( fs1.equals( fs2 )){
+	
+	   					same_drive = true;
+	   				}
+				}catch( Throwable e ){
+				}
+	    	}
 			
-    		boolean	use_copy = COConfigurationManager.getBooleanParameter("Copy And Delete Data Rather Than Move");
+	    	boolean	use_copy = COConfigurationManager.getBooleanParameter("Copy And Delete Data Rather Than Move");
 
     		if ( use_copy ){
 
@@ -2040,7 +2238,7 @@ public class FileUtil {
     					}
     				}
     				
-    				return( true );
+    				return( null );
     				
     			}else{
     				
@@ -2052,7 +2250,43 @@ public class FileUtil {
     	}
     }
 
-    private static boolean
+    public static Object
+    getFileStore(
+    	File		file )
+    {
+    	return fileHandling.getFileStore(file);
+    }
+    
+    public static String[]
+    getFileStoreNames(
+    	File...	files )
+    {
+    	if ( files == null ){
+    		
+    		return( new String[0] );
+    	}
+    	
+    	List<String>	result = new ArrayList<>( files.length );
+    	
+    	for ( File f: files ){
+    		
+    		if ( f == null ){
+    			
+    			continue;
+    		}
+    		
+    		Object fs = getFileStore( f );
+    		
+    		if ( fs != null ){
+    			
+    			result.add( String.valueOf( fs ));
+    		}
+    	}
+    	   
+    	return( result.toArray( new String[ result.size()] ));
+    }
+    		
+    private static String
     reallyCopyFile(
     	File				from_file,
     	File				to_file,
@@ -2160,23 +2394,67 @@ public class FileUtil {
 
     		to_os = null;
 
-    		if ( !from_file.delete()){
-    			Debug.out( "renameFile: failed to delete '"
-    					+ from_file.toString() + "'" );
-
-    			throw( new Exception( "Failed to delete '" + from_file.toString() + "'"));
+    			// sometimes the file can be locked (by AV?)
+    		
+    		long del_retry_start = 0;
+    		
+    		while( true ){
+    			
+	    		if ( from_file.delete()){
+	    			
+	    			break;
+	    		}
+	    		
+	    		long now = SystemTime.getMonotonousTime();
+	    		
+	    		if ( del_retry_start == 0 ){
+	    			
+	    			del_retry_start = now;
+	    			
+	    		}else if ( now - del_retry_start < 10*1000 ){
+	    			
+	    			try{
+	    				
+	    				Thread.sleep( 250 );
+	    				
+	    			}catch( Throwable e ){
+	    				
+	    			}
+	    		}else{
+	    			
+	    			
+	    			if ( COConfigurationManager.getBooleanParameter( ConfigKeys.File.BCFG_FILE_MOVE_ORIGIN_DELETE_FAIL_IS_WARNING )){
+	    				
+	    				log( "Warning: failed to delete " + from_file.getAbsolutePath() + " after moving via copy" );
+	    				
+	    				break;
+	    			}
+	    			
+	    			Debug.out( "renameFile: failed to delete '" + from_file.toString() + "'" );
+	
+	    			throw( new Exception( "Failed to delete '" + from_file.toString() + "'"));
+	    		}
+	    		
+	    		if ( pl != null ){
+	    		
+	    			int state =  pl.getState();
+	    			
+	    			if ( state == ProgressListener.ST_CANCELLED ){
+						
+						throw( new IOException( "Cancelled" ));
+	    			}
+	    		}
     		}
 
     		success	= true;
 
-    		return( true );
+    		return( null );
 
     	}catch( Throwable e ){
 
-    		Debug.out( "renameFile: failed to rename '" + from_file.toString()
-    		+ "' to '" + to_file.toString() + "'", e );
-
-    		return( false );
+    		return( 
+    			"renameFile: failed to rename '" + from_file.toString()	+ 
+    				"' to '" + to_file.toString() + "': " + Debug.getNestedExceptionMessage(e));
 
     	}finally{
 
@@ -2588,9 +2866,17 @@ public class FileUtil {
 
 			if ( op.getOperationType() == CoreOperation.OP_FILE_MOVE ){
 
-				if ( dm == op.getTask().getDownload()){
+				CoreOperationTask task = op.getTask();
+				
+				if ( task != null && dm == task.getDownload()){
 
-					return( true );
+						// allow recursive task creation for the same download - this happens
+						// when doing a quick-rename of a single file torrent's file from FilesView...
+					
+					if ( !tls.get().contains( task ) ){
+						
+						return( true );
+					}
 				}
 			}
 		}
@@ -2610,9 +2896,17 @@ public class FileUtil {
 		int					op_type,
 		CoreOperationTask 	task )
 	{
-		Core core = CoreFactory.getSingleton();
-
-		core.executeOperation( op_type, task );
+		try{
+			tls.get().add( task );
+		
+			Core core = CoreFactory.getSingleton();
+	
+			core.executeOperation( op_type, task );
+			
+		}finally{
+			
+			tls.get().remove( task );
+		}
 	}
 		
 	
