@@ -188,6 +188,13 @@ PluginCoreUtils
 					info.setSkipped(b);
 				}
 
+				@Override
+				public Boolean 
+				isSkipping()
+				{
+					return( info.isSkipping());
+				}
+				
 				@Override
 				public boolean
 				setLink(
