@@ -33,6 +33,7 @@ import java.util.*;
 import com.biglybt.core.Core;
 import com.biglybt.core.category.Category;
 import com.biglybt.core.config.COConfigurationManager;
+import com.biglybt.core.config.ConfigKeys;
 import com.biglybt.core.disk.DiskManager;
 import com.biglybt.core.download.DownloadManager;
 import com.biglybt.core.download.DownloadManagerInitialisationAdapter;
@@ -1264,7 +1265,23 @@ DownloadManagerImpl
 
 			return( false );
 		}
-
+		
+		/* Although this would seem to be the better thing to do it hides the actual cause
+		 * of the veto (e.g. tag prevents removal) and makes it hard for a user to diagnose the
+		 * reason. Better to allow things to go ahead and then fail on the removal attempt where
+		 * the user can be told of the cause.
+		 
+		try{
+			return( download.canBeRemoved());
+			
+		}catch( Throwable e ){
+			
+			Debug.out( e );
+			
+			return( false );
+		}
+		*/
+		
 		return( true );
 	}
 
@@ -1353,7 +1370,7 @@ DownloadManagerImpl
 			category = null;
 		}
 		
-		DownloadStubImpl stub = new DownloadStubImpl( this,	download, manual_tags, category,  gm_data );
+		DownloadStubImpl stub = new DownloadStubImpl( this,	download, core_dm, manual_tags, category,  gm_data );
 
 		try{
 			informAdded( stub, true );
@@ -1452,6 +1469,46 @@ DownloadManagerImpl
 					Debug.out( e );
 				}
 
+				File existing_location = core_dm.getSaveLocation();
+				
+				String stub_location = stub.getSavePath();
+				
+				if ( stub_location != null && !stub_location.isEmpty()){
+					
+					File f_stub = FileUtil.newFile( stub_location );
+					
+					if ( !f_stub.equals( existing_location )){
+						
+						core_dm.setTorrentSaveDir( f_stub, true );
+					}
+				}
+				
+				try{
+					
+					File	torrent_file = FileUtil.newFile( core_dm.getTorrentFileName() );
+
+					if ( !torrent_file.exists()){
+						
+						// user might have moved torrent file backup location, see if the
+						// torrent exists there
+						
+						String save_dir = COConfigurationManager.getDirectoryParameter( ConfigKeys.File.SCFG_GENERAL_DEFAULT_TORRENT_DIRECTORY );
+
+						if ( !save_dir.isEmpty()){
+							
+							File	test_file = FileUtil.newFile( save_dir, torrent_file.getName());
+
+							if ( test_file.exists()){
+								
+								core_dm.setTorrentFileName( test_file.getAbsolutePath());
+							}
+						}
+					}
+				}catch( Throwable e ){
+					
+					Debug.out( e );
+				}
+				
 				synchronized( download_stubs ){
 
 					download_stubs.remove( stub );
