@@ -22,6 +22,7 @@ package com.biglybt.pifimpl.local.sharing;
 
 import java.io.File;
 import java.util.ArrayList;
+import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 
@@ -57,7 +58,7 @@ ShareResourceDirContentsImpl
 
 		root 		= _dir;
 		recursive	= _recursive;
-		properties	= _properties;
+		properties	= _properties==null?new HashMap<>():_properties;
 
 		if ( !root.exists()){
 
@@ -298,40 +299,6 @@ ShareResourceDirContentsImpl
 		}
 	}
 
-	@Override
-	protected ShareResource 
-	lookupShare(
-		byte[] torrent_hash)
-	{
-		for (int i=0;i<children.length;i++){
-
-			try{
-				if ( children[i] instanceof ShareResourceImpl ){
-
-					ShareResource res = ((ShareResourceImpl)children[i]).lookupShare(torrent_hash);
-					
-					if ( res != null ){
-						
-						return( res );
-					}
-				}else{
-
-					ShareResource res = ((shareNode)children[i]).lookupShare(torrent_hash);
-
-					if ( res != null ){
-						
-						return( res );
-					}
-				}
-			}catch( Throwable e ){
-
-				Debug.printStackTrace( e );
-			}
-		}
-		
-		return( null );
-	}
-
 	@Override
 	protected void
 	serialiseResource(
@@ -361,7 +328,7 @@ ShareResourceDirContentsImpl
 
 		throws ShareException
 	{
-		File root = new File(new String((byte[]) map.get("file"), Constants.DEFAULT_ENCODING_CHARSET));
+		File root = FileUtil.newFile(new String((byte[]) map.get("file"), Constants.DEFAULT_ENCODING_CHARSET));
 
 		boolean recursive = ((Long) map.get("recursive")).longValue() == 1;
 
@@ -512,35 +479,6 @@ ShareResourceDirContentsImpl
 		{
 			return( new TorrentAttribute[0]);
 		}
-		
-		protected ShareResource 
-		lookupShare(byte[] torrent_hash)
-		{
-			for (int i=0;i<node_children.length;i++){
-
-				Object	o = node_children[i];
-
-				if ( o instanceof ShareResourceImpl ){
-
-					ShareResource res = ((ShareResourceImpl)o).lookupShare(torrent_hash);
-					
-					if ( res != null ){
-						
-						return( res );
-					}
-				}else{
-
-					ShareResource res = ((shareNode)o).lookupShare(torrent_hash);
-					
-					if ( res != null ){
-						
-						return( res );
-					}
-				}
-			}
-			
-			return( null );
-		}
 
 		@Override
 		public void
