@@ -234,6 +234,19 @@ RPDownload
 
 			return( null );
 
+		}else if ( method.equals( "stopAndRemove")){
+
+			try{
+				Object[] params = request.getParams();
+				delegate.stopAndRemove((Boolean) params[0], (Boolean) params[1]);
+
+			}catch( Throwable e ){
+
+				return( new RPReply(e));
+			}
+
+			return( null );
+
 		}else if ( method.equals( "setForceStart[boolean]")){
 
 			boolean	b = ((Boolean)request.getParams()[0]).booleanValue();
@@ -514,6 +527,33 @@ RPDownload
 		}
 	}
 
+	@Override
+	public void stopAndRemove(boolean delete_torrent, boolean delete_data)
+		throws DownloadException, DownloadRemovalVetoException
+	{
+		try{
+			_dispatcher.dispatch( new RPRequest( this, "stopAndRemove", new Object[] {
+				delete_torrent, delete_data
+			} )).getResponse();
+
+		}catch( RPException e ){
+
+			Throwable cause = e.getCause();
+
+			if ( cause instanceof DownloadException ){
+
+				throw((DownloadException)cause);
+			}
+
+			if ( cause instanceof DownloadRemovalVetoException ){
+
+				throw((DownloadRemovalVetoException)cause);
+			}
+
+			throw( e );
+		}
+	}
+
 	@Override
 	public void
 	remove(
@@ -959,10 +999,10 @@ RPDownload
 	}
 
   @Override
-  public int getSeedingRank() {
+  public SeedingRank getSeedingRank() {
 		notSupported();
 
-		return( 0 );
+		return( null );
   }
 
  	@Override
@@ -1065,7 +1105,7 @@ RPDownload
 		 * @see com.biglybt.pif.download.Download#setSeedingRank(int)
 		 */
 		@Override
-		public void setSeedingRank(int rank) {
+		public void setSeedingRank(SeedingRank rank) {
 			// TODO Auto-generated method stub
 
 		}
@@ -1206,4 +1246,4 @@ RPDownload
 	public DiskManagerFileInfo getPrimaryFile() {
 		return getDiskManagerFileInfo(0);
 	}
-}
\ No newline at end of file
+}
