@@ -34,11 +34,14 @@ import com.biglybt.core.dht.control.DHTControlActivity;
 import com.biglybt.core.dht.control.DHTControlContact;
 import com.biglybt.core.dht.nat.DHTNATPuncher;
 import com.biglybt.core.dht.router.DHTRouterContact;
+import com.biglybt.core.dht.transport.DHTTransportAlternativeContact;
+import com.biglybt.core.dht.transport.DHTTransportAlternativeNetwork;
 import com.biglybt.core.dht.transport.DHTTransportContact;
 import com.biglybt.core.dht.transport.DHTTransportFullStats;
 import com.biglybt.core.dht.transport.DHTTransportReplyHandlerAdapter;
 import com.biglybt.core.dht.transport.udp.DHTTransportUDP;
 import com.biglybt.core.dht.transport.udp.impl.DHTTransportUDPImpl;
+import com.biglybt.core.dht.transport.udp.impl.DHTUDPUtils;
 import com.biglybt.core.networkmanager.admin.NetworkAdmin;
 import com.biglybt.core.networkmanager.impl.udp.UDPNetworkManager;
 import com.biglybt.core.util.*;
@@ -384,6 +387,35 @@ DHTPlugin
 
 										return;
 
+									}else if ( lc.equals( "seed_test" )){
+										
+										for ( int net: DHTTransportAlternativeNetwork.AT_ALL_PUB ){
+										
+											log.log( "Network " + net );;
+											
+											List<DHTTransportAlternativeContact> contacts = 
+												DHTUDPUtils.getAlternativeContacts( net, 16 );
+											
+											for ( DHTTransportAlternativeContact contact: contacts ){
+												
+												log.log( "    " + contact.getProperties());
+											}
+										}
+									
+										for ( int net: DHTTransportAlternativeNetwork.AT_ALL_I2P){
+											
+											log.log( "Network " + net );;
+											
+											List<DHTTransportAlternativeContact> contacts = 
+												DHTUDPUtils.getAlternativeContacts( net, 16 );
+											
+											for ( DHTTransportAlternativeContact contact: contacts ){
+												
+												log.log( "    " + contact.getProperties());
+											}
+										}
+										
+										return;
 									}else if ( lc.equals( "bridge_put" )){
 
 										try{
@@ -998,6 +1030,8 @@ DHTPlugin
 
 													// adapter only added to first DHTPluginImpl we create
 
+												boolean hasV4 = NetworkAdmin.getSingleton().hasDHTIPV4();
+												
 												DHTPluginImplAdapter adapter =
 										        		new DHTPluginImplAdapter()
 										        		{
@@ -1013,7 +1047,7 @@ DHTPlugin
 										        			}
 										        		};
 
-												if ( AZ_MAIN_DHT_ENABLE ){
+												if ( AZ_MAIN_DHT_ENABLE && hasV4 ){
 
 													main_dht =
 														new DHTPluginImpl(
@@ -1035,32 +1069,29 @@ DHTPlugin
 													adapter = null;
 												}
 
-												if ( AZ_MAIN_DHT_V6_ENABLE ){
+												if ( AZ_MAIN_DHT_V6_ENABLE && NetworkAdmin.getSingleton().hasDHTIPV6() ){
 
-													if ( NetworkAdmin.getSingleton().hasDHTIPV6()){
+													main_v6_dht =
+														new DHTPluginImpl(
+															plugin_interface,
+															CoreFactory.getSingleton().getNATTraverser(),
+															adapter,
+															DHTTransportUDP.PROTOCOL_VERSION_AZ_MAIN,
+															DHT.NW_AZ_MAIN_V6,
+															true,
+															null,
+															dht_data_port,
+															reseed,
+															warn_user,
+															logging,
+															log, dht_log );
 
-														main_v6_dht =
-															new DHTPluginImpl(
-																plugin_interface,
-																CoreFactory.getSingleton().getNATTraverser(),
-																adapter,
-																DHTTransportUDP.PROTOCOL_VERSION_AZ_MAIN,
-																DHT.NW_AZ_MAIN_V6,
-																true,
-																null,
-																dht_data_port,
-																reseed,
-																warn_user,
-																logging,
-																log, dht_log );
+													plugins.add( main_v6_dht );
 
-														plugins.add( main_v6_dht );
-
-														adapter = null;
-													}
+													adapter = null;
 												}
-
-												if ( Constants.isCVSVersion() && AZ_CVS_DHT_ENABLE ){
+												
+												if ( Constants.isCVSVersion() && AZ_CVS_DHT_ENABLE && hasV4 ){
 
 													cvs_dht =
 														new DHTPluginImpl(
@@ -1082,7 +1113,7 @@ DHTPlugin
 													adapter = null;
 												}
 
-												if ( BIGLYBT_MAIN_DHT_ENABLE ){
+												if ( BIGLYBT_MAIN_DHT_ENABLE && hasV4  ){
 
 													biglybt_dht =
 														new DHTPluginImpl(
@@ -2067,30 +2098,33 @@ DHTPlugin
 
 			// try decode with dht[0] and fix up if IPv6 as the contact includes a reference to the underlying DHT
 		
-		DHTPluginContact result = dhts[0].importContact( map );
+		DHTPluginImpl dht0 = dhts[0];
+				
+		DHTPluginContact result = dht0.importContact( map );
 		
 		if ( result != null ){
 			
-			try{
-				if ( result.getAddress().getAddress() instanceof Inet6Address ){
-					
+			boolean want_ipv6 = result.getAddress().getAddress() instanceof Inet6Address;
+			
+			if ( want_ipv6 != dht0.isIPV6()){
+				
+				try{					
 					for ( DHTPluginImpl dht: dhts ){
-
-						if ( dht.isIPV6()){
-							
+	
+						if ( want_ipv6 == dht.isIPV6()){
+								
 							DHTPluginContact temp = dht.importContact( map );
-							
+									
 							if ( temp != null ){
-								
+										
 								result = temp;
-								
+										
 								break;
 							}
 						}
 					}
-					
+				}catch( Throwable e ){
 				}
-			}catch( Throwable e ){
 			}
 		}
 		
