@@ -127,6 +127,7 @@ MagnetPlugin
 
 	protected static final Object	DM_TAG_CACHE 		= new Object();
 	protected static final Object	DM_CATEGORY_CACHE 	= new Object();
+	protected static final Object	DM_DN_CHANGED	 	= new Object();
 	
 	private PluginInterface		plugin_interface;
 
@@ -134,7 +135,7 @@ MagnetPlugin
 
 	private boolean			first_download	= true;
 
-	private static final int	PLUGIN_DOWNLOAD_TIMEOUT_SECS_DEFAULT 	= 10*60;	// needs to be fairly large as non-public downloads can take a while...
+	private static final int	PLUGIN_DOWNLOAD_TIMEOUT_SECS_DEFAULT 	= 0;	// increased to infinite; (needs to be fairly large as non-public downloads can take a while...)
 
 	// private BooleanParameter 	secondary_lookup; removed
 	private BooleanParameter 	md_lookup;
@@ -663,7 +664,7 @@ MagnetPlugin
 						Debug.printStackTrace(e);
 					}
 
-					return( recoverableDownload( muh_listener, hash, args, sources, Collections.emptyList(), Collections.emptyMap(), timeout, false ));
+					return( recoverableDownload( muh_listener, hash, args, sources, Collections.emptyList(), Collections.emptyMap(), timeout, SystemTime.getCurrentTime(), false ));
 				}
 
 				@Override
@@ -785,13 +786,7 @@ MagnetPlugin
 							for ( Download download: to_delete ){
 	
 								try{
-									download.stop();
-	
-								}catch( Throwable e ){
-								}
-	
-								try{
-									download.remove( true, true );
+									download.stopAndRemove( true, true );
 	
 								}catch( Throwable e ){
 	
@@ -930,7 +925,34 @@ MagnetPlugin
 			
 			ThreadPool tp = new ThreadPool( "Magnet Recovery", conc, true );
 			
-			for ( Map map: active.values()){
+			List<Map> sorted = new ArrayList<Map>( active.values());
+			
+			Collections.sort(
+				sorted,
+				(m1, m2)->{
+					Long t1 = (Long)m1.get("added" );
+					Long t2 = (Long)m2.get("added" );
+					
+					if ( t1 == t2 ){
+						return(0);
+					}else if ( t1 == null ){
+						return( -1 );
+					}else if ( t2 == null ){
+						return( 1 );
+					}else{
+						long diff = t1 - t2;
+						
+						if ( diff < 0 ){
+							return( -1 );
+						}else if ( diff == 0 ){
+							return( 0 );
+						}else{
+							return( 1 );
+						}
+					}
+				});
+			
+			for ( Map map: sorted ){
 					
 				//System.out.println( "Recovering: " + map );
 					
@@ -987,6 +1009,8 @@ MagnetPlugin
 										
 					long timeout = ((Number)map.get( "timeout" )).longValue();
 					
+					Long added_time = (Long)map.get( "added" );
+										
 					final InetSocketAddress[] f_sources = sources;
 					
 					tp.run(
@@ -996,7 +1020,7 @@ MagnetPlugin
 							runSupport()
 							{
 								try{
-									byte[] result = recoverableDownload( null, hash, args, f_sources, f_tags, other_metadata, timeout, true );
+									byte[] result = recoverableDownload( null, hash, args, f_sources, f_tags, other_metadata, timeout, added_time, true );
 									
 									if ( result != null ){
 										
@@ -1066,7 +1090,13 @@ MagnetPlugin
 								}
 							}
 						});
-														
+					
+						// we want the metadata downloads to be added in the correct order. as the thread pool
+						// will fire things off concurrently we add this hack to give things a chance to 
+						// end up correct...
+					
+					Thread.sleep(500);
+					
 				}catch( Throwable e ){
 					
 					Debug.out( e );
@@ -1134,6 +1164,7 @@ MagnetPlugin
 		List<String>								tags,
 		Map<String,Object>							other_metadata,
 		final long									timeout,
+		Long										added_time,
 		boolean										is_recovering )
 	
 		throws MagnetURIHandlerException
@@ -1152,6 +1183,11 @@ MagnetPlugin
 				map.put( "hash", hash );
 				map.put( "args", args );
 				
+				if ( added_time != null ){
+					
+					map.put( "added", added_time );
+				}
+				
 				List<Map> l_sources = new ArrayList<>();
 				
 				map.put( "sources", l_sources );
@@ -1404,7 +1440,7 @@ MagnetPlugin
 			
 			tags = (List<String>)dm.getUserData( DM_TAG_CACHE );
 			
-			other_metadata = TorrentUtils.getInitialMetadata( dm );
+			other_metadata = getInitialMetadata( dm );
 			
 			String category = (String)dm.getUserData( DM_CATEGORY_CACHE );
 			
@@ -1681,7 +1717,7 @@ MagnetPlugin
 			}
 		}
 		
-		Map<String,Object>	other_metadata = TorrentUtils.getInitialMetadata( from_dm );
+		Map<String,Object>	other_metadata = getInitialMetadata( from_dm );
 	
 		if ( !other_metadata.isEmpty()){
 		
@@ -1700,6 +1736,20 @@ MagnetPlugin
 		return( updated );
 	}
 	
+	protected void
+	setDNChanged(
+		DownloadManager		dm )
+	{
+		dm.setUserData( DM_DN_CHANGED, "" );
+	}
+	
+	protected Map<String,Object>
+	getInitialMetadata(
+		DownloadManager	dm )
+	{
+		return( TorrentUtils.getInitialMetadata( dm, dm.getUserData( DM_DN_CHANGED ) != null ));
+	}
+	
 	protected void
 	setInitialMetadata(
 		TOTorrent			torrent,
@@ -1714,7 +1764,7 @@ MagnetPlugin
 			TorrentUtils.setInitialTags( torrent, tag_names );
 		}
 		
-		Map<String,Object>	other_metadata = TorrentUtils.getInitialMetadata( from_dm );
+		Map<String,Object>	other_metadata = getInitialMetadata( from_dm );
 		
 		if ( !other_metadata.isEmpty()){
 			
@@ -1750,7 +1800,7 @@ MagnetPlugin
 		
 		if ( other_metadata != null && !other_metadata.isEmpty()){
 			
-			TorrentUtils.setInitialMetadata( to_dm, other_metadata );
+			TorrentUtils.setInitialMetadata( to_dm, other_metadata, true );
 		}
 	}
 	
