@@ -156,23 +156,28 @@ public class StartStopConfigModel
 		listFP.add(model.createGroup(null, firstPriorityType));
 
 		// row
-		String[] minQueueLabels = new String[55];
-		int[] minQueueValues = new int[55];
-		minQueueLabels[0] = "1:2 (" + 0.5 + ")";
-		minQueueValues[0] = 500;
-		minQueueLabels[1] = "3:4 (" + 0.75 + ")";
-		minQueueValues[1] = 750;
-		minQueueLabels[2] = "1:1";
-		minQueueValues[2] = 1000;
-		minQueueLabels[3] = "5:4 (" + 1.25 + ")";
-		minQueueValues[3] = 1250;
-		minQueueLabels[4] = "3:2 (" + 1.50 + ")";
-		minQueueValues[4] = 1500;
-		minQueueLabels[5] = "7:4 (" + 1.75 + ")";
-		minQueueValues[5] = 1750;
-		for (int i = 6; i < minQueueLabels.length; i++) {
-			minQueueLabels[i] = i - 4 + ":1";
-			minQueueValues[i] = (i - 4) * 1000;
+		String[] minQueueLabels = new String[57];
+		int[] minQueueValues = new int[57];
+		int mqpos = 0;
+		minQueueLabels[mqpos] = MessageText.getString("ConfigView.text.ignore");
+		minQueueValues[mqpos++] = 0;
+		minQueueLabels[mqpos] = "1:2 (" + 0.5 + ")";
+		minQueueValues[mqpos++] = 500;
+		minQueueLabels[mqpos] = "3:4 (" + 0.75 + ")";
+		minQueueValues[mqpos++] = 750;
+		minQueueLabels[mqpos] = "1:1";
+		minQueueValues[mqpos++] = 1000;
+		minQueueLabels[mqpos] = "5:4 (" + 1.25 + ")";
+		minQueueValues[mqpos++] = 1250;
+		minQueueLabels[mqpos] = "3:2 (" + 1.50 + ")";
+		minQueueValues[mqpos++] = 1500;
+		minQueueLabels[mqpos] = "7:4 (" + 1.75 + ")";
+		minQueueValues[mqpos++] = 1750;
+		minQueueLabels[mqpos] = "2:1";
+		minQueueValues[mqpos++] = 2000;
+		for (int i = mqpos; i < minQueueLabels.length; i++) {
+			minQueueLabels[i] = i - 5 + ":1";
+			minQueueValues[i] = (i - 5) * 1000;
 		}
 		listFP.add(
 				model.addIntListParameter2("StartStopManager_iFirstPriority_ShareRatio",
@@ -444,6 +449,17 @@ public class StartStopConfigModel
 		addDefaultedBooleanParam(model, "Use Super Seeding",
 				"ConfigView.label.userSuperSeeding");
 
+		addDefaultedBooleanParam(model, "Enable Light Seeding",
+				"ConfigView.label.enableLightSeeding");
+
+		model.addIntParameter2("Light Seed Slots Reserved",
+				"ConfigView.label.light.seeding.reserved.slots", 4, 0,
+				1024);
+		
+		model.addIntParameter2("Flexible Seed Slots",
+				"ConfigView.label.seeding.flexible.slots", 5, 0,
+				1024);
+
 		model.addBooleanParameter2("StartStopManager_bAutoReposition",
 				"ConfigView.label.seeding.autoReposition", false);
 
@@ -594,7 +610,8 @@ public class StartStopConfigModel
 		System.arraycopy(activeDLValues, 0, activeSeedingValues, 0,
 				activeSeedingValues.length);
 
-		model.addIntListParameter2("StartStopManager_iMinSpeedForActiveSeeding",
+		IntListParameter minSpeedForActiveSeeding = model.addIntListParameter2(
+				"StartStopManager_iMinSpeedForActiveSeeding",
 				"ConfigView.label.minSpeedForActiveSeeding", activeSeedingValues,
 				activeSeedingLabels, 512);
 
@@ -610,6 +627,15 @@ public class StartStopConfigModel
 				"ConfigView.label.maxStalledSeedingIgnoreZP", true);
 		maxStalledSeedingIgnoreZP.setIndent(1, true);
 
+		ParameterListener minSpeedForActiveSeedingListener = p -> {
+			boolean enabled = minSpeedForActiveSeeding.getValue() != 0;
+			maxStalledSeeding.setEnabled(enabled);
+			maxStalledSeedingIgnoreZP.setEnabled(enabled);
+		};
+		minSpeedForActiveSeeding.addListener(minSpeedForActiveSeedingListener);
+		minSpeedForActiveSeedingListener.parameterChanged(null);
+
+
 		model.addBooleanParameter2("StartStopManager_bStopOnceBandwidthMet",
 				"ConfigView.label.queue.stoponcebandwidthmet", true);
 
