@@ -19,7 +19,9 @@
 package com.biglybt.ui.common.table.impl;
 
 import java.util.*;
+import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
+import java.util.regex.PatternSyntaxException;
 import java.util.stream.Collectors;
 
 import com.biglybt.core.config.impl.ConfigurationManager;
@@ -29,7 +31,6 @@ import com.biglybt.core.logging.Logger;
 import com.biglybt.core.util.*;
 import com.biglybt.ui.common.table.*;
 import com.biglybt.ui.selectedcontent.SelectedContentManager;
-
 import com.biglybt.pif.ui.tables.TableColumn;
 import com.biglybt.pif.ui.tables.TableRow;
 import com.biglybt.pif.ui.tables.TableRowRefreshListener;
@@ -56,6 +57,36 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 	// Shorter name for ConfigManager, easier to read code
 	protected static final ConfigurationManager configMan = ConfigurationManager.getInstance();
 
+	private class
+	rowSorterRunnable
+		extends AERunnable
+	{
+		private AtomicBoolean	forceRefreshPending = new AtomicBoolean();
+
+		@Override
+		public void 
+		runSupport()
+		{
+			sortRows( forceRefreshPending.getAndSet( false ), false );
+		}
+		
+		public void 
+		setForceRefresh(
+			boolean fs)
+		{
+			if ( fs ){
+				
+				forceRefreshPending.set( true);
+			}
+		}
+	}
+
+	private rowSorterRunnable rowSorterRunnable = new rowSorterRunnable();
+	
+	private FrequencyLimitedDispatcher rowSorter = 
+		new FrequencyLimitedDispatcher(rowSorterRunnable, 250 );
+		
+			
 	/** TableID (from {@link com.biglybt.pif.ui.tables.TableManager})
 	 * of the table this class is
 	 * handling.  Config settings are stored with the prefix of
@@ -655,8 +686,16 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 
 			for (DATASOURCETYPE dst : unfilteredArray) {
 				boolean bHave = existing.contains(dst);
-				boolean isOurs = filter.checker.filterCheck(dst, filter.text,
-						filter.regex);
+				boolean isOurs;
+				
+				try{
+					isOurs = filter.checker.filterCheck(dst, filter.text,filter.regex);
+					
+				}catch( PatternSyntaxException e ){
+					// get this with malformed filter regex, ignore
+					isOurs = true;
+				}	
+				
 				if (!isOurs) {
 					if (bHave) {
 						listRemoves.add(dst);
@@ -697,7 +736,13 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 			return( true );
 		}
 
-		return( filter.checker.filterCheck( ds, filter.text, filter.regex ));
+		try{
+			return( filter.checker.filterCheck( ds, filter.text, filter.regex ));
+			
+		}catch( PatternSyntaxException e ){
+			// get this with malformed filter regex, ignore		
+			return( false );
+		}
 	}
 
 	protected abstract boolean
@@ -796,11 +841,15 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 					+ Debug.getCompressedStackTrace(4));
 		}
 
-		if (!skipFilterCheck && filter != null
-				&& !filter.checker.filterCheck(dataSource, filter.text, filter.regex)) {
-			return;
+		try{
+			if (!skipFilterCheck && filter != null
+					&& !filter.checker.filterCheck(dataSource, filter.text, filter.regex)) {
+				return;
+			}
+		}catch( PatternSyntaxException e ){
+			// get this with malformed filter regex, ignore
 		}
-
+		
 		if (DataSourceCallBackUtil.IMMEDIATE_ADDREMOVE_DELAY == 0) {
 			reallyAddDataSources(new Object[] {
 				dataSource
@@ -866,11 +915,15 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 
 		if (DataSourceCallBackUtil.IMMEDIATE_ADDREMOVE_DELAY == 0) {
 			if (!skipFilterCheck && filter != null) {
-				for (int i = 0; i < dataSources.length; i++) {
-					if (!filter.checker.filterCheck(dataSources[i], filter.text,
-							filter.regex)) {
-						dataSources[i] = null;
+				try{
+					for (int i = 0; i < dataSources.length; i++) {
+						if (!filter.checker.filterCheck(dataSources[i], filter.text,
+								filter.regex)) {
+							dataSources[i] = null;
+						}
 					}
+				}catch( PatternSyntaxException e ){
+					// get this with malformed filter regex, ignore
 				}
 			}
 			reallyAddDataSources(dataSources);
@@ -889,12 +942,18 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 				if (dataSource == null) {
 					continue;
 				}
-				if (!skipFilterCheck
-						&& filter != null
-						&& !filter.checker.filterCheck(dataSource, filter.text,
-								filter.regex)) {
-					continue;
+				
+				try{
+					if (!skipFilterCheck
+							&& filter != null
+							&& !filter.checker.filterCheck(dataSource, filter.text,
+									filter.regex)) {
+						continue;
+					}
+				}catch( PatternSyntaxException e ){
+					// get this with malformed filter regex, ignore
 				}
+				
 				dataSourcesToRemove.remove(dataSource); // may be pending removal, override
 
 				if (dataSourcesToAdd.containsKey(dataSource)) {
@@ -1387,6 +1446,8 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 
 		ArrayList<TableRowCore> itemsToRemove;
 
+		boolean	hadSelected;
+		
 		synchronized (rows_sync) {
 
 			itemsToRemove = new ArrayList<>(mapDataSourceToRow.values());
@@ -1398,6 +1459,8 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 
 			listUnfilteredDataSources.clear();
 
+			hadSelected = !selectedRows.isEmpty();
+			
 			selectedRows.clear();
 			listSelectedCoreDataSources = null;
 
@@ -1406,6 +1469,11 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 			}
 		}
 
+		if ( hadSelected ){
+			
+			triggerTabViewsDataSourceChanged();
+		}
+		
 			// parg - added this to ensure resources associated with rows (e.g. graphics) are released properly
 			// not sure if any of the other things that normally happen on row-removal are also required to happen here
 			// e.g. triggerListenerRowRemoved(item); and uiRemoveRows(...)
@@ -1496,6 +1564,8 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 			triggerSelectionChangeListeners( new TableRowCore[0], deselected );
 			
 			triggerDeselectionListeners( deselected );
+			
+			triggerTabViewsDataSourceChanged();
 		}
 
 		if (DEBUGADDREMOVE) {
@@ -1525,8 +1595,21 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 		_sortColumn(bForceDataRefresh, true, false);
 	}
 
-	public void sortRows(boolean bForceDataRefresh) {
-		_sortColumn(bForceDataRefresh, false, false);
+	public void 
+	sortRows(
+		boolean bForceDataRefresh, 
+		boolean async ) 
+	{
+		if ( async ){
+			
+			rowSorterRunnable.setForceRefresh(bForceDataRefresh);
+			
+			rowSorter.dispatch();
+			
+		}else{
+			
+			_sortColumn(bForceDataRefresh, false, false);
+		}
 	}
 
 	protected void _sortColumn(final boolean bForceDataRefresh, final boolean bFillGapsOnly,
@@ -1745,64 +1828,69 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 				}
 
 				synchronized (rows_sync) {
-  				try {
-  					int index = 0;
-  					if (sortedRows.size() > 0) {
-  						// If we are >= to the last item, then just add it to the end
-  						// instead of relying on binarySearch, which may return an item
-  						// in the middle that also is equal.
-  						TableRowCore lastRow = sortedRows.get(sortedRows.size() - 1);
-  						// todo: use multi-sort
-  						if (sortColumns.isEmpty() || sortColumns.get(0).compare(row, lastRow) >= 0) {
-  							index = sortedRows.size();
-  							sortedRows.add(row);
-  							if (DEBUGADDREMOVE) {
-  								debug("Adding new row to bottom");
-  							}
-  						} else {
-  							index = Collections.binarySearch(sortedRows, row, sortColumns.get(0));
-  							if (index < 0) {
-  								index = -1 * index - 1; // best guess
-  							}
-
-  							if (index > sortedRows.size()) {
-  								index = sortedRows.size();
-  							}
-
-  							if (DEBUGADDREMOVE) {
-  								debug("Adding new row at position " + index + " of "
-  										+ (sortedRows.size() - 1));
-  							}
-  							sortedRows.add(index, row);
-  						}
-  					} else {
-  						if (DEBUGADDREMOVE) {
-  							debug("Adding new row to bottom (1st Entry)");
-  						}
-  						index = sortedRows.size();
-  						sortedRows.add(row);
-  					}
-
-  					rowsAdded.add(row);
-
-  					// XXX Don't set table item here, it will mess up selected rows
-  					//     handling (which is handled in fillRowGaps called later on)
-  					//row.setTableItem(index);
-
-
-  					//row.setIconSize(ptIconSize);
-  				} catch (Exception e) {
-  					e.printStackTrace();
-  					Logger.log(new LogEvent(LOGID, "Error adding a row to table "
-  							+ getTableID(), e));
-  					try {
-  						if (!sortedRows.contains(row)) {
-  							sortedRows.add(row);
-  						}
-  					} catch (Exception e2) {
-  						Debug.out(e2);
-  					}
-  				}
+						// check that the row item hasn't been removed in the meantime while lock 
+						// not held
+					
+					if (row ==  mapDataSourceToRow.get( dataSource )){
+						try {
+							int index = 0;
+							if (sortedRows.size() > 0) {
+								// If we are >= to the last item, then just add it to the end
+								// instead of relying on binarySearch, which may return an item
+								// in the middle that also is equal.
+								TableRowCore lastRow = sortedRows.get(sortedRows.size() - 1);
+								// todo: use multi-sort
+								if (sortColumns.isEmpty() || sortColumns.get(0).compare(row, lastRow) >= 0) {
+									index = sortedRows.size();
+									sortedRows.add(row);
+									if (DEBUGADDREMOVE) {
+										debug("Adding new row to bottom");
+									}
+								} else {
+									index = Collections.binarySearch(sortedRows, row, sortColumns.get(0));
+									if (index < 0) {
+										index = -1 * index - 1; // best guess
+									}
+	
+									if (index > sortedRows.size()) {
+										index = sortedRows.size();
+									}
+	
+									if (DEBUGADDREMOVE) {
+										debug("Adding new row at position " + index + " of "
+												+ (sortedRows.size() - 1));
+									}
+									sortedRows.add(index, row);
+								}
+							} else {
+								if (DEBUGADDREMOVE) {
+									debug("Adding new row to bottom (1st Entry)");
+								}
+								index = sortedRows.size();
+								sortedRows.add(row);
+							}
+	
+							rowsAdded.add(row);
+	
+							// XXX Don't set table item here, it will mess up selected rows
+							//     handling (which is handled in fillRowGaps called later on)
+							//row.setTableItem(index);
+	
+	
+							//row.setIconSize(ptIconSize);
+						} catch (Exception e) {
+							e.printStackTrace();
+							Logger.log(new LogEvent(LOGID, "Error adding a row to table "
+									+ getTableID(), e));
+							try {
+								if (!sortedRows.contains(row)) {
+									sortedRows.add(row);
+								}
+							} catch (Exception e2) {
+								Debug.out(e2);
+							}
+						}
+					}
 				}
 			} // for dataSources
 
@@ -2311,7 +2399,7 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 			}
  			uiChangeColumnIndicator();
  			resetLastSortedOn();
- 			sortRows(columnsChanged);
+ 			sortRows(columnsChanged,false);
 			return columnsChanged;
 		}
 	}
@@ -2437,20 +2525,26 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 	protected void
 	reaffirmSelection()
 	{
-		List<TableRowCore> oldSelectionList = new ArrayList<>();
-
-		synchronized (rows_sync) {
-			if (selectedRows.size() == 0 ){
-				return;
+			// we don't want an inactive table to grab back selection when something else changes
+			// (e.g. a new row is added to it)
+		
+		if ( isTableFocused()){
+			
+			List<TableRowCore> oldSelectionList = new ArrayList<>();
+	
+			synchronized (rows_sync) {
+				if (selectedRows.size() == 0 ){
+					return;
+				}
+	
+				oldSelectionList.addAll(selectedRows);
 			}
-
-			oldSelectionList.addAll(selectedRows);
+			
+			TableRowCore[] rows = oldSelectionList.toArray(new TableRowCore[0]);
+			
+			triggerSelectionChangeListeners( rows, rows );
+			triggerSelectionListeners(rows);
 		}
-		
-		TableRowCore[] rows = oldSelectionList.toArray(new TableRowCore[0]);
-		
-		triggerSelectionChangeListeners( rows, rows );
-		triggerSelectionListeners(rows);
 	}
 
 	public abstract boolean isSingleSelection();
@@ -2499,7 +2593,11 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 		this.provideIndexesOnRemove = provideIndexesOnRemove;
 	}
 
+	@Override
 	public boolean isTableSelected() {
 		return SelectedContentManager.getCurrentlySelectedTableView() == this;
 	}
+	
+	protected abstract boolean
+	isTableFocused();
 }
