@@ -38,7 +38,6 @@ import com.biglybt.platform.PlatformManagerFactory;
 import com.biglybt.ui.UIFunctions;
 import com.biglybt.ui.UIFunctionsManager;
 import com.biglybt.ui.UIFunctionsUserPrompter;
-
 import com.biglybt.pif.platform.PlatformManagerException;
 import com.biglybt.pif.ui.UIInstance;
 import com.biglybt.pif.ui.config.ActionParameter;
@@ -200,6 +199,7 @@ public class ConfigSectionStartShutdown
 
 		BooleanParameterImpl enablePrompt = new BooleanParameterImpl(
 				BCFG_PROMPT_TO_ABORT_SHUTDOWN, "ConfigView.label.prompt.abort");
+		enablePrompt.setAllowedUiTypes(UIInstance.UIT_SWT);
 		add(enablePrompt, Parameter.MODE_INTERMEDIATE, listStop);
 
 		add("pgStop", new ParameterGroupImpl("ConfigView.label.stop", listStop));
@@ -209,6 +209,7 @@ public class ConfigSectionStartShutdown
 		IntParameterImpl paramRestartWhenIdle = new IntParameterImpl(
 				ICFG_AUTO_RESTART_WHEN_IDLE, "ConfigView.label.restart.auto", 0,
 				100000);
+		paramRestartWhenIdle.setSuffixLabelKey("ConfigView.text.minutes");
 		add(paramRestartWhenIdle);
 		paramRestartWhenIdle.setMinimumRequiredUserMode(
 				Parameter.MODE_INTERMEDIATE);
@@ -216,11 +217,12 @@ public class ConfigSectionStartShutdown
 		add("pgRestart",
 				new ParameterGroupImpl("label.restart", paramRestartWhenIdle));
 
+
+		List<Parameter> listJVM = new ArrayList<>();
+
 		if (platform.hasCapability(
 				PlatformManagerCapabilities.AccessExplicitVMOptions)) {
 
-			List<Parameter> listJVM = new ArrayList<>();
-
 			// wiki link
 
 			add(new HyperlinkParameterImpl("ConfigView.label.please.visit.here",
@@ -272,9 +274,42 @@ public class ConfigSectionStartShutdown
 					Debug.getNestedExceptionMessage(e)
 				}));
 			}
-
-			add("pgJVM", new ParameterGroupImpl("ConfigView.label.jvm", listJVM));
 		}
+		
+		ActionParameterImpl history_button = new ActionParameterImpl("jvm.mem.history",
+				"label.view");
+		history_button.setAllowedUiTypes(UIInstance.UIT_SWT);
+		add(history_button, Parameter.MODE_INTERMEDIATE, listJVM);
+
+		history_button.addListener(param -> {
+			try{
+
+				UIFunctions uif = UIFunctionsManager.getUIFunctions();
+				
+				if ( uif == null ){
+					
+					Debug.out( "No UI Functions!" );
+				}else{
+					
+					List<String>	lines = AEJavaManagement.getMemoryHistory();
+					
+					String content = "";
+					
+					for ( String line: lines ){
+						
+						content += line + "\n";
+					}
+					
+					uif.showText( MessageText.getString( "GeneralView.section.info" ), content );
+				}
+			
+			} catch (Throwable e) {
+
+				Debug.out(e);
+			}
+		});
+
+		add("pgJVM", new ParameterGroupImpl("ConfigView.label.jvm", listJVM));
 	}
 
 	private void buildOptions(final PlatformManager platform,
@@ -333,7 +368,7 @@ public class ConfigSectionStartShutdown
 	
 						long min_mem = AEJavaManagement.getJVMLongOption(cur_options, "-Xms");
 	
-						if (min_mem == -1 || min_mem > cur_max_mem) {
+						if ( min_mem > cur_max_mem) {
 	
 							cur_options = AEJavaManagement.setJVMLongOption(cur_options, "-Xms", cur_max_mem);
 						}
@@ -574,9 +609,16 @@ public class ConfigSectionStartShutdown
 			return (val + " MB");
 		}
 
-		val = val / 1024;
-
-		return (val + " GB");
+		if ( val %1024 == 0 ){
+			
+			val = val / 1024;
+	
+			return (val + " GB");
+			
+		}else{
+			
+			return (val + " MB");
+		}
 	}
 
 	private static long 
