@@ -1042,9 +1042,7 @@ WebPlugin
 		PairingManager pm = PairingManagerFactory.getSingleton();
 
 		URL url = pm.getServiceURL();
-		
-		url = UrlUtils.setProtocol( url, "http" );
-	
+			
 		String res = url.toExternalForm() + "/web/test?sid=" + sid;
 
 		if ( pm.isEnabled()){
@@ -1213,6 +1211,8 @@ WebPlugin
 
 			welcome_files[i] = FileUtil.newFile( file_root, welcome_pages[i] );
 		}
+		
+		initStage(2);
 	}
 
 	private void
@@ -1435,6 +1435,13 @@ WebPlugin
 
 							options.put( AEProxyFactory.SP_PORT, server_port );
 
+							InetAddress bind = NetworkAdmin.getSingleton().getSingleHomedServiceBindAddress();
+							
+							if ( bind != null && !bind.isAnyLocalAddress()){
+								
+								options.put( AEProxyFactory.SP_BIND, bind.getHostAddress());
+							}
+
 							Map<String,Object> reply =
 									AEProxyFactory.getPluginServerProxy(
 										plugin_interface.getPluginName(),
@@ -1477,6 +1484,13 @@ WebPlugin
 
 							options.put( AEProxyFactory.SP_PORT, server_port );
 
+							InetAddress bind = NetworkAdmin.getSingleton().getSingleHomedServiceBindAddress();
+							
+							if ( bind != null && !bind.isAnyLocalAddress()){
+								
+								options.put( AEProxyFactory.SP_BIND, bind.getHostAddress());
+							}
+
 							Map<String,Object> reply =
 									AEProxyFactory.getPluginServerProxy(
 										plugin_interface.getPluginName(),
@@ -2321,6 +2335,25 @@ WebPlugin
 		return( url.getPort()==-1?url.getDefaultPort():url.getPort());
 	}
 
+	protected String
+	getServerURL()
+	{
+		InetAddress bind_ip = getServerBindIP();
+
+		InetAddress address;
+		
+		if ( bind_ip.isAnyLocalAddress()){
+
+			address = NetworkAdmin.getSingleton().getLoopbackAddress();
+			
+		}else{
+
+			address = bind_ip;
+		}
+		
+		return( getProtocol().toLowerCase( Locale.US ) + "://" + UrlUtils.getURLForm( address, getPort()) + "/" );
+	}
+	
 	public int
 	getPort()
 	{
