@@ -400,7 +400,7 @@ public class TorrentMenuFancy
 
 	}
 
-	public void showMenu(TableColumnCore acolumn, final Menu fallbackMenu ) {
+	public void showMenu(Point locationOnDiplay, TableColumnCore acolumn, final Menu fallbackMenu ) {
 		this.column = acolumn;
 		Display d = parentShell.getDisplay();
 
@@ -509,9 +509,7 @@ public class TorrentMenuFancy
 			addHeader(HEADER_OTHER, HEADER_MSG_PREFIX + HEADER_OTHER, null);
 		}
 
-		originalShellLocation = d.getCursorLocation();
-		originalShellLocation.x -= 5;
-		originalShellLocation.y -= 16;
+		originalShellLocation = locationOnDiplay;
 
 		shell.setLocation(originalShellLocation);
 
@@ -870,9 +868,11 @@ public class TorrentMenuFancy
 		createActionButton(dms, cQuickCommands, "MyTorrentsView.menu.queue",
 				"start", start,
 				(ListenerGetOffSWT) event -> TorrentUtil.queueDataSources(dms, true));
-
+		
 		// Force Start
-		if (userMode > 0) {
+		
+		if ( TorrentUtil.isForceStartVisible( dms )) {
+			
 			boolean forceStart = false;
 			boolean forceStartEnabled = false;
 
@@ -1269,6 +1269,7 @@ public class TorrentMenuFancy
 							boolean	hasClearableLinks = false;
 							boolean hasRevertableFiles	= false;
 							boolean lrrecheck			= false;
+							boolean allAllocatable		= true;
 							
 							for (DownloadManager dm : dms) {
 								boolean stopped = ManagerUtils.isStopped(dm);
@@ -1306,7 +1307,8 @@ public class TorrentMenuFancy
 								}
 								
 								lrrecheck = lrrecheck || ManagerUtils.canLowResourceRecheck(dm);
-										
+																
+								allAllocatable &= stopped && !dm.isDataAlreadyAllocated() && !dm.isDownloadComplete( false );		
 							}
 
 							boolean fileRescan = allScanSelected || allScanNotSelected;
@@ -1388,6 +1390,27 @@ public class TorrentMenuFancy
 							});
 
 							itemClearLinks.setEnabled(hasClearableLinks);
+							
+								// allocate
+
+							MenuItem itemFileAlloc = new MenuItem(menu, SWT.PUSH);
+							Messages.setLanguageText(itemFileAlloc,
+									"label.allocate");
+							itemFileAlloc.addListener(SWT.Selection, new ListenerDMTask(
+									dms) {
+								@Override
+								public void run(DownloadManager dm) {
+									
+									dm.getDownloadState().setLongAttribute( DownloadManagerState.AT_FILE_ALLOC_STRATEGY, DownloadManagerState.FAS_ZERO_NEW_STOP );
+									
+									dm.getDownloadState().setFlag( DownloadManagerState.FLAG_DISABLE_STOP_AFTER_ALLOC, false );
+									
+									ManagerUtils.queue( dm, null );
+								}
+							});
+	
+							itemFileAlloc.setEnabled(allAllocatable);
+					
 								// clear allocation
 
 							MenuItem itemFileClearAlloc = new MenuItem(menu, SWT.PUSH);
@@ -2422,9 +2445,21 @@ public class TorrentMenuFancy
 					new ListenerDMTask(dms) {
 						@Override
 						public void run(DownloadManager[] dms) {
-							TorrentUtil.moveDataFiles(parentShell, dms);
+							TorrentUtil.moveDataFiles(parentShell, dms,false);
 						}
 					});
+			
+			int userMode = COConfigurationManager.getIntParameter("User Mode");
+			
+			if ( userMode > 0 ){
+				createRow(detailArea, "MyTorrentsView.menu.movedata.batch", null,
+						new ListenerDMTask(dms) {
+							@Override
+							public void run(DownloadManager[] dms) {
+								TorrentUtil.moveDataFiles(parentShell, dms, true);
+							}
+						});
+			}
 		}
 		
 		if ( canSetMOC || canClearMOC ){
