@@ -37,6 +37,7 @@ import com.biglybt.activities.ActivitiesConstants;
 import com.biglybt.activities.ActivitiesEntry;
 import com.biglybt.activities.ActivitiesListener;
 import com.biglybt.activities.ActivitiesManager;
+import com.biglybt.core.CoreFactory;
 import com.biglybt.core.config.COConfigurationManager;
 import com.biglybt.core.disk.*;
 import com.biglybt.core.download.DownloadManager;
@@ -45,6 +46,8 @@ import com.biglybt.core.download.DownloadManagerState;
 import com.biglybt.core.download.impl.DownloadManagerAdapter;
 import com.biglybt.core.global.GlobalManager;
 import com.biglybt.core.global.GlobalManagerAdapter;
+import com.biglybt.core.global.GlobalManagerEvent;
+import com.biglybt.core.global.GlobalManagerEventListener;
 import com.biglybt.core.internat.MessageText;
 import com.biglybt.core.logging.LogAlert;
 import com.biglybt.core.logging.Logger;
@@ -52,7 +55,11 @@ import com.biglybt.core.util.*;
 import com.biglybt.platform.PlatformManager;
 import com.biglybt.platform.PlatformManagerCapabilities;
 import com.biglybt.platform.PlatformManagerFactory;
+import com.biglybt.pif.PluginInterface;
+import com.biglybt.pif.logging.LoggerChannel;
 import com.biglybt.pif.platform.PlatformManagerException;
+import com.biglybt.pif.ui.UIManager;
+import com.biglybt.pif.ui.model.BasicPluginViewModel;
 import com.biglybt.ui.swt.minibar.DownloadBar;
 
 import com.biglybt.ui.UIFunctions;
@@ -72,6 +79,7 @@ UserAlerts
 	private final DiskManagerListener disk_listener;
 	private final DownloadManagerDiskListener dm_disk_listener;
 	private final GlobalManagerAdapter globalManagerListener;
+	private final GlobalManagerEventListener globalManagerEventListener;
 	private final ActivitiesListener activitiesListener;
 	private final GlobalManager global_manager;
 
@@ -98,13 +106,29 @@ UserAlerts
     private TrayIcon	native_tray_icon;
     private int			native_message_count;
     
+    private final LoggerChannel log;
+    
 	public
 	UserAlerts(
 		GlobalManager	global_manager )
  	{
 	  this.global_manager = global_manager;
 	  singleton = this;
+	  
+	  PluginInterface plugin_interface = CoreFactory.getSingleton().getPluginManager().getDefaultPluginInterface();
 
+	  log = plugin_interface.getLogger().getChannel( "Alerts" );
+
+	  UIManager	ui_manager = plugin_interface.getUIManager();
+
+	  BasicPluginViewModel model = ui_manager.createBasicPluginViewModel("ConfigView.section.interface.alerts" );
+
+	  model.getActivity().setVisible( false );
+	  model.getProgress().setVisible( false );
+
+	  model.attachLoggerChannel( log );
+
+		
 	  // @see com.biglybt.core.download.impl.DownloadManagerAdapter#stateChanged(com.biglybt.core.download.DownloadManager, int)
 // if state == STARTED, then open the details window (according to config)
 	  download_manager_listener = new DownloadManagerAdapter()
@@ -283,6 +307,21 @@ UserAlerts
 	  };
 	  global_manager.addListener(globalManagerListener);
 
+	  globalManagerEventListener = new GlobalManagerEventListener(){
+		
+		@Override
+		public void 
+		eventOccurred(
+			GlobalManagerEvent event)
+		{
+			if ( event.getEventType() == GlobalManagerEvent.ET_RECHECK_COMPLETE ){
+				checkComplete( event.getDownload(), (Boolean)event.getEventData());
+			}
+		}
+	  };
+	  
+	  global_manager.addEventListener( globalManagerEventListener );
+	  
 	  activitiesListener = new ActivitiesListener() {
 
 		  @Override
@@ -400,6 +439,28 @@ UserAlerts
   			do_sound, sound_file );
   	}
 
+ 	private void
+  	checkComplete(
+  		DownloadManager			manager,
+  		boolean					explicit )
+  	{
+  		DownloadManagerState dm_state = manager.getDownloadState();
+
+		if ( dm_state.getFlag( DownloadManagerState.FLAG_LOW_NOISE) && !explicit ) {
+
+			return;
+		}
+		
+    	boolean do_popup 	= 	COConfigurationManager.getBooleanParameter( "Popup Check Complete" );
+
+		doStuff(
+	  			manager, manager.getDisplayName(),
+	  			do_popup, "popup.check.complete", false, 
+	  			false, null,
+	  			false, null,
+	  			false, null );
+  	}
+ 	
   	private long	last_error_speech;
   	private long	last_error_sound;
 
@@ -593,8 +654,30 @@ UserAlerts
 			
 			if ( do_native_tray ){
 				
-				if ( native_tray_icon == null ){
-					
+				displayNativeMessage( MessageText.getString( native_text ), item_name, popup_is_error );
+			}
+  		}catch( Throwable e ){
+
+  			Debug.printStackTrace( e );
+
+  		}finally{
+
+  			this_mon.exit();
+  		}
+  	}
+
+  	public boolean
+  	displayNativeMessage(
+  		String		caption,
+  		String		text,
+  		boolean		is_error )
+  	{
+  		try{ 		
+  			this_mon.enter();
+  			
+			if ( native_tray_icon == null ){
+				
+				try{
 					if ( SystemTray.isSupported()){
 						
 						SystemTray st = SystemTray.getSystemTray();
@@ -647,60 +730,63 @@ UserAlerts
 						
 				        st.add( native_tray_icon );
 					}
-				}
-			
-				if ( native_tray_icon != null ){
-				        					
-					native_tray_icon.displayMessage( MessageText.getString( native_text ), item_name, popup_is_error?MessageType.ERROR:MessageType.INFO );
+				}catch( Throwable e ){
 					
-					/* Unfortunately removing the icon also removes any messages associated with it
-					 * that may be in the notification area (on Windows 10 for example)
-					 
-					final int mine = ++native_message_count;
-					
-					SimpleTimer.addEvent(
-						"iconhider",
-						SystemTime.getOffsetTime( 30*1000 ),
-						new TimerEventPerformer(){
-							
-							@Override
-							public void perform(TimerEvent event){
-								try{
-									this_mon.enter();
-									
-									if ( native_message_count == mine ){
-										
-										try{
-											SystemTray st = SystemTray.getSystemTray();
-										
-											st.remove( native_tray_icon );
-											
-											native_tray_icon = null;
-											
-										}catch( Throwable e ){										
-										}
-									}
-									
-								}finally{
-									
-									this_mon.exit();
-								}
-							}
-						});
-					*/
+					Debug.out( e );
 				}
 			}
-  		}catch( Throwable e ){
-
-  			Debug.printStackTrace( e );
-
+		
+			if ( native_tray_icon != null ){
+			        					
+				native_tray_icon.displayMessage( caption, text, is_error?MessageType.ERROR:MessageType.INFO );
+				
+				/* Unfortunately removing the icon also removes any messages associated with it
+				 * that may be in the notification area (on Windows 10 for example)
+				 
+				final int mine = ++native_message_count;
+				
+				SimpleTimer.addEvent(
+					"iconhider",
+					SystemTime.getOffsetTime( 30*1000 ),
+					new TimerEventPerformer(){
+						
+						@Override
+						public void perform(TimerEvent event){
+							try{
+								this_mon.enter();
+								
+								if ( native_message_count == mine ){
+									
+									try{
+										SystemTray st = SystemTray.getSystemTray();
+									
+										st.remove( native_tray_icon );
+										
+										native_tray_icon = null;
+										
+									}catch( Throwable e ){										
+									}
+								}
+								
+							}finally{
+								
+								this_mon.exit();
+							}
+						}
+					});
+				*/
+				
+				return( true );
+			}
+			
+			return( false );
+			
   		}finally{
-
+  			
   			this_mon.exit();
   		}
   	}
 
-
   	private boolean
   	isDLFEnabled(
   		Map		map,
@@ -722,6 +808,16 @@ UserAlerts
 		final int iconID, final String title, final String text, final String details,
 		final Object[] relatedObjects, final int timeoutSecs)
 	{
+		if ( title != null && !title.isEmpty()){
+			log.log( title );
+		}
+		if ( text != null && !text.isEmpty()){
+			log.log( text );
+		}
+		if ( details != null && !details.isEmpty()){
+			log.log( details );
+		}
+		
 		UIFunctionsManager.execWithUIFunctions(
 			new UIFunctionsManager.UIFCallback() {
 
@@ -797,26 +893,26 @@ UserAlerts
   		}
   	}
 
-	public void dispose() {
-    if (singleton == this) {
-      singleton = null;
-	  }
+  	public void dispose() {
+  		if (singleton == this) {
+  			singleton = null;
+  		}
 
-		if (globalManagerListener != null) {
-    	global_manager.removeListener(globalManagerListener);
+  		global_manager.removeListener(globalManagerListener);
+  		global_manager.removeEventListener(globalManagerEventListener);
+  		
+  		List<DownloadManager> dms = global_manager.getDownloadManagers();
+  		for (DownloadManager dm : dms) {
+  			dm.removeListener(download_manager_listener);
+  			dm.removeDiskListener(dm_disk_listener);
 
-			List<DownloadManager> dms = global_manager.getDownloadManagers();
-			for (DownloadManager dm : dms) {
-				dm.removeListener(download_manager_listener);
-				dm.removeDiskListener(dm_disk_listener);
+  			DiskManager diskManager = dm.getDiskManager();
+  			if (diskManager != null) {
+  				diskManager.removeListener(disk_listener);
+  			}
+  		}
 
-				DiskManager diskManager = dm.getDiskManager();
-				if (diskManager != null) {
-					diskManager.removeListener(disk_listener);
-				}
-			}
-		}
 
-		ActivitiesManager.removeListener(activitiesListener);
-	}
+  		ActivitiesManager.removeListener(activitiesListener);
+  	}
 }
\ No newline at end of file
