@@ -847,6 +847,10 @@ public class MainStatusBar
 					return;
 				}
 
+				if ( current_alert != null && current_added && current_alert.isNative ){
+					return;
+				}
+				
 				ArrayList<LogAlert> alerts = Alerts.getUnviewedLogAlerts();
 				int count = alerts.size();
 
@@ -1141,9 +1145,7 @@ public class MainStatusBar
 	 * @since 4.0.0.1
 	 */
 	private void addStatusBarMenu(Composite cSB) {
-		if (!Constants.isCVSVersion()) {
-			return;
-		}
+
 		Menu menu = new Menu(cSB);
 		cSB.setMenu(menu);
 
