@@ -44,9 +44,11 @@ import com.biglybt.ui.swt.MenuBuildUtils;
 import com.biglybt.ui.swt.Utils;
 import com.biglybt.ui.swt.pif.*;
 import com.biglybt.ui.swt.pifimpl.UISWTViewBuilderCore;
+import com.biglybt.ui.swt.pifimpl.UISWTInstanceImpl.SWTViewListener;
 import com.biglybt.ui.swt.skin.SWTSkinObject;
 import com.biglybt.ui.swt.views.ViewManagerSWT;
 import com.biglybt.ui.swt.views.skin.SkinView;
+import com.biglybt.util.DataSourceUtils;
 import com.biglybt.util.MapUtils;
 
 import com.biglybt.pif.PluginInterface;
@@ -107,6 +109,8 @@ public abstract class BaseMDI
 
 	private String closeableConfigFile = null;
 
+	private SWTViewListener swtViewListener;
+
 	private volatile boolean 	initialized;
 	private volatile boolean	closed;
 
@@ -298,6 +302,24 @@ public abstract class BaseMDI
 		}
 	}
 
+	@Override
+	public MdiEntry 
+	getEntry(
+		String	id, Object datasource )
+	{
+		if ( datasource != null ){
+			
+			String hash = DataSourceUtils.getHash( datasource );
+	
+			if ( hash != null ){
+			
+				return( getEntry( id + "_" + hash ));
+			}
+		}
+		
+		return( getEntry( id ));
+	}
+	
 	/**
 	 * @param skinView
 	 * @return
@@ -524,52 +546,87 @@ public abstract class BaseMDI
 		return loadEntryByID(id, true, false, datasource);
 	}
 	
+	public boolean popoutEntryByID(String id, Object datasource, boolean onTop ) {
+		
+		MdiEntry existing = getEntry( id, datasource );
+		
+		if ( existing != null ){
+			
+			return( popoutEntry( existing, onTop ));
+		}
+		
+		MdiEntry entry = loadAndGetEntryByID( id, false, false, datasource );
+		
+		if ( entry != null ){
+				
+			try{
+				return( popoutEntry( entry, onTop ));
+				
+			}finally{
+				
+				entry.closeView();
+			}
+		}
+		
+		return( false );
+	}
+	
+	public abstract boolean
+	popoutEntry(
+		MdiEntry	entry,
+		boolean		onTop );
+	
 	@Override
 	public Object skinObjectInitialShow(SWTSkinObject skinObject, Object params) {
-		final UIManager ui_manager = PluginInitializer.getDefaultInterface().getUIManager();
-		ui_manager.addUIListener(new UIManagerListener() {
-			@Override
-			public void UIDetached(UIInstance instance) {
-			}
-
-			@Override
-			public void UIAttached(UIInstance instance) {
-				if (instance instanceof UISWTInstance) {
-					ui_manager.removeUIListener(this);
-
-					final AESemaphore wait_sem = new AESemaphore( "SideBar:wait" );
-
-					Utils.execSWTThread(new AERunnable() {
-						@Override
-						public void runSupport() {
-							try{
-								try {
-									loadCloseables();
-								} catch (Throwable t) {
-									Debug.out(t);
+		
+		if ( closeableConfigFile != null ){
+			
+			final UIManager ui_manager = PluginInitializer.getDefaultInterface().getUIManager();
+			
+			ui_manager.addUIListener(new UIManagerListener() {
+				@Override
+				public void UIDetached(UIInstance instance) {
+				}
+	
+				@Override
+				public void UIAttached(UIInstance instance) {
+					if (instance instanceof UISWTInstance) {
+						ui_manager.removeUIListener(this);
+	
+						final AESemaphore wait_sem = new AESemaphore( "SideBar:wait" );
+	
+						Utils.execSWTThread(new AERunnable() {
+							@Override
+							public void runSupport() {
+								try{
+									try {
+										loadCloseables();
+									} catch (Throwable t) {
+										Debug.out(t);
+									}
+	
+									setupPluginViews();
+	
+								}finally{
+	
+									initialized = true;
+									
+									wait_sem.release();
 								}
-
-								setupPluginViews();
-
-							}finally{
-
-								initialized = true;
-								
-								wait_sem.release();
 							}
+						});
+	
+							// we need to wait for the loadCloseables to complete as there is code in MainMDISetup that runs on the 'UIAttachedComplete'
+							// callback that needs the closables to be loaded (when setting 'start tab') otherwise the order gets broken
+	
+						if ( !wait_sem.reserve(10*1000)){
+	
+							Debug.out( "eh?");
 						}
-					});
-
-						// we need to wait for the loadCloseables to complete as there is code in MainMDISetup that runs on the 'UIAttachedComplete'
-						// callback that needs the closables to be loaded (when setting 'start tab') otherwise the order gets broken
-
-					if ( !wait_sem.reserve(10*1000)){
-
-						Debug.out( "eh?");
 					}
 				}
-			}
-		});
+			});
+		}
 
 		return null;
 	}
@@ -591,6 +648,11 @@ public abstract class BaseMDI
 			}
 		}
 
+		if ( swtViewListener != null ){
+		
+			ViewManagerSWT.getInstance().removeSWTViewListener( swtViewListener );
+		}
+		
 		return null;
 	}
 
@@ -614,14 +676,26 @@ public abstract class BaseMDI
 	@Override
 	public boolean loadEntryByID(String id, boolean activate,
 	                             boolean onlyLoadOnce, Object datasource) {
+		
+		return( loadAndGetEntryByID( id, activate, onlyLoadOnce, datasource ) != null );
+	}
+
+	private MdiEntry 
+	loadAndGetEntryByID(
+		String		id, 
+		boolean		activate,                    
+		boolean		onlyLoadOnce, 
+		Object		datasource ) 
+	{
+
 		if (id == null) {
-			return false;
+			return( null );
 		}
 
 		@SuppressWarnings("deprecation")
 		boolean loadedOnce = wasEntryLoadedOnce(id);
 		if (loadedOnce && onlyLoadOnce) {
-			return false;
+			return( null );
 		}
 
 		MdiEntry entry = getEntry(id);
@@ -632,7 +706,7 @@ public abstract class BaseMDI
 			if (activate) {
 				showEntry(entry);
 			}
-			return true;
+			return( entry );
 		}
 
 		Map autoOpenInfo = new HashMap();
@@ -663,10 +737,10 @@ public abstract class BaseMDI
 			if (activate) {
 				showEntry(mdiEntry);
 			}
-			return true;
+			return( mdiEntry );
 		}
 
-		return false;
+		return( null );
 	}
 
 	protected abstract void setEntryLoadedOnce(String id);
@@ -744,8 +818,8 @@ public abstract class BaseMDI
 
 		// When a new Plugin View is added, check out auto-open list to see if
 		// the user had it open
-		ViewManagerSWT vi = ViewManagerSWT.getInstance();
-		vi.addSWTViewListener((forDSTypeOrViewID, builder) -> {
+		
+		swtViewListener = (forDSTypeOrViewID, builder) -> {
 			if (forDSTypeOrViewID != null
 					&& !forDSTypeOrViewID.equals(UISWTInstance.VIEW_MAIN)) {
 				return;
@@ -760,10 +834,12 @@ public abstract class BaseMDI
 			if (o instanceof Map<?, ?>) {
 				processAutoOpenMap(builder.getViewID(), (Map<?, ?>) o, builder);
 			}
-		});
+		};
+		
+		ViewManagerSWT.getInstance().addSWTViewListener( swtViewListener );
 	}
 
-	public void loadCloseables() {
+	private void loadCloseables() {
 		if (closeableConfigFile == null) {
 			return;
 		}
@@ -840,7 +916,7 @@ public abstract class BaseMDI
 		}
 	}
 
-	public void saveCloseables(){
+	protected void saveCloseables(){
 		saveCloseables( false );
 	}
 	
