@@ -183,7 +183,13 @@ public abstract class BaseMdiEntry
 
 		if (mdi != null) {
 			MdiEntry entry = mdi.getEntry(id);
-			if (entry != null) {
+			
+			// it is possible the entry has been replaced in the meantime, if it
+			// has the tidyup for the old one will already have been done and we 
+			// don't want to go and remove the new replacement that is probably queued
+			// for addition in the SWT tubes
+			
+			if (entry == this ) {
 				mdi.closeEntryByID(id,userInitiated);
 				return;
 			}
@@ -193,25 +199,34 @@ public abstract class BaseMdiEntry
 	}
 	
 	protected void destroyEntry( boolean userInitiated ) {
-		triggerCloseListeners( userInitiated );
-
-		try {
-			setEventListener(null, null, false);
-		} catch (UISWTViewEventCancelledException ignore) {
+		try{
+			triggerCloseListeners( userInitiated );
+	
+			try {
+				setEventListener(null, null, false);
+			} catch (UISWTViewEventCancelledException ignore) {
+			}
+	
+			SWTSkinObject so = getSkinObject();
+			if (so != null) {
+				setSkinObjectMaster(null);
+				so.getSkin().removeSkinObject(so);
+			}
+	
+			// Fires off destroy event and destroys SWT widgets
+			super.closeView();
+			
+		}finally{
+			destroyEntryAlways();
 		}
-
-		ViewTitleInfoManager.removeListener(this);
-
-		SWTSkinObject so = getSkinObject();
-		if (so != null) {
-			setSkinObjectMaster(null);
-			so.getSkin().removeSkinObject(so);
-		}
-
-		// Fires off destroy event and destroys SWT widgets
-		super.closeView();
 	}
 
+	protected void destroyEntryAlways()
+	{
+			// we gotta always do this regardless - might be called > once
+		ViewTitleInfoManager.removeListener(this);		
+	}
+	
 	public Object getDatasourceCore() {
 		return datasource;
 	}
@@ -836,15 +851,23 @@ public abstract class BaseMdiEntry
 
 	// @see MdiEntry#setDefaultExpanded(boolean)
 	@Override
-	public void setDefaultExpanded(boolean defaultExpanded) {
-		COConfigurationManager.setBooleanDefault("SideBar.Expanded." + id,
-				defaultExpanded);
+	public void 
+	setDefaultExpanded(
+		boolean defaultExpanded) 
+	{
+		String configID = "SideBar.Expanded2." + Base32.encode( id.getBytes( Constants.UTF_8 ));
+		
+		COConfigurationManager.setBooleanDefault( configID, defaultExpanded );
 	}
 
 	@Override
-	public boolean isExpanded() {
+	public boolean 
+	isExpanded() 
+	{
+		String configID = "SideBar.Expanded2." + Base32.encode( id.getBytes( Constants.UTF_8 ));
+
 		return isExpanded == null
-				? COConfigurationManager.getBooleanParameter("SideBar.Expanded." + id)
+				? COConfigurationManager.getBooleanParameter(configID)
 				: isExpanded;
 			}
 
@@ -852,18 +875,24 @@ public abstract class BaseMdiEntry
 	 * @see MdiEntry#setExpanded(boolean)
 	 */
 	@Override
-	public void setExpanded(boolean expanded) {
+	public void 
+	setExpanded(boolean expanded) 
+	{
+		String configID = "SideBar.Expanded2." + Base32.encode( id.getBytes( Constants.UTF_8 ));
+
 		isExpanded = expanded;
+		
 		boolean defExpanded = true;
-		try {
+		
+		try{
 			defExpanded = ConfigurationDefaults.getInstance().getBooleanParameter(
-					"SideBar.Expanded." + id);
+					configID);
 		} catch (ConfigurationParameterNotFoundException e) {
 		}
 		if (isExpanded == defExpanded) {
-			COConfigurationManager.removeParameter("SideBar.Expanded." + id);
+			COConfigurationManager.removeParameter(configID);
 		} else {
-			COConfigurationManager.setParameter("SideBar.Expanded." + id, isExpanded);
+			COConfigurationManager.setParameter(configID, isExpanded);
 		}
 	}
 
