@@ -336,7 +336,7 @@ public class TabbedEntry
 
 					skinnedDialog.setTitle(	getTitle() + (ds_str.length() == 0 ? "" : (" - " + ds_str)));
 
-					skinnedDialog.open();
+					skinnedDialog.open(  "mdi.popout:" + getId(), true );
 
 				} else {
 
@@ -409,6 +409,7 @@ public class TabbedEntry
 		}
 
 		if (swtItem == null) {
+			destroyEntryAlways();
 			return;
 		}
 
@@ -515,18 +516,6 @@ public class TabbedEntry
 	public void setSelectable(boolean selectable) {
 	}
 
-	// @see MdiEntrySWT#addListener(MdiSWTMenuHackListener)
-	@Override
-	public void addListener(MdiSWTMenuHackListener l) {
-		// TODO Auto-generated method stub
-	}
-
-	// @see MdiEntrySWT#removeListener(MdiSWTMenuHackListener)
-	@Override
-	public void removeListener(MdiSWTMenuHackListener l) {
-		// TODO Auto-generated method stub
-	}
-
 	// @see BaseMdiEntry#setParentID(java.lang.String)
 	@Override
 	public void setParentEntryID(String parentEntryID) {
