@@ -50,6 +50,7 @@ import com.biglybt.core.util.*;
 import com.biglybt.ui.common.util.MenuItemManager;
 import com.biglybt.ui.common.viewtitleinfo.ViewTitleInfo;
 import com.biglybt.ui.mdi.MdiEntry;
+import com.biglybt.ui.mdi.MultipleDocumentInterface;
 import com.biglybt.ui.swt.MenuBuildUtils;
 import com.biglybt.ui.swt.UIFunctionsManagerSWT;
 import com.biglybt.ui.swt.MenuBuildUtils.MenuBuilder;
@@ -90,7 +91,7 @@ public class TabbedMDI
 	
 	private CTabFolder tabFolder;
 
-	private LinkedList<MdiEntry>	select_history = new LinkedList<>();
+	private LinkedList<TabbedEntry>	select_history = new LinkedList<>();
 
 	protected boolean minimized;
 
@@ -393,32 +394,14 @@ public class TabbedMDI
 				if ( entry != null ){
 					entry.setUserInitiatedClose();
 				}
-				if (select_history.remove(entry)) {
+				
+				TabbedEntry prev = getPrevious( entry );
 
-					if (select_history.size() > 0) {
-
-						final MdiEntry next = select_history.getLast();
-
-						if (!next.isEntryDisposed() && next != entry) {
-
-							// If tabfolder's selected entry is the one we are closing,
-							// CTabFolder will try to move to next CTabItem.  Disable
-							// this feature by moving tabfolder's selection away from us
-							CTabItem[] items = tabFolder.getItems();
-							for (int i = 0; i < items.length; i++) {
-								CTabItem item = items[i];
-								TabbedEntry scanEntry = getEntryFromTabItem(item);
-								if (scanEntry == next) {
-									tabFolder.setSelection(item);
-									break;
-								}
-							}
-
-							showEntry(next);
-						}
-					}
+				if ( prev != null ){
+					
+					showEntry( prev );
 				}
-
+				
 				// since showEntry is slightly delayed, we must slightly delay
 				// the closing of the entry the user clicked.  Otherwise, it would close
 				// first, and the first tab would auto-select (on windows), and then
@@ -981,21 +964,61 @@ public class TabbedMDI
 		COConfigurationManager.setParameter("tab.once." + id, true);
 	}
 
+	private TabbedEntry
+	getPrevious(
+		TabbedEntry		current )
+	{
+		if ( select_history.remove(current)){
+
+			if (select_history.size() > 0) {
+
+					// we can only do the crud below if we're on the SWT thread and this isn't
+					// always the case - fixing this properly is a bit of a job so hacking it atm
+					// (specifically unloading the mlDHT plugin in classic UI triggers this)
+				
+				if ( Utils.isSWTThread()){
+					
+					final TabbedEntry next = select_history.getLast();
+	
+					if (!next.isEntryDisposed() && next != current) {
+	
+						// If tabfolder's selected entry is the one we are closing,
+						// CTabFolder will try to move to next CTabItem.  Disable
+						// this feature by moving tabfolder's selection away from us
+						CTabItem[] items = tabFolder.getItems();
+						for (int i = 0; i < items.length; i++) {
+							CTabItem item = items[i];
+							TabbedEntry scanEntry = getEntryFromTabItem(item);
+							if (scanEntry == next) {
+								tabFolder.setSelection(item);
+								break;
+							}
+						}
+	
+						return( next );
+					}
+				}
+			}
+		}
+		
+		return( null );
+	}
+	
 	@Override
-	public void showEntry(final MdiEntry newEntry) {
-		if (newEntry == null) {
+	public void showEntry(final MdiEntry _newEntry) {
+		if (_newEntry == null) {
 			return;
 		}
 
-		if (newEntry != null) {
-  		select_history.remove( newEntry );
+		TabbedEntry newEntry = (TabbedEntry)_newEntry;
+		
+		select_history.remove( newEntry );
 
-  		select_history.add( newEntry );
+		select_history.add( newEntry );
 
-  		if ( select_history.size() > 64 ){
+		if ( select_history.size() > 64 ){
 
-  			select_history.removeFirst();
-  		}
+			select_history.removeFirst();
 		}
 
 		MdiEntry oldEntry = getCurrentEntry();
@@ -1106,6 +1129,27 @@ public class TabbedMDI
 		return entry;
 	}
 
+	protected BaseMdiEntry closeEntryByID(String id, boolean userInitiated) {
+
+		TabbedEntry existing = getEntry(id);
+
+		TabbedEntry prev  = getPrevious( existing );
+		
+		BaseMdiEntry entry = super.closeEntryByID(id, userInitiated);
+		
+		if (entry == null || Utils.isDisplayDisposed()) {
+			
+			return entry;
+		}
+		
+		if ( userInitiated && prev != null ){
+		
+			showEntry(prev);
+		}
+		
+		return( entry );
+	}
+	
 	@Override
 	public void updateUI() {
 		if (getMinimized() || !isVisible()) {
@@ -1616,36 +1660,8 @@ public class TabbedMDI
 					}else if ( data instanceof TabbedEntry ){
 						target = (TabbedEntry)data;
 					}
-					
-					SkinnedDialog skinnedDialog =
-							new SkinnedDialog(
-									"skin3_dlg_sidebar_popout",
-									"shell",
-									menu==menuItemOnTop?UIFunctionsManagerSWT.getUIFunctionsSWT().getMainShell():null,
-									SWT.RESIZE | SWT.MAX | SWT.DIALOG_TRIM);
 
-					SWTSkin skin = skinnedDialog.getSkin();
-
-					SWTSkinObjectContainer cont = target.buildStandAlone((SWTSkinObjectContainer)skin.getSkinObject( "content-area" ));
-
-					if ( cont != null ){
-
-						String ds_str = "";
-						Object ds = target.getDatasourceCore();
-						DownloadManager dm = DataSourceUtils.getDM(ds);
-
-						if (dm != null) {
-							ds_str = dm.getDisplayName();
-						}
-
-						skinnedDialog.setTitle( target.getTitle() + (ds_str.length()==0?"":(" - " + ds_str )));
-
-						skinnedDialog.open();
-
-					}else{
-
-						skinnedDialog.close();
-					}
+					popoutEntry( target, menu==menuItemOnTop );
 				}
 			};
 			
@@ -1654,6 +1670,61 @@ public class TabbedMDI
 		}
 	}
 	
+	public boolean
+	popoutEntry(
+		MdiEntry	_entry,
+		boolean		onTop )
+	{
+		TabbedEntry	target = (TabbedEntry)_entry;
+		
+		SkinnedDialog skinnedDialog =
+				new SkinnedDialog(
+						"skin3_dlg_sidebar_popout",
+						"shell",
+						onTop?UIFunctionsManagerSWT.getUIFunctionsSWT().getMainShell():null,
+						SWT.RESIZE | SWT.MAX | SWT.DIALOG_TRIM);
+
+		SWTSkin skin = skinnedDialog.getSkin();
+
+		SWTSkinObjectContainer cont = target.buildStandAlone((SWTSkinObjectContainer)skin.getSkinObject( "content-area" ));
+
+		if ( cont != null ){
+
+			String ds_str = "";
+			Object ds = target.getDatasourceCore();
+			DownloadManager dm = DataSourceUtils.getDM(ds);
+
+			if (dm != null) {
+				ds_str = dm.getDisplayName();
+			}
+
+			skinnedDialog.setTitle( target.getTitle() + (ds_str.length()==0?"":(" - " + ds_str )));
+
+			
+			String metrics_id;
+			
+				// hack - we don't want to remember shell metrics on a per-download basis
+			
+			if ( target.getDatasource() instanceof Download ){
+				
+				metrics_id = MultipleDocumentInterface.SIDEBAR_SECTION_TORRENT_DETAILS;
+				
+			}else{
+				
+				metrics_id = target.getId();
+			}
+		
+			skinnedDialog.open( "mdi.popout:" + metrics_id, true );
+
+			return( true );
+			
+		}else{
+
+			skinnedDialog.close();
+			
+			return( false );
+		}
+	}
 	
 	@Override
 	public void fillMenu(Menu menu, final MdiEntry entry, String menuID) {
@@ -1771,7 +1842,7 @@ public class TabbedMDI
 					skinnedDialog.setTitle(((UISWTViewCore) parentView).getFullTitle()
 							+ (ds_str.length() == 0 ? "" : (" - " + ds_str)));
 
-					skinnedDialog.open();
+					skinnedDialog.open( "mdi.popout:" + currentEntry.getId(), true );
 
 				} else {
 
@@ -1798,4 +1869,12 @@ public class TabbedMDI
 	public TabbedEntry getEntryBySkinView(Object skinView) {
 		return (TabbedEntry) super.getEntryBySkinView(skinView);
 	}
+	
+	@Override
+	public void 
+	runWhenIdle(
+		Runnable r )
+	{
+		r.run();	// nothing smart needed here maybe
+	}
 }
