@@ -24,6 +24,7 @@ import com.biglybt.core.CoreFactory;
 
 import java.util.ArrayList;
 import java.util.List;
+import java.util.function.Supplier;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.events.DisposeEvent;
@@ -39,8 +40,12 @@ import com.biglybt.core.util.AERunnable;
 import com.biglybt.core.util.AEThread2;
 import com.biglybt.core.util.Debug;
 import com.biglybt.core.util.DelayedEvent;
+import com.biglybt.core.util.SystemTime;
+import com.biglybt.ui.swt.UIExitUtilsSWT;
 import com.biglybt.ui.swt.Utils;
+import com.biglybt.ui.swt.UIExitUtilsSWT.canCloseListener;
 import com.biglybt.ui.swt.mainwindow.Colors;
+import com.biglybt.ui.swt.shells.MessageBoxShell;
 import com.biglybt.core.CoreOperation;
 import com.biglybt.core.CoreOperationListener;
 import com.biglybt.core.CoreOperationTask;
@@ -51,10 +56,146 @@ import com.biglybt.ui.swt.imageloader.ImageLoader;
 public class
 ProgressWindow
 {
+	private static canCloseListener canCloseListener;
+	
 	public static void
 	register(
 		Core core )
 	{
+		canCloseListener = new canCloseListener() {
+			@Override
+			public boolean
+			canClose()
+			{
+				try{
+					Supplier<String> getActiveOps = ()->{
+						List<CoreOperation> ops = core.getOperations();
+
+						String active = "";
+						
+						for ( CoreOperation op: ops ){
+							
+							int type = op.getOperationType();
+							
+							if (	type == CoreOperation.OP_DOWNLOAD_CHECKING ||
+									type == CoreOperation.OP_DOWNLOAD_COPY ||
+									type == CoreOperation.OP_DOWNLOAD_EXPORT ||
+									type == CoreOperation.OP_FILE_MOVE ){
+								
+								CoreOperationTask task = op.getTask();
+								
+								if ( task != null ){
+						
+									if ( active.length() > 128 ){
+										
+										active += ", ...";
+										
+										break;
+									}
+									
+									active += ( active.isEmpty()?"":", ") + task.getName();							
+								}							
+							}
+						}
+						
+						return( active );
+					};
+					
+					String active = getActiveOps.get();
+					
+					if ( active.isEmpty()){
+						
+						return( true );
+						
+					}else{
+						
+						String title = MessageText.getString("coreops.active.quit.title");
+						String text = MessageText.getString(
+								"coreops.active.quit.text",
+								new String[] {
+									active
+								});
+	
+						MessageBoxShell mb = new MessageBoxShell(
+								title,
+								text,
+								new String[] {
+										MessageText.getString("UpdateWindow.quit"),
+										MessageText.getString("Content.alert.notuploaded.button.abort")
+								}, 1);
+	
+						mb.open(null);
+		
+						boolean[]	close_on_idle = { false };
+						
+						AEThread2.createAndStartDaemon( "opschecker", ()->{
+							
+							long	idle_start = -1;
+							
+							while( true ){
+								
+								try{
+									Thread.sleep( 250 );
+								
+								}catch( Throwable e ){
+								}
+								
+								if ( getActiveOps.get().isEmpty()){
+								
+									long now = SystemTime.getMonotonousTime();
+									
+									if ( idle_start == -1 ){
+										
+										idle_start = now;
+										
+									}else{
+										
+										if ( now - idle_start > 2500 ){
+									
+											synchronized( close_on_idle ){
+												
+												close_on_idle[0] = true;
+											}
+											
+											mb.close();
+										
+											break;
+										}
+									}
+								}else{
+									
+									idle_start = -1;
+								}
+							}
+						});
+						
+						mb.waitUntilClosed();
+
+						int result = mb.getResult();
+						
+						if ( result == 0 ){
+							
+							return( true );
+							
+						}else{
+							
+							synchronized( close_on_idle ){
+								
+								return( close_on_idle[0] );
+							}
+						}
+					}
+				}catch ( Throwable e ){
+
+					Debug.out(e);
+
+					return true;
+				}
+			}
+		};
+		
+		UIExitUtilsSWT.addListener(canCloseListener);
+		
 		core.addOperationListener(
 			new CoreOperationListener()
 			{
@@ -67,6 +208,7 @@ ProgressWindow
 					
 					if ( 	( 	type == CoreOperation.OP_FILE_MOVE ||
 								type == CoreOperation.OP_DOWNLOAD_EXPORT ||
+								type == CoreOperation.OP_DOWNLOAD_COPY ||
 								type == CoreOperation.OP_PROGRESS ) &&
 							Utils.isThisThreadSWT()){
 
@@ -116,6 +258,12 @@ ProgressWindow
 			});
 	}
 
+	public static void
+	unregister()
+	{
+		UIExitUtilsSWT.removeListener(canCloseListener);
+	}
+	
 	private volatile Shell 			shell;
 	private volatile boolean 		task_complete;
 
@@ -142,6 +290,8 @@ ProgressWindow
 			resource = "progress.window.msg.filemove";
 		}else if ( op_type == CoreOperation.OP_DOWNLOAD_EXPORT ){
 			resource = "progress.window.msg.dlexport";
+		}else if ( op_type == CoreOperation.OP_DOWNLOAD_COPY ){
+			resource = "progress.window.msg.dlcopy";
 		}else{
 			resource = "progress.window.msg.progress";
 			
@@ -381,7 +531,9 @@ ProgressWindow
 		layout.numColumns = 2;
 		shell.setLayout(layout);
 
-		shell.setBackground( Colors.white );
+		Color bg = Utils.isDarkAppearancePartial()?null:Colors.white;
+		
+		shell.setBackground( bg );
 		
 		if ( task != null ){
 			
@@ -399,7 +551,7 @@ ProgressWindow
 				GridData gridData = new GridData( GridData.FILL_HORIZONTAL );
 				gridData.horizontalSpan = 2;
 				lName.setLayoutData(gridData);
-				lName.setBackground( Colors.white );
+				lName.setBackground( bg );
 				
 				lName.setFont( bold_font );
 				
@@ -421,7 +573,7 @@ ProgressWindow
 					gridData.horizontalSpan = 2;
 					gridData.horizontalIndent = 25;
 					subtask_label.setLayoutData(gridData);
-					subtask_label.setBackground( Colors.white );
+					subtask_label.setBackground( bg );
 				}
 			}
 		}
@@ -496,7 +648,7 @@ ProgressWindow
 					}
 				});
 
-	    	canvas.setBackground( Colors.white );
+	    	canvas.setBackground( bg );
 		}
 
 
@@ -505,7 +657,7 @@ ProgressWindow
 		label.setText(MessageText.getString( resource ));
 		GridData gridData = new GridData();
 		label.setLayoutData(gridData);
-		label.setBackground( Colors.white );
+		label.setBackground( bg );
 		
 		if ( progress != null ){
 		
@@ -518,12 +670,12 @@ ProgressWindow
 			layoutProgress.numColumns = 1;
 			layoutProgress.marginWidth = layoutProgress.marginHeight = 0;
 			compProg.setLayout(layoutProgress);
-			compProg.setBackground( Colors.white );
+			compProg.setBackground( bg );
 			
 			progress_bar = new ProgressBar(compProg,SWT.HORIZONTAL );
 			progress_bar.setMinimum(0);
 			progress_bar.setMaximum(1000);
-			progress_bar.setBackground( Colors.white );
+			progress_bar.setBackground( bg );
 			gridData = new GridData( GridData.FILL_HORIZONTAL );
 			gridData.widthHint = 400;
 			progress_bar.setLayoutData(gridData);
@@ -561,7 +713,7 @@ ProgressWindow
 				GridLayout layoutButtons = new GridLayout();
 				layoutButtons.numColumns = num_buttons;
 				comp.setLayout(layoutButtons);
-				comp.setBackground( Colors.white );
+				comp.setBackground( bg );
 		
 				List<Button> buttons = new ArrayList<>();
 		
