@@ -30,6 +30,7 @@ import com.biglybt.core.Core;
 import com.biglybt.core.CoreFactory;
 import com.biglybt.core.CoreRunningListener;
 import com.biglybt.core.util.*;
+import com.biglybt.pifimpl.local.PluginInitializer;
 import com.biglybt.ui.UIFunctionsManager;
 import com.biglybt.ui.UserPrompterResultListener;
 import com.biglybt.ui.common.RememberedDecisionsManager;
@@ -41,7 +42,6 @@ import com.biglybt.ui.selectedcontent.SelectedContent;
 import com.biglybt.ui.selectedcontent.SelectedContentManager;
 import com.biglybt.ui.swt.columns.search.ColumnSearchResultSite;
 import com.biglybt.ui.swt.columns.searchsubs.*;
-import com.biglybt.ui.swt.imageloader.ImageLoader.ImageDownloaderListener;
 import com.biglybt.ui.swt.search.SearchResultsTabArea.SearchQuery;
 import com.biglybt.ui.swt.utils.SearchSubsUtils;
 import com.biglybt.ui.swt.views.skin.VuzeMessageBox;
@@ -69,7 +69,6 @@ import com.biglybt.core.config.COConfigurationManager;
 import com.biglybt.core.internat.MessageText;
 import com.biglybt.pif.ui.tables.TableColumn;
 import com.biglybt.pif.ui.tables.TableColumnCreationListener;
-import com.biglybt.pifimpl.local.PluginInitializer;
 import com.biglybt.ui.swt.Messages;
 import com.biglybt.ui.swt.Utils;
 import com.biglybt.ui.swt.mainwindow.ClipboardCopy;
@@ -123,8 +122,6 @@ SBC_SearchResultsView
 	private Composite			table_parent;
 
 
-	private Text txtFilter;
-
 	private final Object filter_lock = new Object();
 
 	private Spinner spinMinSize;
@@ -190,11 +187,6 @@ SBC_SearchResultsView
 				}
 			});
 
-		SWTSkinObjectTextbox soFilterBox = (SWTSkinObjectTextbox) getSkinObject("filterbox");
-		if (soFilterBox != null) {
-			txtFilter = soFilterBox.getTextControl();
-		}
-
 		if ( vitality_images == null ){
 
 			ImageLoader loader = ImageLoader.getInstance();
@@ -862,7 +854,7 @@ SBC_SearchResultsView
 					}
 
 					if ( tv_subs_results != null) {
-						tv_subs_results.setFilterText("");
+						tv_subs_results.setFilterText("", false);
 					}
 				}
 			});
@@ -1221,20 +1213,13 @@ SBC_SearchResultsView
 		tableManager.setDefaultSortColumnName(TABLE_SR, ColumnSearchSubResultRank.COLUMN_ID);
 
 
-		if (txtFilter != null) {
-			tv_subs_results.enableFilterCheck(txtFilter, this);
+		SWTSkinObjectTextbox soFilterBox = (SWTSkinObjectTextbox) getSkinObject("filterbox");
+		if (soFilterBox != null) {
+			tv_subs_results.enableFilterCheck(soFilterBox.getBubbleTextBox(), this);
 		}
 
 		tv_subs_results.setRowDefaultHeight(COConfigurationManager.getIntParameter( "Search Subs Row Height" ));
 
-		SWTSkinObject soSizeSlider = getSkinObject("table-size-slider");
-		if (soSizeSlider instanceof SWTSkinObjectContainer) {
-			SWTSkinObjectContainer so = (SWTSkinObjectContainer) soSizeSlider;
-			if (!tv_subs_results.enableSizeSlider(so.getComposite(), 16, 100)) {
-				so.setVisible(false);
-			}
-		}
-
 		table_parent = new Composite(control, SWT.NONE);
 		table_parent.setLayoutData(Utils.getFilledFormData());
 		GridLayout layout = new GridLayout();
