@@ -1888,7 +1888,7 @@ public class MainWindowImpl
 			cArea= c2;
 		}
 
-		final Text text = new Text(cArea, DARK_MODE?SWT.BORDER:SWT.NONE);
+		final Text text = new Text(cArea, (DARK_MODE && !Utils.isGTK3 )?SWT.BORDER:SWT.NONE);
 		text.setMessage(MessageText.getString("v3.MainWindow.search.defaultText"));
 		FormData filledFormData = Utils.getFilledFormData();
 		text.setLayoutData(filledFormData);
@@ -1927,8 +1927,10 @@ public class MainWindowImpl
 		Color colorSearchTextBG = properties.getColor("color.search.text.bg");
 		Color colorSearchTextFG = properties.getColor("color.search.text.fg");
 
-		if (colorSearchTextBG != null) {
-			text.setBackground(colorSearchTextBG);
+		if ( !Utils.isDarkAppearancePartial()) {
+			if (colorSearchTextBG != null) {
+				text.setBackground(colorSearchTextBG);
+			}
 		}
 
 		final TextWithHistory twh = new TextWithHistory( "mainwindow.search.history", text );
