@@ -87,7 +87,7 @@ SBC_SubscriptionResultsView
 
 	private static boolean columnsAdded = false;
 
-	private static TableRowSWT.ColorRequester colour_requester = ()->1;
+	private static TableViewSWT.ColorRequester colour_requester = ()->1;
 	
 	private TableViewSWT<SubscriptionResultFilterable> tv_subs_results;
 
@@ -95,8 +95,6 @@ SBC_SubscriptionResultsView
 	private Composite			table_parent;
 
 
-	private Text txtFilter;
-
 	private final Object filter_lock = new Object();
 
 
@@ -199,11 +197,6 @@ SBC_SubscriptionResultsView
 			}
 		}
 
-		SWTSkinObjectTextbox soFilterBox = (SWTSkinObjectTextbox) getSkinObject("filterbox");
-		if (soFilterBox != null) {
-			txtFilter = soFilterBox.getTextControl();
-		}
-
 		final SWTSkinObject soFilterArea = getSkinObject("filterarea");
 		if (soFilterArea != null) {
 
@@ -1064,21 +1057,14 @@ SBC_SubscriptionResultsView
 				tcc.setDefaultSortAscending( true );
 			}
 		}
-		
-		if (txtFilter != null) {
-			tv_subs_results.enableFilterCheck(txtFilter, this);
+
+		SWTSkinObjectTextbox soFilterBox = (SWTSkinObjectTextbox) getSkinObject("filterbox");
+		if (soFilterBox != null) {
+			tv_subs_results.enableFilterCheck(soFilterBox.getBubbleTextBox(), this);
 		}
 
 		tv_subs_results.setRowDefaultHeight(COConfigurationManager.getIntParameter( "Search Subs Row Height" ));
 
-		SWTSkinObject soSizeSlider = getSkinObject("table-size-slider");
-		if (soSizeSlider instanceof SWTSkinObjectContainer) {
-			SWTSkinObjectContainer so = (SWTSkinObjectContainer) soSizeSlider;
-			if (!tv_subs_results.enableSizeSlider(so.getComposite(), 16, 100)) {
-				so.setVisible(false);
-			}
-		}
-
 		table_parent = new Composite(control, SWT.NONE);
 		table_parent.setLayoutData(Utils.getFilledFormData());
 		GridLayout layout = new GridLayout();
