@@ -33,6 +33,15 @@ import com.biglybt.ui.swt.Utils;
  */
 public class FontUtils
 {
+	private static final String[] MONO_FONT_NAMES = {
+		"Monaco",
+		"Consolas",
+		"Lucida Console",
+		"Courier",
+		"Courier New",
+		"Monospace"
+	};
+
 
 	private static Method mFontData_SetHeight;
 
@@ -62,6 +71,11 @@ public class FontUtils
 
 	public static Font getFontWithHeight(Font baseFont, int heightInPixels,
 			int style) {
+		return getFontWithHeight(baseFont, heightInPixels, style, true);
+	}
+
+	private static Font getFontWithHeight(Font baseFont, int heightInPixels,
+			int style, boolean runAgain) {
 		boolean destroyBaseFont = style != SWT.DEFAULT;
 		if (destroyBaseFont) {
 			baseFont = getFontWithStyle(baseFont, style, 1.0f);
@@ -72,7 +86,9 @@ public class FontUtils
 		if (destroyBaseFont) {
 			baseFont.dispose();
 		}
-		return font;
+		// new font might still be bigger than requested height due to Fonts being fonts
+		return runAgain ? getFontWithHeight(font, heightInPixels, style, false)
+				: font;
 	}
 
 		// Used by azemp plugin
@@ -107,6 +123,7 @@ public class FontUtils
 		GC gc = new GC(font.getDevice());
 		try {
 			gc.setFont(font);
+			gc.setTextAntialias(SWT.ON);
 			return gc.textExtent(Utils.GOOD_STRING).y;
 		} finally {
 			gc.dispose();
@@ -221,6 +238,10 @@ public class FontUtils
 	}
 
 	public static void fontToWidgetHeight(Text text) {
+		fontToWidgetHeight(text, null);
+	}
+
+	public static void fontToWidgetHeight(Text text, Runnable runOnFontSizeChange) {
 		text.addListener(SWT.Resize, new Listener() {
 			Font lastFont = null;
 			int	lastHeight = -1;
@@ -255,7 +276,7 @@ public class FontUtils
 				//System.out.println("h=" + h + ";lh=" + lineHeightPX + ";" + getFontHeightInPX(text.getFont()) );
 
 				if ( h == lastHeight ){
-					//return;
+					return;
 				}
 
 				float pctAdjust = h / (float) lineHeightPX;
@@ -282,6 +303,10 @@ public class FontUtils
 				}else{
 					Utils.disposeSWTObjects(lastFont);
 				}
+				
+				if (runOnFontSizeChange != null) {
+					runOnFontSizeChange.run();
+				}
 
 				lastFont = font;
 			}
@@ -316,6 +341,7 @@ public class FontUtils
 	public static double getCharacterWidth(Font f) {
 		GC gc = new GC(f.getDevice());
 		gc.setFont(f);
+		gc.setTextAntialias(SWT.ON);
 		FontMetrics metrics = gc.getFontMetrics();
 		double d;
 		try {
@@ -327,4 +353,41 @@ public class FontUtils
 		gc.dispose();
 		return d;
 	}
+
+	public static Font getMonospaceFont(Device device, int heightInPoints) {
+		Font fontMonospace = null;
+
+		for (String tryName : MONO_FONT_NAMES) {
+			fontMonospace = new Font(device, tryName, heightInPoints, SWT.NORMAL);
+			FontData[] fontData = fontMonospace.getFontData();
+			if (fontData.length > 0) {
+				int w1 = getTextWidth(device, fontMonospace, "i");
+				int w2 = getTextWidth(device, fontMonospace, "w");
+
+				//System.out.println("MonoFont " + tryName + ". widths=" + w1 + "," + w2);
+
+				if (w1 == w2) {
+					break;
+				}
+			}
+			fontMonospace.dispose();
+			fontMonospace = null;
+		}
+
+		return fontMonospace;
+	}
+
+	public static int getTextWidth(Device device, Font font, String text) {
+		int width = 0;
+		try {
+			GC gc = new GC(device);
+			gc.setFont(font);
+			gc.setTextAntialias(SWT.ON);
+			width = gc.textExtent(text).x;
+			gc.dispose();
+		} catch (Exception ex) {
+			width = text.length() * 5;
+		}
+		return width;
+	}
 }
