@@ -165,7 +165,7 @@ public class ConfigView implements UISWTViewCoreEventListener {
 			BubbleTextBox bubbleTextBox = new BubbleTextBox(cLeftSide, SWT.BORDER
 					| SWT.SEARCH | SWT.ICON_SEARCH | SWT.ICON_CANCEL | SWT.SINGLE);
 			final Text txtFilter = bubbleTextBox.getTextWidget();
-			Composite cFilterArea = bubbleTextBox.getParent();
+			Composite cFilterArea = bubbleTextBox.getMainWidget();
 
       txtFilter.setMessage(MessageText.getString("ConfigView.filter"));
       txtFilter.addModifyListener(e -> filterTree(txtFilter.getText()));
@@ -181,6 +181,7 @@ public class ConfigView implements UISWTViewCoreEventListener {
       formData.bottom = new FormAttachment(100, -5);
 			formData.left = new FormAttachment(0, 2);
 			formData.right = new FormAttachment(100, -2);
+	    formData.height = (int) (FontUtils.getFontHeightInPX(txtFilter.getFont()) * 1.4);
 	    cFilterArea.setLayoutData(formData);
 
       formData = new FormData();
