@@ -109,6 +109,7 @@ public class FilesView
     new DoneItem(),
     new PercentItem(),
     new FirstPieceItem(),
+    new LastPieceItem(),
     new PieceCountItem(),
     new RemainingPiecesItem(),
     new PiecesDoneAndCountItem(),
@@ -141,7 +142,7 @@ public class FilesView
 
   public static final String MSGID_PREFIX = "FilesView";
 
-	private DownloadManager[] managers = new DownloadManager[0];
+  private DownloadManager[] managers = new DownloadManager[0];
 
   public boolean hide_dnd_files;
   public boolean tree_view;
@@ -304,31 +305,32 @@ public class FilesView
 		lblHeader = new BufferedLabel(cTop, SWT.CENTER | ( Constants.isLinux?0: SWT.DOUBLE_BUFFERED));
 
 		BubbleTextBox bubbleTextBox = new BubbleTextBox(cTop, SWT.BORDER | SWT.SEARCH | SWT.ICON_SEARCH | SWT.ICON_CANCEL | SWT.SINGLE);
-		Text bubbleTextWidget = bubbleTextBox.getTextWidget();
-		FontUtils.fontToWidgetHeight(bubbleTextWidget);
-		bubbleTextWidget.setMessage(MessageText.getString("TorrentDetailsView.filter"));
+		Composite mainBubbleWidget = bubbleTextBox.getMainWidget();
+		bubbleTextBox.setMessage(MessageText.getString("TorrentDetailsView.filter"));
 
-		FormData fd = Utils.getFilledFormData();
-		fd.left = null;
-		bubbleTextBox.getParent().setLayoutData(fd);
+		FormData fd = new FormData();
+		fd.right = new FormAttachment(100, 0);
+		fd.height = (int) (FontUtils.getFontHeightInPX(mainBubbleWidget.getFont()) * 1.4);
+		fd.width = 140;
+		mainBubbleWidget.setLayoutData(fd);
 
 		fd = new FormData();
-		fd.top = new FormAttachment(bubbleTextBox.getParent(), 10, SWT.CENTER);
+		fd.top = new FormAttachment(mainBubbleWidget, 10, SWT.CENTER);
 		fd.left = new FormAttachment(0, 0);
 		btnShowDND.setLayoutData(fd);
 
 		fd = new FormData();
-		fd.top = new FormAttachment(bubbleTextBox.getParent(), 10, SWT.CENTER);
+		fd.top = new FormAttachment(mainBubbleWidget, 10, SWT.CENTER);
 		fd.left = new FormAttachment(btnShowDND, 10);
 		btnTreeView.setLayoutData(fd);
 		
 		fd = new FormData();
-		fd.top = new FormAttachment(bubbleTextBox.getParent(), 10, SWT.CENTER);
+		fd.top = new FormAttachment(mainBubbleWidget, 10, SWT.CENTER);
 		fd.left = new FormAttachment(btnTreeView, 10);
-		fd.right = new FormAttachment(bubbleTextBox.getParent(), -10);
+		fd.right = new FormAttachment(mainBubbleWidget, -10);
 		lblHeader.setLayoutData(fd);
 
-		tv.enableFilterCheck(bubbleTextWidget, this, true );
+		tv.enableFilterCheck(bubbleTextBox, this, true );
 
 		Composite tableParent = new Composite(parent, SWT.NONE);
 
@@ -343,6 +345,38 @@ public class FilesView
 		return tableParent;
 	}
 
+	private void
+	addManagerListeners(
+		DownloadManager[]		managers )
+	{
+		if ( managers == null ){
+			return;
+		}
+		
+		for (DownloadManager manager: managers ){
+			manager.getDownloadState().addListener(this,
+					DownloadManagerState.AT_FILE_LINKS2,
+					DownloadManagerStateAttributeListener.WRITTEN);
+
+			manager.addListener(this);
+		}
+	}
+	
+	private void
+	removeManagerListeners(
+		DownloadManager[]		managers )
+	{
+		if ( managers == null ){
+			return;
+		}
+
+		for (DownloadManager manager: managers ){
+			manager.getDownloadState().removeListener(this,
+					DownloadManagerState.AT_FILE_LINKS2,
+					DownloadManagerStateAttributeListener.WRITTEN);
+			manager.removeListener(this);
+		}
+	}
 
   // @see TableDataSourceChangedListener#tableDataSourceChanged(java.lang.Object)
 	@Override
@@ -359,12 +393,7 @@ public class FilesView
 		
 		if (tv != null) {
 	
-			for (DownloadManager manager: managers ){
-				manager.getDownloadState().removeListener(this,
-						DownloadManagerState.AT_FILE_LINKS2,
-						DownloadManagerStateAttributeListener.WRITTEN);
-				manager.removeListener(this);
-			}
+			removeManagerListeners( managers );
 		}
 
 		if (tags != null && tags.length > 0 && tag_listener != null) {
@@ -416,13 +445,7 @@ public class FilesView
 			return;
 		}
 
-		for (DownloadManager manager: managers ){
-			manager.getDownloadState().addListener(this,
-					DownloadManagerState.AT_FILE_LINKS2,
-					DownloadManagerStateAttributeListener.WRITTEN);
-
-			manager.addListener(this);
-		}
+		addManagerListeners( managers );
 
 		if (!tv.isDisposed()) {
 
@@ -501,7 +524,19 @@ public class FilesView
 		
 		if ( file != null ){
 			
-			TableRowCore[] rows = tv.getVisibleRows();
+			TableRowCore[] rows;
+			
+			if ( tree_view ){
+				
+					// have to get all rows, not just visible ones, as changed file might be collasped
+					// but we still want to update parent state
+				
+				rows = tv.getRowsAndSubRows( true );	
+				
+			}else{
+				
+				rows = tv.getVisibleRows();
+			}
 			
 			for ( TableRowCore row: rows ){
 				
@@ -583,7 +618,7 @@ public class FilesView
 			return( true );
 		}
 
-		if ( tv.getFilterControl() == null ){
+		if ( !tv.hasFilterControl() ){
 
 				// view has no visible filter control so ignore any current filter as the
 				// user's going to get confused...
@@ -598,28 +633,37 @@ public class FilesView
 			return( true );
 		}
 		
+		boolean filterOnPath = filter.startsWith("p:");
+		if (filterOnPath) {
+			filter = filter.substring(2);
+		}
+		
 		try {			
 			File file = ds.getFile(true);
 
 			String name;
 			
-			if ( filter.startsWith( File.separator )){
-				
-				filter = filter.substring( 1 );
-				
-				if ( filter.isEmpty()){
-					
-					return( true );
-				}
-				
+			if (filterOnPath) {
 				name = file.getAbsolutePath();
-				
-			}else{
-				
-				name = file.getName();
+			} else {
+				if ( filter.startsWith( File.separator )){
+					
+					filter = filter.substring( 1 );
+					
+					if ( filter.isEmpty()){
+						
+						return( true );
+					}
+					
+					name = file.getAbsolutePath();
+					
+				}else{
+					
+					name = file.getName();
+				}
 			}
 
-			String s = regex ? filter : "\\Q" + filter.replaceAll("[|;]", "\\\\E|\\\\Q") + "\\E";
+			String s = regex ? filter : RegExUtil.splitAndQuote( filter, "\\s*[|;]\\s*" );
 
 			boolean	match_result = true;
 
@@ -787,7 +831,18 @@ public class FilesView
 				ManagerUtils.browse(fileInfo);
 
 				break;
+				
+			case "6":{
 
+				DownloadManager dm = fileInfo.getDownloadManager();
+				
+				if ( dm != null ){
+				
+					UIFunctionsManager.getUIFunctions().getMDI().popoutEntryByID( MultipleDocumentInterface.SIDEBAR_SECTION_TORRENT_DETAILS, dm, true );
+				}
+				
+				break;
+			}
 			default:
 
 				int file_index = fileInfo.getIndex();
@@ -1175,6 +1230,9 @@ public class FilesView
 				hide_dnd_files = COConfigurationManager.getBooleanParameter(
 						"FilesView.hide.dnd");
 				COConfigurationManager.addParameterListener("FilesView.hide.dnd", this);
+				
+				addManagerListeners( managers );
+				
 				break;
 			}
 
@@ -1182,14 +1240,9 @@ public class FilesView
 				COConfigurationManager.removeParameterListener("FilesView.hide.dnd",
 						this);
 
-				for (DownloadManager manager : managers) {
-					manager.getDownloadState().removeListener(this,
-							DownloadManagerState.AT_FILE_LINKS2,
-							DownloadManagerStateAttributeListener.WRITTEN);
-
-					manager.removeListener(this);
-				}
-
+				removeManagerListeners( managers );
+				
+				break;
 			}
 		}
 	}
@@ -1290,10 +1343,79 @@ public class FilesView
 
 	// @see org.eclipse.swt.events.KeyListener#keyPressed(org.eclipse.swt.events.KeyEvent)
 	@Override
-	public void keyPressed(KeyEvent e) {
-		if (e.keyCode == SWT.F2 && (e.stateMask & SWT.MODIFIER_MASK) == 0) {
+	public void 
+	keyPressed(
+		KeyEvent e) 
+	{
+		if ( e.keyCode == SWT.F2 && (e.stateMask & SWT.MODIFIER_MASK) == 0 ){
+			
 			FilesViewMenuUtil.rename(tv, tv.getSelectedDataSources(true), true, false,false);
+			
 			e.doit = false;
+			
+		}else{
+			
+			Composite table_composite = tv.getComposite();
+			
+			char character = e.character;
+			
+			if ( 	e.widget == table_composite &&
+					( character == ' ' || character == '+' || character == '-' )){
+		
+				Object[] data_sources = tv.getSelectedDataSources().toArray();
+							
+				List<DiskManagerFileInfo>	files = new ArrayList<>();
+				
+				boolean all_skipped = true;
+	
+				for ( int i=0;i<data_sources.length;i++ ){
+					
+					DiskManagerFileInfo file = (DiskManagerFileInfo)data_sources[i];
+													
+					if ( file instanceof FilesViewNodeInner ){
+						
+						FilesViewNodeInner inner = (FilesViewNodeInner)file;
+						
+						inner.getFiles( files, true );
+						
+						if ( all_skipped ){
+							
+							int state = inner.getSkippedState();
+													
+							if ( state != 0 ){
+								
+								all_skipped = false;
+							}
+						}
+					}else{
+					
+						files.add( file );
+	
+						if ( all_skipped ){
+							
+							if ( !file.isSkipped()){
+								
+								all_skipped = false;
+							}
+						}
+					}
+				}
+				
+				if ( character == ' ' ){
+					
+					FilesViewMenuUtil.setSkipped( files, !all_skipped, 2, true );
+				}else{
+					
+					for ( DiskManagerFileInfo file: files ){
+						
+						int pri = file.getPriority();
+						
+						file.setPriority( character=='+'?pri+1:pri-1);
+					}
+				}
+			
+				e.doit = false;
+			}  
 		}
 	}
 
@@ -1803,6 +1925,9 @@ public class FilesView
 		setSkipped(
 			boolean		b );
 		
+		public Boolean
+		isSkipping();
+		
 		public void
 		recheck();
 		
@@ -1991,6 +2116,37 @@ public class FilesView
 			}
 		}
 
+		@Override
+		public Boolean 
+		isSkipping()
+		{
+			Boolean result = null;
+			
+			for ( FilesViewTreeNode kid: kids.values()){
+				
+				Boolean k = kid.isSkipping();
+				
+				if ( k == null ){
+					
+					if ( result != null ){
+						
+						return( null );
+					}
+				}else{
+					
+					if ( result == null ){
+						
+						result = k;
+						
+					}else if ( result.booleanValue() != k ){
+						
+						return( null );
+					}
+				}
+			}
+			
+			return( result );
+		}
 
 		@Override
 		public boolean
@@ -2371,6 +2527,13 @@ public class FilesView
 			ManagerUtils.setFileSkipped( delegate, b );
 		}
 
+		@Override
+		public Boolean 
+		isSkipping()
+		{
+			return( delegate.isSkipping());
+		}
+		
 		@Override
 		public boolean
 		setLink(
