@@ -21,6 +21,7 @@
 package com.biglybt.ui.swt.views;
 
 import java.io.File;
+import java.nio.file.Files;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
@@ -36,7 +37,6 @@ import java.util.Set;
 
 import com.biglybt.core.util.*;
 import org.eclipse.swt.SWT;
-import org.eclipse.swt.events.SelectionListener;
 import org.eclipse.swt.widgets.*;
 import com.biglybt.core.config.COConfigurationManager;
 import com.biglybt.core.disk.DiskManagerFileInfo;
@@ -49,6 +49,7 @@ import com.biglybt.core.peer.PEPeerManager;
 import com.biglybt.core.peermanager.piecepicker.PiecePicker;
 import com.biglybt.core.torrent.PlatformTorrentUtils;
 import com.biglybt.core.torrent.TOTorrent;
+import com.biglybt.core.torrent.TOTorrentFile;
 import com.biglybt.pif.ui.UIInputReceiver;
 import com.biglybt.pif.ui.UIInputReceiverListener;
 import com.biglybt.pif.ui.menus.MenuManager;
@@ -63,7 +64,6 @@ import com.biglybt.ui.swt.columns.torrent.ColumnUnopened;
 import com.biglybt.ui.swt.mainwindow.ClipboardCopy;
 import com.biglybt.ui.swt.mainwindow.MenuFactory;
 import com.biglybt.ui.swt.sharing.ShareUtils;
-import com.biglybt.ui.swt.shells.AdvRenameWindow;
 import com.biglybt.ui.swt.shells.MessageBoxShell;
 import com.biglybt.ui.swt.shells.RevertFileLocationsWindow;
 import com.biglybt.ui.swt.views.utils.ManagerUtils;
@@ -445,7 +445,8 @@ public class FilesViewMenuUtil
 		boolean all_normal_pri 			= true;
 		boolean all_low_pri 			= true;
 		boolean	all_complete			= true;
-
+		boolean	all_pad					= true;
+		
 		boolean	any_relocated			= false;
 
 		final List<DiskManagerFileInfo>		files_with_links = new ArrayList<>();
@@ -469,6 +470,13 @@ public class FilesViewMenuUtil
 
 				DiskManagerFileInfo file_info = files[i];
 
+				TOTorrentFile tf = file_info.getTorrentFile();
+				
+				if ( tf != null && !tf.isPadFile()){
+					
+					all_pad = false;
+				}
+			
 				if (open && file_info.getAccessMode() != DiskManagerFileInfo.READ) {
 
 					open = false;
@@ -539,6 +547,11 @@ public class FilesViewMenuUtil
 			}
 		}
 
+		if ( all_pad ){
+			
+			itemPriority.setEnabled( false );
+		}
+		
 		if ( itemOpen != null ){
 			
 			// we can only open files if they are read-only
@@ -1027,6 +1040,7 @@ public class FilesViewMenuUtil
 									moveFile(
 										manager,
 										file,
+										existing_file,
 										target_file,
 										dont_delete_existing,
 										new Runnable()
@@ -1170,6 +1184,7 @@ public class FilesViewMenuUtil
 					moveFile(
 						fileInfo.getDownloadManager(),
 						fileInfo,
+						existing_file,
 						ff_target,
 						f_dont_delete_existing,
 						new Runnable()
@@ -1186,9 +1201,8 @@ public class FilesViewMenuUtil
 		} finally {
 			if ( affected_files.size() > 0 ){
 
-				Utils.getOffOfSWTThread(new AERunnable() {
-					@Override
-					public void runSupport() {
+				AEThread2.createAndStartDaemon( "FilesViewWaiter", ()->{
+					
 						for ( int i=0;i<affected_files.size();i++){
 							task_sem.reserve();
 						}
@@ -1198,8 +1212,7 @@ public class FilesViewMenuUtil
 						}
 
 						invalidateRows( tv, affected_files );
-					}
-				});
+					});
 			}
 		}
 	}
@@ -1216,13 +1229,17 @@ public class FilesViewMenuUtil
 
 		Set<TableRowCore>	done = new HashSet<>();
 
+		TableRowCore[]	all_rows = null;
+		
 		for ( DiskManagerFileInfo file: files ){
 
 			TableRowCore row =  tv.getRow(file);
 
 			if ( row == null ){
 
-				row = tv.getRow( file.getDownloadManager());
+				DownloadManager dm = file.getDownloadManager();
+				 
+				row = tv.getRow( dm );
 
 				if ( row != null ){
 
@@ -1277,6 +1294,29 @@ public class FilesViewMenuUtil
 							}
 							
 						}
+					}else{
+					
+						if ( all_rows == null ){
+							
+							all_rows = tv.getRowsAndSubRows( false );
+						}
+						
+						for ( TableRowCore r: all_rows ){
+							
+							Object o = r.getDataSource( true );
+							
+							if ( o instanceof DiskManagerFileInfo ){
+								
+								DiskManagerFileInfo f = (DiskManagerFileInfo)o;
+								
+								if ( f.getDownloadManager() == dm && f.getIndex() == file.getIndex()){
+									
+									row = r;
+									
+									break;
+								}
+							}
+						}
 					}
 				}
 			}
@@ -1453,7 +1493,7 @@ public class FilesViewMenuUtil
 				dm.setFilePriorities(fileInfos, -1);
 			}
 
-			boolean paused = setSkipped(dm, fileInfos, skipped, delete_action, prompt );
+			boolean paused = setSkipped(dm, fileInfos, skipped, delete_action?1:0, prompt );
 
 			if (paused) {
 
@@ -1462,6 +1502,38 @@ public class FilesViewMenuUtil
 		}
 	}
 
+	public static void setSkipped( List<DiskManagerFileInfo> file_list, boolean skipped, int delete_action, boolean prompt ) {
+
+		if (file_list == null || file_list.size() == 0) {
+			return;
+		}
+
+		Map<DownloadManager, ArrayList<DiskManagerFileInfo>> mapDMtoDMFI = new IdentityHashMap<>();
+
+		for ( DiskManagerFileInfo file: file_list ){
+
+			DownloadManager dm = file.getDownloadManager();
+			ArrayList<DiskManagerFileInfo> listFileInfos = mapDMtoDMFI.get(dm);
+			if (listFileInfos == null) {
+				listFileInfos = new ArrayList<>(1);
+				mapDMtoDMFI.put(dm, listFileInfos);
+			}
+			listFileInfos.add(file);
+		}
+
+		for (DownloadManager dm : mapDMtoDMFI.keySet()) {
+			ArrayList<DiskManagerFileInfo> list = mapDMtoDMFI.get(dm);
+			DiskManagerFileInfo[] fileInfos = list.toArray(new DiskManagerFileInfo[0]);
+
+			boolean paused = setSkipped(dm, fileInfos, skipped, delete_action, prompt );
+
+			if (paused) {
+
+				dm.resume();
+			}
+		}
+	}
+	
 	private static void changePriorityManual(final List<DiskManagerFileInfo> file_list) {
 
 		SimpleTextEntryWindow entryWindow = new SimpleTextEntryWindow(
@@ -1509,7 +1581,7 @@ public class FilesViewMenuUtil
 				for (DownloadManager dm : mapDMtoDMFI.keySet()) {
 					ArrayList<DiskManagerFileInfo> list = mapDMtoDMFI.get(dm);
 					DiskManagerFileInfo[] fileInfos = list.toArray(new DiskManagerFileInfo[0]);
-					boolean paused = setSkipped(dm, fileInfos, false, false,true);
+					boolean paused = setSkipped(dm, fileInfos, false, 0, true);
 
 					if (paused) {
 
@@ -1578,7 +1650,7 @@ public class FilesViewMenuUtil
 		for (DownloadManager dm : mapDMtoDMFI.keySet()) {
 			ArrayList<DiskManagerFileInfo> list = mapDMtoDMFI.get(dm);
 			DiskManagerFileInfo[] fileInfos = list.toArray(new DiskManagerFileInfo[0]);
-			boolean paused = setSkipped(dm, fileInfos, false, false, true);
+			boolean paused = setSkipped(dm, fileInfos, false, 0, true);
 
 			if (paused) {
 
@@ -1713,6 +1785,7 @@ public class FilesViewMenuUtil
 				moveFile(
 						fileInfo.getDownloadManager(),
 						fileInfo,
+						existing_file,
 						f_target,
 						false,
 						new Runnable() {
@@ -1809,79 +1882,142 @@ public class FilesViewMenuUtil
 
 	// same code is used in tableitems.files.NameItem
 	
-	private static final AESemaphore moveCopyFileSem = new AESemaphore( "moveCopyFile", 1 );
+	private static final AsyncDispatcher moveCopyDispatcher = new AsyncDispatcher( "moveCopyFile" );
 	
 	private static void
 	moveFile(
-		final DownloadManager 			manager,
-		final DiskManagerFileInfo 		fileInfo,
-		final File 						target,
-		boolean							dont_delete_existing,
-		final Runnable					done )
+		DownloadManager 			manager,
+		DiskManagerFileInfo 		fileInfo,
+		File						source,
+		File 						target,
+		boolean						dont_delete_existing,
+		Runnable					done )
 	{
-		Utils.getOffOfSWTThread(new AERunnable() {
-			@Override
-			public void runSupport(){
-				moveCopyFileSem.reserve();
-					
-				moveFileSupport(
-					manager, fileInfo, target, dont_delete_existing, 
-					new Runnable()
-					{
-						public void 
-						run()
-						{
-							moveCopyFileSem.release();
-							
-							if ( done != null ){
-								
-								done.run();
-							}
-						};
-					});
-			}
+		moveCopyDispatcher.dispatch(()->{					
+			moveFileSupport(
+				manager, fileInfo, source, target, dont_delete_existing, 
+				()->{
+					if ( done != null ){
+						
+						done.run();
+					}
+				});
 		});
 	}
 	
 	private static void
-	copyFile(
-		final DownloadManager 			manager,
-		final DiskManagerFileInfo 		fileInfo,
-		final File 						target,
-		boolean							dont_delete_existing,
-		final Runnable					done )
+	hardLinkFile(
+		DownloadManager 			manager,
+		DiskManagerFileInfo 		file_info,
+		File						source,
+		File 						target,
+		Runnable					done )
 	{
-		Utils.getOffOfSWTThread(new AERunnable() {
-			@Override
-			public void runSupport(){
-				moveCopyFileSem.reserve();
+			// this behaviour should be put further down in the core but I'd rather not
+			// do so close to release :(
+
+		manager.setUserData("is_changing_links", true);
+
+			// I don't link this one bit, but there's a lot of things I don't like and this isn't the worst
+		
+		manager.setUserData("set_link_dont_delete_existing", true);
+		
+		String failure_msg = null;
+		
+		try{
+			try{
+				Files.createLink( target.toPath(), source.toPath());
 					
-				copyFileSupport(
-					manager, fileInfo, target, dont_delete_existing, 
-					new Runnable()
-					{
-						public void 
-						run()
-						{
-							moveCopyFileSem.release();
-							
-							if ( done != null ){
-								
-								done.run();
-							}
-						};
-					});
+			}catch( Throwable e ){
+			
+				failure_msg =
+					MessageText.getString( 
+						"hardlink.failed.text", 
+						new String[]{ target.toString(), source.toString(), Debug.getNestedExceptionMessage(e) });
 			}
+			
+			if ( failure_msg == null ){
+			
+				boolean ok = file_info.setLink(target);
+	
+				if ( !ok ){
+	
+					target.delete();
+					
+					String error = file_info.getLastError();
+					
+					if ( error == null ){
+						
+						error = MessageText.getString( "SpeedView.stats.unknown" );
+					}
+					
+					failure_msg = 
+						MessageText.getString( 
+							"hardlink.failed.text", 
+							new String[]{ target.toString(), source.toString(), error });
+				}
+			}
+			
+			if ( failure_msg != null ){
+				
+				new MessageBoxShell(
+					SWT.ICON_ERROR | SWT.OK,
+					MessageText.getString("hardlink.failed.title"),
+					failure_msg ).open(
+						new UserPrompterResultListener() {
+	
+							@Override
+							public void prompterClosed(int result) {
+								if ( done != null ){
+									done.run();
+								}
+							}
+						});
+	
+			}
+		}finally{
+			manager.setUserData("is_changing_links", false);
+			manager.setUserData("set_link_dont_delete_existing", null);
+
+			if ( failure_msg == null ){
+
+				if ( done != null ){
+					done.run();
+				}
+			}
+		}
+	}
+	
+	private static void
+	copyFile(
+		DownloadManager 			manager,
+		DiskManagerFileInfo 		fileInfo,
+		File						source,
+		File 						target,
+		boolean						dont_delete_existing,
+		Runnable					done )
+	{
+		moveCopyDispatcher.dispatch(()->{
+					
+			copyFileSupport(
+				manager, fileInfo, source, target, dont_delete_existing, 
+				()->{				
+					if ( done != null ){
+						
+						done.run();
+					}
+				});
 		});
 	}
 	
 	private static void
 	moveFileSupport(
-		final DownloadManager 			manager,
-		final DiskManagerFileInfo 		fileInfo,
-		final File 						target,
-		boolean							dont_delete_existing,
-		final Runnable					done )
+		DownloadManager 			manager,
+		DiskManagerFileInfo 		fileInfo,
+		File						source,
+		File 						target,
+		boolean						dont_delete_existing,
+		Runnable					done )
 	{
 
 		// this behaviour should be put further down in the core but I'd rather not
@@ -1895,7 +2031,7 @@ public class FilesViewMenuUtil
 		}
 
 		try{
-
+			
 			FileUtil.runAsTask(new CoreOperationTask() {
 				
 				@Override
@@ -1912,6 +2048,13 @@ public class FilesViewMenuUtil
 					return( manager );
 				}
 				
+				@Override
+				public String[] 
+				getAffectedFileSystems()
+				{
+					return( FileUtil.getFileStoreNames( source, target ));
+				}
+				
 				@Override
 				public ProgressCallback 
 				getProgressCallback() 
@@ -1978,11 +2121,12 @@ public class FilesViewMenuUtil
 
 	private static void
 	copyFileSupport(
-		final DownloadManager 			manager,
-		final DiskManagerFileInfo 		fileInfo,
-		final File 						target,
-		boolean							dont_delete_existing,
-		final Runnable					done )
+		DownloadManager 			manager,
+		DiskManagerFileInfo 		fileInfo,
+		File						source,
+		File 						target,
+		boolean						dont_delete_existing,
+		Runnable					done )
 	{
 
 		// this behaviour should be put further down in the core but I'd rather not
@@ -2013,6 +2157,13 @@ public class FilesViewMenuUtil
 					return( manager );
 				}
 				
+				@Override
+				public String[] 
+				getAffectedFileSystems()
+				{
+					return( FileUtil.getFileStoreNames( source, target ));
+				}
+
 				@Override
 				public ProgressCallback 
 				getProgressCallback() 
@@ -2084,9 +2235,25 @@ public class FilesViewMenuUtil
 		}
 	}
 	
-	// Returns true if it was paused here.
-	private static boolean setSkipped(DownloadManager manager,
-			DiskManagerFileInfo[] infos, boolean skipped, boolean delete_action, boolean prompt ) {
+		// Returns true if it was paused here.
+	
+	/**
+	 * 
+	 * @param manager
+	 * @param infos
+	 * @param skipped
+	 * @param delete_action  0 - not delete (explicit DND), 1 - delete, 2 - delete if file doesn't exist, DND otherwise
+	 * @param prompt
+	 * @return
+	 */
+	private static boolean 
+	setSkipped(
+		DownloadManager			manager,
+		DiskManagerFileInfo[]	infos, 
+		boolean 				skipped, 
+		int 					delete_action, 
+		boolean 				prompt ) 
+	{
 		// if we're not managing the download then don't do anything other than
 		// change the file's priority
 
@@ -2149,7 +2316,7 @@ public class FilesViewMenuUtil
 				}
 
 				if (perform_check && existing_file.exists()) {
-					if (delete_action) {
+					if (delete_action == 1) {
 						
 						if ( prompt ){
 							MessageBoxShell mb = new MessageBoxShell(SWT.OK | SWT.CANCEL,
@@ -2188,7 +2355,7 @@ public class FilesViewMenuUtil
 				}
 				// File does not exist.
 				else {
-					new_storage_type = delete_action?compact_target:non_compact_target;
+					new_storage_type = delete_action != 0?compact_target:non_compact_target;
 				}
 			}else{
 				new_storage_type = non_compact_target;
@@ -2246,9 +2413,20 @@ public class FilesViewMenuUtil
 		}
 
 		if (ok) {
+			boolean[] toChange = new boolean[nbFiles];
+			
+			for (int i = 0; i < infos.length; i++) {
+				if ( infos[i].isSkipped() != skipped ){
+					toChange[infos[i].getIndex()] = true;
+				}
+			}
+			
+			manager.getDiskManagerFileInfoSet().setSkipped(toChange, skipped );
+			/*
 			for (int i = 0; i < infos.length; i++) {
 				infos[i].setSkipped(skipped);
 			}
+			*/
 		}
 
 		return paused;
@@ -2293,11 +2471,11 @@ public class FilesViewMenuUtil
 	{
 		RevertFileLocationsWindow window = new RevertFileLocationsWindow();
 		
-		window.open(( ok, copy, retain )->{
+		window.open(( ok, hard_link, copy, retain )->{
 				
 			if ( ok ){
 				
-				revertFiles( tv, files, copy, retain );
+				revertFiles( tv, files, hard_link, copy, retain );
 			}
 		});
 	}
@@ -2306,6 +2484,7 @@ public class FilesViewMenuUtil
 	revertFiles(
 		final TableView<?>					tv,
 		List<DiskManagerFileInfo>			files,
+		boolean								hard_link,
 		boolean								copy,
 		boolean								retain_names )
 	{
@@ -2359,50 +2538,45 @@ public class FilesViewMenuUtil
 							
 							affected_files.add( file_info );
 
-							if ( copy ){
+							if ( hard_link ){
+							
+								hardLinkFile(
+									manager,
+									file_info,
+									source,
+									target,
+									()->{ task_sem.release(); });
+								
+							}else if ( copy ){
 								
 								copyFile(
-										manager,
-										file_info,
-										target,
-										true,
-										new Runnable()
-										{
-											@Override
-											public void
-											run()
-											{
-												task_sem.release();													
-											}
-										});
+									manager,
+									file_info,
+									source,
+									target,
+									true,
+									()->{ task_sem.release(); });
 								
 							}else{
 								
 								moveFile(
 									manager,
 									file_info,
+									source,
 									target,
 									true,
-									new Runnable()
-									{
-										@Override
-										public void
-										run()
-										{
-											task_sem.release();													
-										}
-									});
+									()->{ task_sem.release(); });
 							}
 			    		}
 			    	}
 		    	}
 			}
 		}finally{
+			
 			if ( affected_files.size() > 0 ){
 
-				Utils.getOffOfSWTThread(new AERunnable() {
-					@Override
-					public void runSupport() {
+				AEThread2.createAndStartDaemon( "FilesViewWaiter", ()->{
+					
 						for ( int i=0;i<affected_files.size();i++){
 							task_sem.reserve();
 						}
@@ -2411,8 +2585,7 @@ public class FilesViewMenuUtil
 						}
 
 						invalidateRows( tv, affected_files );
-					}
-				});
+					});		
 			}
 		}
 	}
