@@ -21,11 +21,14 @@ package com.biglybt.ui.swt.views;
 
 
 import java.util.*;
+import java.util.regex.Pattern;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.events.SelectionAdapter;
 import org.eclipse.swt.events.SelectionEvent;
 import org.eclipse.swt.graphics.Color;
+import org.eclipse.swt.layout.FormAttachment;
+import org.eclipse.swt.layout.FormData;
 import org.eclipse.swt.layout.FormLayout;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.layout.GridLayout;
@@ -41,27 +44,34 @@ import com.biglybt.core.CoreFactory;
 import com.biglybt.core.config.COConfigurationManager;
 import com.biglybt.core.config.ParameterListener;
 import com.biglybt.core.disk.DiskManager;
+import com.biglybt.core.disk.DiskManagerFileInfo;
 import com.biglybt.core.disk.DiskManagerPiece;
 import com.biglybt.core.disk.DiskManagerReadRequest;
+import com.biglybt.core.disk.impl.piecemapper.DMPieceList;
+import com.biglybt.core.disk.impl.piecemapper.DMPieceMapEntry;
 import com.biglybt.core.download.DownloadManager;
 import com.biglybt.core.global.GlobalManager;
 import com.biglybt.core.peer.PEPeer;
 import com.biglybt.core.peer.PEPeerManager;
 import com.biglybt.core.peer.PEPiece;
 import com.biglybt.core.peermanager.piecepicker.PiecePicker;
+import com.biglybt.core.tracker.AllTrackersManager.AllTrackersTracker;
 import com.biglybt.core.util.CopyOnWriteSet;
 import com.biglybt.core.util.HashWrapper;
 import com.biglybt.core.util.IdentityHashSet;
+import com.biglybt.core.util.RegExUtil;
 import com.biglybt.core.util.SystemTime;
 import com.biglybt.ui.UIFunctions;
 import com.biglybt.ui.UIFunctionsManager;
 import com.biglybt.ui.common.table.*;
+import com.biglybt.ui.common.table.TableViewFilterCheck.TableViewFilterCheckEx;
 import com.biglybt.ui.common.table.impl.TableColumnManager;
 import com.biglybt.ui.common.viewtitleinfo.ViewTitleInfo2;
 import com.biglybt.ui.mdi.MdiEntry;
 import com.biglybt.ui.mdi.MultipleDocumentInterface;
 import com.biglybt.ui.swt.Messages;
 import com.biglybt.ui.swt.Utils;
+import com.biglybt.ui.swt.components.BubbleTextBox;
 import com.biglybt.ui.swt.components.Legend;
 import com.biglybt.ui.swt.mdi.MdiEntrySWT;
 import com.biglybt.ui.swt.mdi.MultipleDocumentInterfaceSWT;
@@ -69,6 +79,7 @@ import com.biglybt.ui.swt.pif.UISWTViewEvent;
 import com.biglybt.ui.swt.pif.UISWTViewEventListener;
 import com.biglybt.ui.swt.pifimpl.UISWTViewBuilderCore;
 import com.biglybt.ui.swt.pifimpl.UISWTViewCoreEventListener;
+import com.biglybt.ui.swt.utils.FontUtils;
 import com.biglybt.ui.swt.views.piece.MyPieceDistributionView;
 import com.biglybt.ui.swt.views.piece.PieceInfoView;
 import com.biglybt.ui.swt.views.table.TableViewSWT;
@@ -96,6 +107,7 @@ public abstract class PiecesViewBase
 		TableViewSWTMenuFillListener,
 		TableSelectionListener,
 		UISWTViewCoreEventListener,
+		TableViewFilterCheck<PEPiece>,
 		ViewTitleInfo2
 {
 	public static final Class<PEPiece> PLUGIN_DS_TYPE = PEPiece.class;
@@ -142,6 +154,7 @@ public abstract class PiecesViewBase
 		return( Legend.getLegendColor( key, legendKeys, BlocksItem.colors ));
 	}
 	
+	private BubbleTextBox bubbleTextBox;
 	
 	protected TableViewSWT<PEPiece> 	tv;
 
@@ -161,6 +174,9 @@ public abstract class PiecesViewBase
 		super( id );
 	}
 	
+	protected abstract String
+	getTableID();
+
 	@Override
 	public Composite 
 	initComposite(
@@ -179,15 +195,21 @@ public abstract class PiecesViewBase
 			parent.setLayoutData(Utils.getFilledFormData());
 		}
 		
+		boolean hasFilter = getTableID() == TableManager.TABLE_ALL_PIECES;
+
 		Composite header = new Composite(parent, SWT.NONE);
-		layout = new GridLayout(1,true);
-		layout.marginHeight = layout.marginWidth = 0;
-		header.setLayout(layout);
+		
+		header.setLayout( new FormLayout());
+		
 		
 		header.setLayoutData(new GridData( GridData.FILL_HORIZONTAL ));
 
 		Button lp_enable = new Button( header, SWT.CHECK );
-		lp_enable.setLayoutData(new GridData( GridData.FILL_HORIZONTAL ));
+		
+		FormData fd = Utils.getFilledFormData();
+		fd.left = new FormAttachment(0, 10);
+		fd.right = null;
+		lp_enable.setLayoutData( fd );
 		
 		Messages.setLanguageText( lp_enable, "label.show.uploading.pieces" );
 		lp_enable.addListener( SWT.Selection, (ev)->{
@@ -215,6 +237,25 @@ public abstract class PiecesViewBase
 					setShowUploading( enabled );
 				}
 			});
+
+		if ( hasFilter ){
+			bubbleTextBox = new BubbleTextBox(header, SWT.BORDER | SWT.SEARCH | SWT.ICON_SEARCH | SWT.ICON_CANCEL | SWT.SINGLE);
+			Composite bubbleTextWidget = bubbleTextBox.getMainWidget();
+			
+			fd = Utils.getFilledFormData();
+			fd.width = 150;
+			fd.top = null;
+			fd.height = (int) (FontUtils.getFontHeightInPX(bubbleTextWidget.getFont()) * 1.4);
+			fd.right = new FormAttachment(100, -10);
+			fd.left = null;
+			bubbleTextWidget.setLayoutData(fd);
+			bubbleTextBox.setAllowRegex( true );
+			
+			if ( tv != null ){
+				
+				tv.enableFilterCheck(bubbleTextBox, this );
+			}
+		}
 		
 		Composite tableParent = new Composite(parent, SWT.NONE);
 		
@@ -251,6 +292,11 @@ public abstract class PiecesViewBase
 					basicItems[0].getName(), SWT.SINGLE | SWT.FULL_SELECTION | SWT.VIRTUAL);
 		}
 
+		if ( bubbleTextBox != null ){
+			
+			tv.enableFilterCheck(bubbleTextBox, this );
+		}
+		
 		registerPluginViews();
 
 		tv.addMenuFillListener(this);
@@ -571,7 +617,76 @@ public abstract class PiecesViewBase
 		}
 	}
 	
+	@Override
+	public void 
+	filterSet(String filter)
+	{	
+	}
 	
+	@Override
+	public boolean
+	filterCheck(
+		PEPiece 	piece, 
+		String 		filter, 
+		boolean 	regex )
+	{
+		if ( filter.isEmpty()){
+			
+			return( true );
+		}
+		
+		List<String>	names = new ArrayList<>();
+		
+		PEPeerManager manager = piece.getManager();
+		
+		if ( manager != null ){
+			
+			names.add( manager.getDisplayName());
+		}
+		
+        DiskManagerPiece dmp = piece.getDMPiece();
+        
+        if ( dmp != null ){
+       	 
+        	DMPieceList l = dmp.getManager().getPieceList( piece.getPieceNumber());
+
+        	for ( int i=0;i<l.size();i++) {
+
+        		DMPieceMapEntry entry = l.get( i );
+
+        		String name = entry.getFile().getTorrentFile().getRelativePath();
+        		
+        		names.add( name );
+        	}
+        }
+
+		String s = regex ? filter : RegExUtil.splitAndQuote( filter, "\\s*[|;]\\s*" );
+
+		boolean	match_result = true;
+
+		if ( regex && s.startsWith( "!" )){
+
+			s = s.substring(1);
+
+			match_result = false;
+		}
+
+		Pattern pattern = RegExUtil.getCachedPattern( "piecesview:search", s, Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE );
+
+		boolean result = !match_result;
+		
+		for ( String name: names ){
+		
+			if ( pattern.matcher(name).find()){
+			
+				result = match_result;
+				
+				break;
+			}
+		}
+		
+		return( result );
+	}
 	
 	protected boolean
 	updateUploadingPieces(
