@@ -21,12 +21,15 @@
 package com.biglybt.ui.swt.views.columnsetup;
 
 import java.util.*;
+import java.util.List;
+import java.util.function.Consumer;
 import java.util.regex.Pattern;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.dnd.*;
 import org.eclipse.swt.events.*;
 import org.eclipse.swt.graphics.GC;
+import org.eclipse.swt.graphics.Point;
 import org.eclipse.swt.graphics.Rectangle;
 import org.eclipse.swt.layout.*;
 import org.eclipse.swt.widgets.*;
@@ -40,13 +43,16 @@ import com.biglybt.ui.common.table.impl.TableColumnManager;
 import com.biglybt.ui.common.updater.UIUpdatable;
 import com.biglybt.ui.swt.Messages;
 import com.biglybt.ui.swt.Utils;
+import com.biglybt.ui.swt.Utils.ColorButton;
 import com.biglybt.ui.swt.components.BubbleTextBox;
 import com.biglybt.ui.swt.components.shell.ShellFactory;
 import com.biglybt.ui.swt.imageloader.ImageLoader;
 import com.biglybt.ui.swt.mainwindow.ClipboardCopy;
+import com.biglybt.ui.swt.mainwindow.Colors;
 import com.biglybt.ui.swt.shells.GCStringPrinter;
 import com.biglybt.ui.swt.shells.MessageBoxShell;
 import com.biglybt.ui.swt.uiupdater.UIUpdaterSWT;
+import com.biglybt.ui.swt.utils.FontUtils;
 import com.biglybt.ui.swt.views.table.TableRowSWT;
 import com.biglybt.ui.swt.views.table.TableViewSWT;
 import com.biglybt.ui.swt.views.table.impl.TableViewFactory;
@@ -81,7 +87,7 @@ public class TableColumnSetupWindow
 
 	private Composite cCategories;
 
-	private TableViewSWT tvChosen;
+	private TableViewSWT<TableColumnCore> tvChosen;
 
 	private Composite cTableChosen;
 
@@ -111,8 +117,14 @@ public class TableColumnSetupWindow
 	private Button btnApply;
 	private Button btnExport;
 		
-	public TableColumnSetupWindow(final Class<?> forDataSourceType, String _tableID,
-			TableRow sampleRow, TableStructureModificationListener<?> _listener) {
+	public 
+	TableColumnSetupWindow(
+		Class<?> 		forDataSourceType, 
+		String 			_tableID,
+		TableColumnCore	selectedColumn,
+		TableRow 		sampleRow, 
+		TableStructureModificationListener<?> _listener) 
+	{
 		this.sampleRow = sampleRow;
 		this.listener = _listener;
 		FormData fd;
@@ -291,12 +303,14 @@ public class TableColumnSetupWindow
 		cTableAvail.setLayout(gridLayout);
 
 		BubbleTextBox bubbleTextBox = new BubbleTextBox(cTableAvail, SWT.BORDER | SWT.SEARCH | SWT.ICON_SEARCH | SWT.ICON_CANCEL | SWT.SINGLE);
-		bubbleTextBox.getTextWidget().setMessage(MessageText.getString("column.setup.search"));
+		bubbleTextBox.setMessage(MessageText.getString("column.setup.search"));
 		GridData gd = new GridData(SWT.RIGHT,SWT.CENTER,true,false);
-		bubbleTextBox.getParent().setLayoutData( gd );
+		gd.widthHint = 150;
+		gd.heightHint = (int) (FontUtils.getFontHeightInPX(bubbleTextBox.getTextWidget().getFont()) * 1.4);
+		bubbleTextBox.getMainWidget().setLayoutData( gd );
 
 		tvAvail.enableFilterCheck(
-			bubbleTextBox.getTextWidget(),
+			bubbleTextBox,
 			new TableViewFilterCheck<TableColumn>()
 			{
 				@Override
@@ -333,7 +347,7 @@ public class TableColumnSetupWindow
 							continue;
 						}
 
-						String s = regex ? filter : "\\Q" + filter.replaceAll("[|;]", "\\\\E|\\\\Q") + "\\E";
+						String s = regex ? filter : RegExUtil.splitAndQuote( filter, "\\s*[|;]\\s*" );
 
 						boolean	match_result = true;
 
@@ -533,12 +547,22 @@ public class TableColumnSetupWindow
 
 		ImageLoader imageLoader = ImageLoader.getInstance();
 
+		Button[] alignButtons = new Button[3];
+		
+		Consumer<Integer> updateAlignButtons = (align)->{
+			alignButtons[0].setBackground( align==TableColumn.ALIGN_LEAD?Colors.fadedBlue:null);
+			alignButtons[1].setBackground( align==TableColumn.ALIGN_CENTER?Colors.fadedBlue:null);
+			alignButtons[2].setBackground( align==TableColumn.ALIGN_TRAIL?Colors.fadedBlue:null);
+		};
+		
 		Button btnLeft = new Button(cColumnButtonArea, SWT.PUSH);
 		imageLoader.setButtonImage(btnLeft, "alignleft");
+		Messages.setLanguageTooltip(btnLeft, "MyTracker.column.left" );
 		btnLeft.addSelectionListener(new SelectionListener() {
 			@Override
 			public void widgetSelected(SelectionEvent e) {
 				alignChosen( TableColumnCore.ALIGN_LEAD);
+				updateAlignButtons.accept( TableColumnCore.ALIGN_LEAD );
 			}
 
 			@Override
@@ -548,10 +572,12 @@ public class TableColumnSetupWindow
 
 		Button btnCentre = new Button(cColumnButtonArea, SWT.PUSH);
 		imageLoader.setButtonImage(btnCentre, "aligncentre");
+		Messages.setLanguageTooltip(btnCentre, "label.center" );
 		btnCentre.addSelectionListener(new SelectionListener() {
 			@Override
 			public void widgetSelected(SelectionEvent e) {
 				alignChosen( TableColumnCore.ALIGN_CENTER );
+				updateAlignButtons.accept( TableColumnCore.ALIGN_CENTER );
 			}
 
 			@Override
@@ -561,10 +587,13 @@ public class TableColumnSetupWindow
 
 		Button btnRight = new Button(cColumnButtonArea, SWT.PUSH);
 		imageLoader.setButtonImage(btnRight, "alignright");
+		Messages.setLanguageTooltip(btnRight, "label.right" );
+		
 		btnRight.addSelectionListener(new SelectionListener() {
 			@Override
 			public void widgetSelected(SelectionEvent e) {
 				alignChosen( TableColumnCore.ALIGN_TRAIL );
+				updateAlignButtons.accept( TableColumnCore.ALIGN_TRAIL );
 			}
 
 			@Override
@@ -572,8 +601,32 @@ public class TableColumnSetupWindow
 			}
 		});
 
+		alignButtons[0] = btnLeft;
+		alignButtons[1] = btnCentre;
+		alignButtons[2] = btnRight;
+		
+		ColorButton colorForeground = 
+				Utils.createColorButton( 
+					cColumnButtonArea, new Point( 16, 16 ), true, null, null,
+					(rgb)->{
+						setChosenColor( rgb, true );
+					});
+				
+		Button btnForeground = colorForeground.getButton();
+		
+		ColorButton colorBackground = 
+				Utils.createColorButton( 
+					cColumnButtonArea, new Point( 16, 16 ), false, null, null,
+					(rgb)->{
+						setChosenColor( rgb, false );
+					});
+				
+		Button btnBackground= colorBackground.getButton();
+
+		
 		Button btnUp = new Button(cColumnButtonArea, SWT.PUSH);
 		imageLoader.setButtonImage(btnUp, "up");
+		Messages.setLanguageTooltip(btnUp, "label.move.up" );
 		btnUp.addSelectionListener(new SelectionListener() {
 			@Override
 			public void widgetSelected(SelectionEvent e) {
@@ -587,6 +640,7 @@ public class TableColumnSetupWindow
 
 		Button btnDown = new Button(cColumnButtonArea, SWT.PUSH);
 		imageLoader.setButtonImage(btnDown, "down");
+		Messages.setLanguageTooltip(btnDown, "label.move.down" );
 		btnDown.addSelectionListener(new SelectionListener() {
 			@Override
 			public void widgetSelected(SelectionEvent e) {
@@ -600,6 +654,7 @@ public class TableColumnSetupWindow
 
 		Button btnDel = new Button(cColumnButtonArea, SWT.PUSH);
 		imageLoader.setButtonImage(btnDel, "delete2");
+		Messages.setLanguageTooltip(btnDel, "MySharesView.menu.remove" );
 		btnDel.addSelectionListener(new SelectionListener() {
 			@Override
 			public void widgetSelected(SelectionEvent e) {
@@ -648,16 +703,38 @@ public class TableColumnSetupWindow
 			{
 				@Override
 				public void selectionChanged(TableRowCore[] selected_rows, TableRowCore[] deselected_rows){
-					
-					boolean hasSelection = !tvChosen.getSelectedDataSources().isEmpty();
-					
+										
 					Utils.execSWTThread(()->{
+						
+						List<Object> ds = tvChosen.getSelectedDataSources();
+						
+						boolean hasSelection = ds.size() > 0;
+
 						btnLeft.setEnabled( hasSelection );
 						btnCentre.setEnabled( hasSelection );
 						btnRight.setEnabled( hasSelection );
+						btnForeground.setEnabled( hasSelection );
+						btnBackground.setEnabled( hasSelection );
 						btnUp.setEnabled( hasSelection );
 						btnDown.setEnabled( hasSelection );
 						btnDel.setEnabled( hasSelection );
+						
+						if ( hasSelection && ds.size() == 1 ){
+							
+							TableColumnCore tc = (TableColumnCore)ds.get(0);
+							
+							colorForeground.setColor(tc.getForegroundColor());
+							colorBackground.setColor(tc.getBackgroundColor());
+							
+							int align = tc.getAlignment();
+							
+							updateAlignButtons.accept( align );
+						}else{
+							colorForeground.setColor( null );
+							colorBackground.setColor( null );
+							
+							updateAlignButtons.accept( -1 );
+						}
 					});
 				}
 			}, true);
@@ -675,9 +752,20 @@ public class TableColumnSetupWindow
 				tvChosen.addDataSource(columnsCurrentOrder[i]);
 			}
 		}
-		tvChosen.processDataSourceQueue();
-
+		tvChosen.processDataSourceQueueSync();
 
+		if ( selectedColumn != null ){
+			
+			TableRowCore row = tvChosen.getRow( selectedColumn );
+			
+			if ( row != null ){
+				
+				tvChosen.setSelectedRows( new TableRowCore[]{ row } );
+				
+				Utils.execSWTThreadLater( 100, ()->{ tvChosen.showRow( row ); });
+			}
+		}
+		
 		Button btnReset = new Button(cResultButtonArea, SWT.PUSH);
   		Messages.setLanguageText(btnReset, "Button.reset");
   		
@@ -808,6 +896,10 @@ public class TableColumnSetupWindow
 		  						setHasChanges( false );
 		  						
 		  						btnCancel.setEnabled(false);
+		  						
+		  						updateAlignButtons.accept( -1 );
+		  						colorForeground.setColor( null );
+								colorBackground.setColor( null );
 							}
 						}
 					});
@@ -867,10 +959,25 @@ public class TableColumnSetupWindow
 		fd.bottom = new FormAttachment(btnLeft, 0, SWT.BOTTOM);
 		btnRight.setLayoutData(fd);
 
+			// colours
+		
+		fd = new FormData();
+		fd.left = new FormAttachment(btnRight, 3);
+		fd.top = new FormAttachment(btnLeft, 0, SWT.TOP);
+		fd.bottom = new FormAttachment(btnLeft, 0, SWT.BOTTOM);
+		btnForeground.setLayoutData(fd);
+
+		fd = new FormData();
+		fd.left = new FormAttachment(btnForeground, 3);
+		fd.top = new FormAttachment(btnLeft, 0, SWT.TOP);
+		fd.bottom = new FormAttachment(btnLeft, 0, SWT.BOTTOM);
+		btnBackground.setLayoutData(fd);
+
+
 			// move
 
 		fd = new FormData();
-		fd.left = new FormAttachment(0, 3);
+		fd.left = new FormAttachment(btnCentre, 0, SWT.LEFT );
 		fd.top = new FormAttachment(btnLeft, 2);
 		btnUp.setLayoutData(fd);
 
@@ -1105,11 +1212,13 @@ public class TableColumnSetupWindow
 	 */
 	protected void removeSelectedChosen() {
 		Object[] datasources = tvChosen.getSelectedDataSources().toArray();
+		TableColumnCore[] cols = new TableColumnCore[datasources.length];
 		for (int i = 0; i < datasources.length; i++) {
 			TableColumnCore column = (TableColumnCore) datasources[i];
+			cols[i] = column;
 			mapNewVisibility.put(column, Boolean.FALSE);
 		}
-		tvChosen.removeDataSources(datasources);
+		tvChosen.removeDataSources(cols);
 		tvChosen.processDataSourceQueue();
 		for (int i = 0; i < datasources.length; i++) {
 			TableRowSWT row = (TableRowSWT) tvAvail.getRow((TableColumn)datasources[i]);
@@ -1196,6 +1305,24 @@ public class TableColumnSetupWindow
 		setHasChanges( true );
 	}
 
+	protected void setChosenColor( int[] rgb, boolean fg ) {
+		TableRowCore[] selectedRows = tvChosen.getSelectedRows();
+		for (int i = 0; i < selectedRows.length; i++) {
+			TableRowCore row = selectedRows[i];
+			TableColumnCore column = (TableColumnCore) row.getDataSource();
+			if (column != null) {
+				if ( fg ){
+					column.setForegroundColor(rgb);
+				}else{
+					column.setBackgroundColor(rgb);
+				}
+			}
+		}
+		tvChosen.tableInvalidate();
+		tvChosen.refreshTable(true);
+		
+		setHasChanges( true );
+	}
 
 	/**
 	 *
@@ -1221,7 +1348,7 @@ public class TableColumnSetupWindow
 	 *
 	 * @since 4.0.0.5
 	 */
-	private TableViewSWT<?> createTVChosen() {
+	private TableViewSWT<TableColumnCore> createTVChosen() {
 		final TableColumnManager tcm = TableColumnManager.getInstance();
 		TableColumnCore[] columnTVChosen = tcm.getAllTableColumnCoreAsArray(
 				TableColumn.class, TABLEID_CHOSEN);
@@ -1236,7 +1363,7 @@ public class TableColumnSetupWindow
 			}
 		}
 
-		final TableViewSWT<?> tvChosen = TableViewFactory.createTableViewSWT(
+		final TableViewSWT<TableColumnCore> tvChosen = TableViewFactory.createTableViewSWT(
 				TableColumn.class, TABLEID_CHOSEN, TABLEID_CHOSEN, columnTVChosen,
 				ColumnTC_ChosenColumn.COLUMN_ID, SWT.FULL_SELECTION | SWT.VIRTUAL
 						| SWT.MULTI);
@@ -1576,5 +1703,15 @@ public class TableColumnSetupWindow
 				}
 			}
 		}
+		
+		if ( tvAvail != null ){
+			
+			tvAvail.delete();
+		}
+		
+		if ( tvChosen != null ){
+			
+			tvChosen.delete();
+		}
 	}
 }
