@@ -32,8 +32,8 @@ ConfigSectionInterfaceTagsSWT
 	extends ConfigSectionInterfaceTags
 {
 	@Override
-	public void 
-	build() 
+	protected void 
+	buildUISpecific() 
 	{
 		boolean isAZ3 = Utils.isAZ3UI();
 		
@@ -116,7 +116,5 @@ ConfigSectionInterfaceTagsSWT
 		
 		add(new ParameterGroupImpl("ConfigView.section.style.library",
 				listLibrary));
-
-		super.build();
 	}
 }
