@@ -1,4 +1,4 @@
-/* 
+ /* 
  * Copyright (C) Bigly Software, Inc, All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -71,6 +71,7 @@ import com.biglybt.pif.ui.UIInputReceiverListener;
 import com.biglybt.pif.ui.UIPluginViewToolBarListener;
 import com.biglybt.pif.ui.tables.TableColumn;
 import com.biglybt.pif.ui.tables.TableColumnCreationListener;
+import com.biglybt.pif.ui.toolbar.UIToolBarItem;
 import com.biglybt.plugin.net.buddy.BuddyPluginUtils;
 
 
@@ -85,8 +86,6 @@ public class SBC_AllTrackersView
 
 	TableViewSWT<AllTrackersViewEntry> tv;
 
-	private Text txtFilter;
-
 	private Composite table_parent;
 
 	private boolean columnsAdded = false;
@@ -255,7 +254,6 @@ public class SBC_AllTrackersView
 						new ColumnAllTrackersResponseTime(column);
 					}
 				});
-
 		
 		tableManager.registerColumn(AllTrackersViewEntry.class, ColumnAllTrackersHasPrivate.COLUMN_ID,
 				new TableColumnCreationListener() {
@@ -264,6 +262,14 @@ public class SBC_AllTrackersView
 						new ColumnAllTrackersHasPrivate(column);
 					}
 				});
+		
+		tableManager.registerColumn(AllTrackersViewEntry.class, ColumnAllTrackersRemovable.COLUMN_ID,
+				new TableColumnCreationListener() {
+					@Override
+					public void tableColumnCreated(TableColumn column) {
+						new ColumnAllTrackersRemovable(column);
+					}
+				});
 
 		
 		tableManager.setDefaultColumnNames(TABLE_NAME,
@@ -316,13 +322,6 @@ public class SBC_AllTrackersView
 	{
 		super.skinObjectShown( skinObject, params );
 
-		SWTSkinObjectTextbox soFilter = (SWTSkinObjectTextbox)getSkinObject( "filterbox" );
-
-		if ( soFilter != null ){
-
-			txtFilter = soFilter.getTextControl();
-		}
-
 		SWTSkinObject so_list = getSkinObject( "all-trackers-list" );
 
 		if ( so_list != null ){
@@ -383,9 +382,10 @@ public class SBC_AllTrackersView
 					ColumnAllTrackersTracker.COLUMN_ID,
 					SWT.MULTI | SWT.FULL_SELECTION | SWT.VIRTUAL);
 
-			if ( txtFilter != null){
-
-				tv.enableFilterCheck( txtFilter, this );
+			SWTSkinObjectTextbox soFilter = (SWTSkinObjectTextbox) getSkinObject(
+				"filterbox");
+			if (soFilter != null) {
+				tv.enableFilterCheck(soFilter.getBubbleTextBox(), this);
 			}
 
 			tv.setRowDefaultHeightEM(1);
@@ -520,30 +520,25 @@ public class SBC_AllTrackersView
 			return( false );
 		}
 
-		/*
+		boolean didSomething = false;
+		
 		List<Object> datasources = tv.getSelectedDataSources();
 
-		if ( datasources.size() > 0 ){
-
-			List<AllTrackersViewEntry>	dms = new ArrayList<>(datasources.size());
-
-			for ( Object o: datasources ){
-
-				dms.add((AllTrackersViewEntry)o);
-			}
-
+		for ( Object o: datasources ){
+			
+			AllTrackersViewEntry entry = (AllTrackersViewEntry)o;
+			
 			String id = item.getID();
 
 			if ( id.equals("remove")) {
 
+				entry.remove();
+				
+				didSomething = true;
 			}
-
-
-			return true;
-		}
-		*/
+		}		
 		
-		return false;
+		return( didSomething );
 	}
 
 	@Override
@@ -557,19 +552,21 @@ public class SBC_AllTrackersView
 			return;
 		}
 
-		boolean canEnable = false;
+		List<Object> datasources = tv.getSelectedDataSources();
+		
+		boolean canRemove = !datasources.isEmpty();
 
-		Object[] datasources = tv.getSelectedDataSources().toArray();
-
-		if ( datasources.length > 0 ){
-
-			canEnable = true;
+		for ( Object o: datasources ){
+			
+			AllTrackersViewEntry entry = (AllTrackersViewEntry)o;
+			
+			if ( !entry.isRemovable()){
+				
+				canRemove = false;
+			}
 		}
 
-		//list.put( "start", canEnable ? UIToolBarItem.STATE_ENABLED : 0);
-		//list.put( "startstop", canEnable ? UIToolBarItem.STATE_ENABLED : 0);
-
-		//list.put( "remove", canEnable ? UIToolBarItem.STATE_ENABLED : 0);
+		list.put( "remove", canRemove ? UIToolBarItem.STATE_ENABLED : 0);
 	}
 	
 	private MdiEntrySWT getActiveView() {
@@ -870,6 +867,8 @@ public class SBC_AllTrackersView
 
 		Set<String>	assoc_templates = new TreeSet<>();
 		
+		boolean canRemove = hasSelection;
+		
 		for ( AllTrackersTracker tracker: trackers ){
 			
 			String str = tracker.getTrackerName().toLowerCase( Locale.US );
@@ -880,6 +879,11 @@ public class SBC_AllTrackersView
 				
 				assoc_templates.addAll( list );
 			}
+			
+			if ( !tracker.isRemovable()){
+				
+				canRemove = false;
+			}
 		}
 		
 		if ( !assoc_templates.isEmpty()){
@@ -1015,8 +1019,6 @@ public class SBC_AllTrackersView
 			// options
 		
 		Menu options_menu = new Menu( menu.getShell(), SWT.DROP_DOWN);
-
-		boolean opt_enabled = trackers.size() == 1;
 				
 		MenuItem options_item = new MenuItem( menu, SWT.CASCADE);
 
@@ -1024,64 +1026,149 @@ public class SBC_AllTrackersView
 
 		options_item.setMenu( options_menu );
 		
-		MenuItem itemOptCryptoPort = new MenuItem(options_menu, SWT.CHECK);
+			// light seeding
+		
+		Menu ls_menu = new Menu( menu.getShell(), SWT.DROP_DOWN);
+				
+		MenuItem ls_item = new MenuItem( options_menu, SWT.CASCADE);
 
-		Messages.setLanguageText( itemOptCryptoPort, "menu.tracker.cryptoport.disabled" );
+		Messages.setLanguageText( ls_item, "ManagerItem.lightseeding" );
 
-		itemOptCryptoPort.setEnabled( opt_enabled );
+		ls_item.setMenu( ls_menu );
+			
+		MenuItem ls_default = new MenuItem( ls_menu, SWT.RADIO );
+		Messages.setLanguageText( ls_default, "label.default" );
+		
+		MenuItem ls_enabled = new MenuItem( ls_menu, SWT.RADIO );
+		Messages.setLanguageText( ls_enabled, "label.enabled" );
+		
+		MenuItem ls_disabled = new MenuItem( ls_menu, SWT.RADIO );
+		Messages.setLanguageText( ls_disabled, "label.disabled" );
+		
+		MenuItem[] ls_items = { ls_default, ls_enabled, ls_disabled };
+		
+			// cryptoport
+		
+		Menu cp_menu = new Menu( menu.getShell(), SWT.DROP_DOWN);
+		
+		MenuItem cp_item = new MenuItem( options_menu, SWT.CASCADE);
+
+		cp_item.setText( "cryptoport" );
+
+		cp_item.setMenu( cp_menu );
+			
+		MenuItem cp_default = new MenuItem( cp_menu, SWT.RADIO );
+		Messages.setLanguageText( cp_default, "label.default" );
+		
+		MenuItem cp_enabled = new MenuItem( cp_menu, SWT.RADIO );
+		Messages.setLanguageText( cp_enabled, "label.enabled" );
+		
+		MenuItem cp_disabled = new MenuItem( cp_menu, SWT.RADIO );
+		Messages.setLanguageText( cp_disabled, "label.disabled" );
+
+		MenuItem[] cp_items = { cp_default, cp_enabled, cp_disabled };
+	
+		boolean opt_enabled = trackers.size() == 1;
+
+		ls_item.setEnabled( opt_enabled );
+		cp_item.setEnabled( opt_enabled );
 		
 		if ( opt_enabled ){
 			
 			AllTrackersTracker tracker = trackers.get(0);
 			
+			/* allow configuration in the absence of torrents, doesn't harm anything as
+			 * if the tracker ends up being public it light seeding is ignored anyway
+			if ( tracker.getPrivatePercentage() == 0 ){
+				
+				ls_item.setEnabled( false );
+			}
+			*/
+			
 			Map<String,Object> init_options = tracker.getOptions();
 			
+			int	ls_state 	= 0;
+			int cp_state	= 0;
+			
 			if ( init_options != null ){
 				
-				Number opt = (Number)init_options.get( AllTrackersTracker.OPT_CRYPTO_PORT );
+				Number opt = (Number)init_options.get( AllTrackersTracker.OPT_LIGHT_SEEDING );
 				
-				if ( opt != null && opt.intValue() == 2 ){
+				if ( opt != null ){
 					
-					itemOptCryptoPort.setSelection( true );
+					ls_state = opt.intValue();
+				}
+				
+				opt = (Number)init_options.get( AllTrackersTracker.OPT_CRYPTO_PORT );
+				
+				if ( opt != null ){
+					
+					cp_state = opt.intValue();
 				}
 			}
+			
+			ls_items[ls_state].setSelection( true );
+			cp_items[cp_state].setSelection( true );
 		
-			itemOptCryptoPort.addListener(
-				SWT.Selection,
-				(e)->{
+			Listener l = (e)->{
 										
-					Map<String,Object> options = tracker.getOptions();
-					
-					boolean	sel = itemOptCryptoPort.getSelection();
-					
-					if ( sel ){
-						
-						if ( options == null ){
-							
-							options = new HashMap<>();
-						}
+				Map<String,Object> options = tracker.getOptions();
 				
-						options.put( AllTrackersTracker.OPT_CRYPTO_PORT, 2 );
-						
-						tracker.setOptions( options );
-						
-					}else{
-						
-						if ( options != null ){
-							
-							options.remove( AllTrackersTracker.OPT_CRYPTO_PORT );
-							
-							if ( options.isEmpty()){
-								
-								options = null;
-							}
-							
-							tracker.setOptions( options );
-						}
+				if ( options == null ){
+					
+					options = new HashMap<>();
+				}
+				
+				int new_ls_state = 0;
+				int new_cp_state = 0;
+				
+				for ( int i=0; i<ls_items.length;i++){
+					if ( ls_items[i].getSelection()){
+						new_ls_state = i;
+						break;
 					}
-				});
+				}
+				
+				for ( int i=0; i<cp_items.length;i++){
+					if ( cp_items[i].getSelection()){
+						new_cp_state = i;
+						break;
+					}
+				}
+													
+				options.put( AllTrackersTracker.OPT_LIGHT_SEEDING, new_ls_state );
+					
+				options.put( AllTrackersTracker.OPT_CRYPTO_PORT, new_cp_state );
+				
+				tracker.setOptions( options );
+			};
+			
+			for ( MenuItem mi: ls_items ){
+				mi.addListener( SWT.Selection, l );
+			}
+			
+			for ( MenuItem mi: cp_items ){
+				mi.addListener( SWT.Selection, l );
+			}
 		}
 		
+			// remove
+
+		MenuItem itemRemove = new MenuItem(menu, SWT.PUSH );
+
+		Messages.setLanguageText(itemRemove, "MySharesView.menu.remove");
+		Utils.setMenuItemImage(itemRemove, "delete");
+
+		itemRemove.addListener(SWT.Selection, (ev)->{
+			
+			for ( AllTrackersTracker t: trackers ){
+				
+				t.remove();
+			}			
+		});
+		
+		itemRemove.setEnabled( canRemove );
+		
 			// edit templates
 		
 		new MenuItem( menu, SWT.SEPARATOR );
@@ -1146,9 +1233,14 @@ public class SBC_AllTrackersView
 					
 					String[] val = prop.getStringList();
 					
-					has_any_constraint = val.length > 0;
+					if ( val != null && val.length > 0 ){
+						
+						String constraint = val[0];
+											
+						has_any_constraint = constraint != null && !constraint.trim().isEmpty();
 					
-					has_true_constraint = has_any_constraint && val[0].equalsIgnoreCase( "true" );
+						has_true_constraint = has_any_constraint && constraint.equalsIgnoreCase( "true" );
+					}
 				}
 			}
 			
@@ -1509,7 +1601,7 @@ public class SBC_AllTrackersView
 					}
 				}
 			}
-		}else{
+		}else if ( type == AllTrackersEvent.ET_TRACKER_REMOVED ){
 			
 			tv.removeDataSources( entries.toArray( new AllTrackersViewEntry[0] ));
 		}
@@ -1547,7 +1639,7 @@ public class SBC_AllTrackersView
 		
 		String name = tracker.getTrackerName();
 
-		String s = regex ? filter : "\\Q" + filter.replaceAll("\\s*[|;]\\s*", "\\\\E|\\\\Q") + "\\E";
+		String s = regex ? filter : RegExUtil.splitAndQuote( filter, "\\s*[|;]\\s*" );
 
 		boolean	match_result = true;
 
@@ -1562,7 +1654,7 @@ public class SBC_AllTrackersView
 
 		boolean result = pattern.matcher(name).find() == match_result;
 		
-		if ( !result ){
+		if ( result != match_result ){
 			
 			String status = tracker.getStatusString();
 			
@@ -1738,5 +1830,19 @@ public class SBC_AllTrackersView
 		{
 			return( getSelectionTag());
 		}
+		
+		@Override
+		public boolean 
+		isRemovable()
+		{
+			return( tracker.isRemovable());
+		}
+		
+		@Override
+		public void 
+		remove()
+		{
+			tracker.remove();
+		}
 	}
 }
