@@ -80,8 +80,6 @@ public class SBC_ArchivedDownloadsView
 
 	TableViewSWT<DownloadStub> tv;
 
-	private Text txtFilter;
-
 	private Composite table_parent;
 
 	private boolean columnsAdded = false;
@@ -212,6 +210,23 @@ public class SBC_ArchivedDownloadsView
 					}
 				});
 
+		tableManager.registerColumn(DownloadStub.class,
+				ColumnArchiveDLCompDate.COLUMN_ID,
+				new TableColumnCoreCreationListener() {
+					@Override
+					public TableColumnCore createTableColumnCore(
+							Class<?> forDataSourceType, String tableID, String columnID) {
+						return new ColumnDateSizer(DownloadStub.class, columnID,
+								TableColumnCreator.DATE_COLUMN_WIDTH, tableID) {
+						};
+					}
+
+					@Override
+					public void tableColumnCreated(TableColumn column) {
+						new ColumnArchiveDLCompDate(column);
+					}
+				});
+		
 		tableManager.setDefaultColumnNames(TABLE_NAME,
 				new String[] {
 					ColumnArchiveDLName.COLUMN_ID,
@@ -259,13 +274,6 @@ public class SBC_ArchivedDownloadsView
 	{
 		super.skinObjectShown( skinObject, params );
 
-		SWTSkinObjectTextbox soFilter = (SWTSkinObjectTextbox)getSkinObject( "filterbox" );
-
-		if ( soFilter != null ){
-
-			txtFilter = soFilter.getTextControl();
-		}
-
 		SWTSkinObject so_list = getSkinObject( "archived-dls-list" );
 
 		if ( so_list != null ){
@@ -320,9 +328,10 @@ public class SBC_ArchivedDownloadsView
 					TABLE_NAME, new TableColumnCore[0], ColumnArchiveDLName.COLUMN_ID,
 					SWT.MULTI | SWT.FULL_SELECTION | SWT.VIRTUAL);
 
-			if ( txtFilter != null){
-
-				tv.enableFilterCheck( txtFilter, this );
+			SWTSkinObjectTextbox soFilter = (SWTSkinObjectTextbox) getSkinObject(
+				"filterbox");
+			if (soFilter != null) {
+				tv.enableFilterCheck(soFilter.getBubbleTextBox(), this);
 			}
 
 			tv.setRowDefaultHeightEM(1);
@@ -734,28 +743,41 @@ public class SBC_ArchivedDownloadsView
 		String 			filter,
 		boolean 		regex)
 	{
-		if ( filter.toLowerCase( Locale.US ).startsWith( "f:" )){
+		boolean do_files = filter.toLowerCase( Locale.US ).startsWith( "f:" );
+		
+		if ( do_files ){
 
 			filter = filter.substring(2).trim();
+		}
+		
+		boolean	match_result = true;
 
-			DownloadStubFile[] files = ds.getStubFiles();
-
-			String s = regex ? filter : "\\Q" + filter.replaceAll("\\s*[|;]\\s*", "\\\\E|\\\\Q") + "\\E";
-
-			boolean	match_result = true;
-
-			if ( regex && s.startsWith( "!" )){
-
-				s = s.substring(1);
+		String expr;
+		
+		if ( regex ){
+			
+			expr = filter;
+			
+			if ( expr.startsWith( "!" )){
+				
+				expr = expr.substring(1);
 
 				match_result = false;
 			}
+		}else{
+			
+			expr = RegExUtil.convertAndOrToExpr( filter );
+		}
 
-			Pattern pattern = RegExUtil.getCachedPattern( "archiveview:search", s, Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE );
+		if ( do_files ){
 
+			DownloadStubFile[] files = ds.getStubFiles();
+
+			Pattern pattern = RegExUtil.getCachedPattern( "archiveview:search", expr, Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE );
 
 			boolean result = !match_result;
 
+			boolean	try_save_path = true;
 
 			for ( DownloadStubFile file: files ){
 
@@ -765,28 +787,27 @@ public class SBC_ArchivedDownloadsView
 
 					result = match_result;
 
+					try_save_path = false;
+					
 					break;
 				}
 			}
 
+			if ( try_save_path ){
+				
+				if ( pattern.matcher( ds.getSavePath()).find()){
+
+					result = match_result;
+				}
+			}
+			
 			return( result );
 
 		}else{
 
 			String name = ds.getName();
 
-			String s = regex ? filter : "\\Q" + filter.replaceAll("\\s*[|;]\\s*", "\\\\E|\\\\Q") + "\\E";
-
-			boolean	match_result = true;
-
-			if ( regex && s.startsWith( "!" )){
-
-				s = s.substring(1);
-
-				match_result = false;
-			}
-
-			Pattern pattern = RegExUtil.getCachedPattern( "archiveview:search", s, Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE );
+			Pattern pattern = RegExUtil.getCachedPattern( "archiveview:search", expr, Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE );
 
 			return( pattern.matcher(name).find() == match_result );
 		}
