@@ -42,7 +42,6 @@ import org.eclipse.swt.widgets.Event;
 import org.eclipse.swt.widgets.Listener;
 import org.eclipse.swt.widgets.Menu;
 import org.eclipse.swt.widgets.MenuItem;
-import org.eclipse.swt.widgets.Text;
 import com.biglybt.core.config.COConfigurationManager;
 import com.biglybt.core.download.DownloadManager;
 import com.biglybt.core.global.GlobalManagerEvent;
@@ -88,8 +87,6 @@ public class SBC_DownloadHistoryView
 
 	private TableViewSWT<DownloadHistory> tv;
 
-	private Text txtFilter;
-
 	private Composite table_parent;
 
 	private boolean columnsAdded = false;
@@ -264,13 +261,6 @@ public class SBC_DownloadHistoryView
 	{
 		super.skinObjectShown( skinObject, params );
 
-		SWTSkinObjectTextbox soFilter = (SWTSkinObjectTextbox)getSkinObject( "filterbox" );
-
-		if ( soFilter != null ){
-
-			txtFilter = soFilter.getTextControl();
-		}
-
 		SWTSkinObject so_list = getSkinObject( "dl-history-list" );
 
 		if ( so_list != null ){
@@ -328,9 +318,10 @@ public class SBC_DownloadHistoryView
 					ColumnDLHistoryName.COLUMN_ID,
 					SWT.MULTI | SWT.FULL_SELECTION | SWT.VIRTUAL);
 
-			if ( txtFilter != null){
-
-				tv.enableFilterCheck( txtFilter, this );
+			SWTSkinObjectTextbox soFilter = (SWTSkinObjectTextbox) getSkinObject(
+				"filterbox");
+			if (soFilter != null) {
+				tv.enableFilterCheck(soFilter.getBubbleTextBox(), this);
 			}
 
 			tv.setRowDefaultHeightEM(1);
@@ -824,24 +815,38 @@ public class SBC_DownloadHistoryView
 
 			o_name = names;
 
+		}else if ( filter.startsWith( "f:" )){
+
+			filter = filter.substring( 2 );
+
+			o_name = ds.getSaveLocation();
+
 		}else{
 
 			o_name = ds.getName();
 
 		}
 
-		String s = regex ? filter : "\\Q" + filter.replaceAll("\\s*[|;]\\s*", "\\\\E|\\\\Q") + "\\E";
-
 		boolean	match_result = true;
 
-		if ( regex && s.startsWith( "!" )){
+		String expr;
+		
+		if ( regex ){
+			
+			expr = filter;
+			
+			if ( expr.startsWith( "!" )){
+				
+				expr = expr.substring(1);
 
-			s = s.substring(1);
-
-			match_result = false;
+				match_result = false;
+			}
+		}else{
+			
+			expr = RegExUtil.convertAndOrToExpr( filter );
 		}
 
-		Pattern pattern = RegExUtil.getCachedPattern( "downloadhistoryview:search", s, Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE );
+		Pattern pattern = RegExUtil.getCachedPattern( "downloadhistoryview:search", expr, Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE );
 
 		boolean bOurs;
 
