@@ -52,6 +52,7 @@ import com.biglybt.ui.selectedcontent.SelectedContentManager;
 import com.biglybt.ui.swt.TorrentUtil;
 import com.biglybt.ui.swt.Utils;
 import com.biglybt.ui.swt.columns.utils.TableColumnCreatorV3;
+import com.biglybt.ui.swt.components.BubbleTextBox;
 import com.biglybt.ui.swt.debug.ObfuscateImage;
 import com.biglybt.ui.swt.pif.UISWTViewEvent;
 import com.biglybt.ui.swt.pif.UISWTViewEventListener;
@@ -138,7 +139,7 @@ public class SBC_LibraryTableView
 
 		SWTSkinObjectTextbox soFilter = (SWTSkinObjectTextbox) skin.getSkinObject(
 				"library-filter", soParent.getParent());
-		Text txtFilter = soFilter == null ? null : soFilter.getTextControl();
+		BubbleTextBox txtFilter = soFilter == null ? null : soFilter.getBubbleTextBox();
 
 		SWTSkinObjectContainer soCats = (SWTSkinObjectContainer) skin.getSkinObject(
 				"library-categories", soParent.getParent());
@@ -246,15 +247,6 @@ public class SBC_LibraryTableView
 		view.initialize(viewComposite);
 
 
-		SWTSkinObject soSizeSlider = skin.getSkinObject("table-size-slider", soParent.getParent());
-		if (soSizeSlider instanceof SWTSkinObjectContainer) {
-			SWTSkinObjectContainer so = (SWTSkinObjectContainer) soSizeSlider;
-			if (tv != null && !tv.enableSizeSlider(so.getComposite(), 16, 100)) {
-				so.setVisible(false);
-			}
-		}
-
-
 		if (torrentFilterMode == SBC_LibraryView.TORRENTS_ALL
 				&& tv != null) {
 			tv.addRefreshListener(new TableRowRefreshListener() {
@@ -382,6 +374,14 @@ public class SBC_LibraryTableView
 			DownloadManager dm = DataSourceUtils.getDM(ds);
 			if (dm != null) {
 				ManagerUtils.browse( dm );
+				return;
+			}
+		}else if (mode.equals("6")) {
+
+				// pop out detailed view
+	
+			if ( UIFunctionsManager.getUIFunctions().getMDI().popoutEntryByID( MultipleDocumentInterface.SIDEBAR_SECTION_TORRENT_DETAILS, ds, true)){
+	
 				return;
 			}
 		}
