@@ -35,13 +35,9 @@ import org.eclipse.swt.widgets.*;
 import com.biglybt.core.category.Category;
 import com.biglybt.core.config.COConfigurationManager;
 import com.biglybt.core.download.DownloadManager;
+import com.biglybt.core.internat.MessageText;
 import com.biglybt.core.tag.*;
-import com.biglybt.core.util.AEThread2;
-import com.biglybt.core.util.Base32;
-import com.biglybt.core.util.Constants;
-import com.biglybt.core.util.Debug;
-import com.biglybt.core.util.RegExUtil;
-import com.biglybt.core.util.SystemTime;
+import com.biglybt.core.util.*;
 import com.biglybt.ui.UIFunctions;
 import com.biglybt.ui.UIFunctionsManager;
 import com.biglybt.ui.common.ToolBarItem;
@@ -54,6 +50,7 @@ import com.biglybt.ui.swt.Messages;
 import com.biglybt.ui.swt.SimpleTextEntryWindow;
 import com.biglybt.ui.swt.Utils;
 import com.biglybt.ui.swt.columns.tag.*;
+import com.biglybt.ui.swt.components.BubbleTextBox;
 import com.biglybt.ui.swt.mainwindow.MenuFactory;
 import com.biglybt.ui.swt.mdi.MdiEntrySWT;
 import com.biglybt.ui.swt.pifimpl.UISWTViewBuilderCore;
@@ -93,8 +90,6 @@ public class SBC_TagsOverview
 
 	TableViewSWT<Tag> tv;
 
-	private Text txtFilter;
-
 	private Composite table_parent;
 
 	private boolean columnsAdded = false;
@@ -508,6 +503,13 @@ public class SBC_TagsOverview
 					}
 				});
 
+		tableManager.registerColumn(Tag.class, ColumnTagEOA.COLUMN_ID,
+				new TableColumnCreationListener() {
+					@Override
+					public void tableColumnCreated(TableColumn column) {
+						new ColumnTagEOA(column);
+					}
+				});
 		
 		tableManager.setDefaultColumnNames(TABLE_TAGS,
 				new String[] {
@@ -562,13 +564,6 @@ public class SBC_TagsOverview
 	{
 		super.skinObjectShown(skinObject, params);
 
-		SWTSkinObjectTextbox soFilter = (SWTSkinObjectTextbox)getSkinObject( "filterbox" );
-
-		if ( soFilter != null ){
-
-			txtFilter = soFilter.getTextControl();
-		}
-
 		SWTSkinObject so_list = getSkinObject("tags-list");
 
 		if (so_list != null) {
@@ -639,9 +634,19 @@ public class SBC_TagsOverview
 					TABLE_TAGS, new TableColumnCore[0], ColumnTagName.COLUMN_ID,
 					SWT.MULTI | SWT.FULL_SELECTION | SWT.VIRTUAL);
 
-			if ( txtFilter != null ){
-				tv.enableFilterCheck(txtFilter, this);
+			SWTSkinObjectTextbox soFilter = (SWTSkinObjectTextbox) getSkinObject(
+					"filterbox");
+			if (soFilter != null) {
+				BubbleTextBox filter = soFilter.getBubbleTextBox();
+				tv.enableFilterCheck(filter, this);
+				
+				String tooltip = MessageText.getString("filter.tt.start");
+				tooltip += MessageText.getString("tagsoverview.filter.tt.line1");
+				tooltip += MessageText.getString("tagsoverview.filter.tt.line2");
+				
+				filter.setTooltip( tooltip );
 			}
+
 			tv.setRowDefaultHeightEM(1);
 
 			table_parent = new Composite(control, SWT.BORDER);
@@ -811,12 +816,19 @@ public class SBC_TagsOverview
 							return;
 						}
 
-						for (DownloadManager dm : dms) {
-							if ( doAdd ){
-								tag.addTaggable( dm );
-							}else{
-								tag.removeTaggable( dm );
+						try{
+							tag.addTaggableBatch( true );
+						
+							for (DownloadManager dm : dms) {
+								if ( doAdd ){
+									tag.addTaggable( dm );
+								}else{
+									tag.removeTaggable( dm );
+								}
 							}
+						}finally{
+							
+							tag.addTaggableBatch( false );
 						}
 					}
 				});
@@ -1123,10 +1135,42 @@ public class SBC_TagsOverview
 	}
 
 	@Override
-	public boolean filterCheck(Tag ds, String filter, boolean regex) {
-		String name = ds.getTagName( true );
-
-		String s = regex ? filter : "\\Q" + filter.replaceAll("\\s*[|;]\\s*", "\\\\E|\\\\Q") + "\\E";
+	public boolean 
+	filterCheck(
+		Tag 		tag, 
+		String 		filter, 
+		boolean 	regex ) 
+	{
+		String name;
+		
+		if ( filter.startsWith( "g:" )){
+			
+			filter = filter.substring( 2 );
+			
+			name = tag.getGroup();
+			
+			if ( name == null ){
+				
+				name = "";
+			}
+		}else if ( filter.startsWith( "p:" )){
+			
+			filter = filter.substring( 2 );
+			
+			if ( tag instanceof TagFeatureProperties ){
+				
+				name = ((TagFeatureProperties)tag).getPropertiesString();
+				
+			}else{
+				
+				name = "";
+			}
+		}else{
+		
+			name = tag.getTagName( true );
+		}
+		
+		String s = regex ? filter : RegExUtil.splitAndQuote( filter, "\\s*[|;]\\s*" );
 
 		boolean	match_result = true;
 
