@@ -1343,7 +1343,24 @@ public class SB_Transfers
 							return "" + dms.size();
 						}
 					}
+				}else if ( propertyID == TITLE_INDICATOR_COLOR ){
+
+					TagType tag_type = category.getTagType();
+
+					int[] def_color = tag_type.getColorDefault();
+
+					int[] tag_color = category.getColor();
+
+					if ( tag_color != def_color ){
+
+						return( tag_color );
+					}
+
+				}else if ( propertyID == TITLE_INDICATOR_TEXT_TOOLTIP ){
+
+					return( TagUtils.getTagTooltip( category ));
 				}
+				
 				return null;
 			}
 		};
@@ -1468,7 +1485,18 @@ public class SB_Transfers
 			
 			boolean is_group		= group != null && !group.isEmpty();
 			
-			boolean parent_is_group = parent_id.startsWith( "Tag." + tag.getTagType().getTagType() + ".group." + (is_group?group:"" ));
+			boolean parent_is_group;
+			
+			String group_prefix = "Tag." + tag.getTagType().getTagType() + ".group.";
+			
+			if ( is_group ){
+				
+				parent_is_group = parent_id.equals( group_prefix + group );
+				
+			}else{
+				
+				parent_is_group = parent_id.startsWith( group_prefix );
+			}
 			
 			if ( is_group != parent_is_group ){
 				
@@ -1563,11 +1591,15 @@ public class SB_Transfers
 					if ( tag.getTaggableTypes() == Taggable.TT_DOWNLOAD || tag.getTaggableTypes() == Taggable.TT_PEER ){
 						
 						group_id = "Tag." + tag_type + ".group." + tag_group;
-						
+												
 						if ( mdi.getEntry( group_id ) == null ){
 							
 							// Create Entry for Group
-						
+													
+							TagGroup group = tag.getGroupContainer();
+
+							String gid= group_id;
+							
 							ViewTitleInfo viewTitleInfo =
 									new ViewTitleInfo()
 									{
@@ -1581,12 +1613,20 @@ public class SB_Transfers
 												return( tag_group );
 												
 											}else if ( pid == TITLE_INDICATOR_TEXT ){
-	
 												
-	
+												MdiEntry entry =  mdi.getEntry( gid );
+												
+												if ( entry != null && !entry.isExpanded()){
+												
+													return( String.valueOf( group.getTags().size()));
+												}
 											}else if ( pid == TITLE_INDICATOR_COLOR ){
 	
 	
+												int[] colour = group.getColor();
+												
+												return( colour );
+												
 											}else if ( pid == TITLE_INDICATOR_TEXT_TOOLTIP ){
 	
 												
@@ -1596,39 +1636,115 @@ public class SB_Transfers
 										}
 									};
 									
+							TagGroupListener tgl = 
+								new TagGroupListener(){
+									
+									@Override
+									public void tagRemoved(TagGroup group, Tag tag){
+										update();
+									}
+									
+									@Override
+									public void tagAdded(TagGroup group, Tag tag){
+										update();
+									}
+									@Override
+									public void groupChanged(TagGroup group){
+										update();
+									}
+									
+									private void
+									update()
+									{
+										MdiEntry entry =  mdi.getEntry( gid );
+
+										if ( entry != null ){
+										
+											if ( entry.isEntryDisposed()){
+												
+												group.removeListener( this );
+												
+											}else{
+											
+												ViewTitleInfoManager.refreshTitleInfo( viewTitleInfo );
+																				
+												requestRedraw( entry );
+											}
+										}
+									}
+								};
+								
+							group.addListener( tgl, false );								
+							
 									// find where to locate this in the sidebar
 
 							String prev_id = getTagPosition( mdi, parent_id, tag_type, tag_group );
 							
 							MdiEntry entry;
 							
+							boolean closeable = true;
+							
 							if ( tag.getTaggableTypes() == Taggable.TT_DOWNLOAD ){
 							
 								entry = mdi.createEntryFromSkinRef(
-									parent_id, group_id, "library", tag_group, viewTitleInfo, tag.getGroupContainer(), false, prev_id );
+									parent_id, group_id, "library", tag_group, viewTitleInfo, group, closeable, prev_id );
 							
 							}else{
 								
 								UISWTViewBuilderCore builder = new UISWTViewBuilderCore( group_id, null,
 										PeersGeneralView.class);
 								builder.setParentEntryID(parent_id);
-								builder.setPreferredAfterID(prev_id).setInitialDatasource( tag.getGroupContainer() );
-								entry = mdi.createEntry(builder, false);
+								builder.setPreferredAfterID(prev_id).setInitialDatasource( group );
+								entry = mdi.createEntry(builder, closeable);
 
 								entry.setViewTitleInfo( viewTitleInfo );
 							}
 							
 							setTagIcon( tag, entry, true );
-							
-							entry.setUserData( TAG_TAG_OR_GROUP_KEY, tag.getGroupContainer());
+														
+							entry.setUserData( TAG_TAG_OR_GROUP_KEY, group );
 
-							// remove header when there are no children
-							entry.addListener((parent, ch, user) -> {
-								String viewID = parent.getViewID();
-								if (mdi.getChildrenOf(viewID).isEmpty()) {
-									mdi.closeEntry(parent,false);
+							entry.addListener((e,user )->{
+								if ( user ){
+									List<Tag> kids = group.getTags();
+									
+									for ( Tag kid: kids ){
+										kid.setVisible( false );
+									}
 								}
 							});
+							
+							// remove header when there are no children
+							entry.addListener(
+								new MdiChildCloseListener(){
+								
+									boolean closed;
+									
+									public void 
+									mdiChildEntryClosed(MdiEntry parent, MdiEntry child, boolean user){
+									
+										String viewID = parent.getViewID();
+										if (mdi.getChildrenOf(viewID).isEmpty()) {
+									
+											synchronized( this ){
+												if ( closed ){
+													return;
+												}
+												closed = true;
+											}
+											
+												// need to defer this in case there are outstanding sidebar
+												// additions that are located relative to this entry...
+											
+											mdi.runWhenIdle(()->{
+												
+												mdi.closeEntry(parent,false);
+												
+												
+											});
+										}
+									}
+								});
 
 
 							if ( entry instanceof MdiEntrySWT ){
@@ -1850,16 +1966,23 @@ public class SB_Transfers
 							return;
 						}
 
-						for (DownloadManager dm : dms) {
-							if ( doAdd ){
-								if ( !tag.hasTaggable( dm )){
-									tag.addTaggable( dm );
-								}
-							}else{
-								if ( tag.hasTaggable( dm )){
-									tag.removeTaggable( dm );
+						try{
+							tag.addTaggableBatch( true );
+							
+							for (DownloadManager dm : dms) {
+								if ( doAdd ){
+									if ( !tag.hasTaggable( dm )){
+										tag.addTaggable( dm );
+									}
+								}else{
+									if ( tag.hasTaggable( dm )){
+										tag.removeTaggable( dm );
+									}
 								}
 							}
+						}finally{
+							
+							tag.addTaggableBatch( false );
 						}
 					}
 				};
@@ -2312,10 +2435,8 @@ public class SB_Transfers
 						@Override
 						public void run(){
 							try{
-								String resource = new File( fif ).toURI().toURL().toExternalForm();
-								
-								ImageLoader.getInstance().getUrlImage(
-									resource, 
+								ImageLoader.getInstance().getFileImage(
+									new File( fif ), 
 									new Point( 20, 14 ),
 									new ImageLoader.ImageDownloaderListener(){
 										
