@@ -63,6 +63,7 @@ import com.biglybt.ui.swt.views.skin.SkinnedDialog;
 
 import com.biglybt.pif.PluginInterface;
 import com.biglybt.pif.PluginManager;
+import com.biglybt.pif.download.Download;
 import com.biglybt.pif.ui.UIInstance;
 import com.biglybt.pif.ui.UIManager;
 import com.biglybt.pif.ui.menus.MenuItem;
@@ -384,27 +385,8 @@ public class SideBar
 					SideBarEntrySWT sbe = getCurrentEntry();
 	
 					if ( sbe != null ){
-						SkinnedDialog skinnedDialog =
-								new SkinnedDialog(
-										"skin3_dlg_sidebar_popout",
-										"shell",
-										menu==menuItemOnTop?UIFunctionsManagerSWT.getUIFunctionsSWT().getMainShell():null,
-										SWT.RESIZE | SWT.MAX | SWT.DIALOG_TRIM);
-	
-						SWTSkin skin = skinnedDialog.getSkin();
-	
-						SWTSkinObjectContainer cont = sbe.buildStandAlone((SWTSkinObjectContainer)skin.getSkinObject( "content-area" ));
-	
-						if ( cont != null ){
-	
-							skinnedDialog.setTitle( sbe.getTitle());
-	
-							skinnedDialog.open();
-	
-						}else{
-	
-							skinnedDialog.close();
-						}
+
+						popoutEntry( sbe, menu==menuItemOnTop );
 					}
 				}
 			};
@@ -414,6 +396,52 @@ public class SideBar
 		}
 	}
 
+	public boolean
+	popoutEntry(
+		MdiEntry	entry,
+		boolean		onTop )
+	{
+		SideBarEntrySWT sbe = (SideBarEntrySWT)entry;
+		
+		SkinnedDialog skinnedDialog =
+				new SkinnedDialog(
+						"skin3_dlg_sidebar_popout",
+						"shell",
+						onTop?UIFunctionsManagerSWT.getUIFunctionsSWT().getMainShell():null,
+						SWT.RESIZE | SWT.MAX | SWT.DIALOG_TRIM);
+	
+		SWTSkin skin = skinnedDialog.getSkin();
+	
+		SWTSkinObjectContainer cont = sbe.buildStandAlone((SWTSkinObjectContainer)skin.getSkinObject( "content-area" ));
+	
+		if ( cont != null ){
+	
+			skinnedDialog.setTitle( sbe.getTitle());
+	
+			String metrics_id;
+			
+				// hack - we don't want to remember shell metrics on a per-download basis
+			
+			if ( sbe.getDatasource() instanceof Download ){
+				
+				metrics_id = MultipleDocumentInterface.SIDEBAR_SECTION_TORRENT_DETAILS;
+				
+			}else{
+				
+				metrics_id = sbe.getId();
+			}
+			
+			skinnedDialog.open( "mdi.popout:" + metrics_id, true );
+	
+			return( true );
+			
+		}else{
+	
+			skinnedDialog.close();
+			
+			return( false );
+		}
+	}
 
 	/**
 	 *
@@ -973,7 +1001,33 @@ public class SideBar
 
 							break;
 						}
+						case SWT.MouseDoubleClick: {
+							
+							int indent = END_INDENT ? tree.getClientArea().width - 1 : 0;
+							treeItem = tree.getItem(new Point(indent, event.y));
+							SideBarEntrySWT entry = (SideBarEntrySWT) ((treeItem == null || treeItem.isDisposed())
+									? null : treeItem.getData("MdiEntry"));
 
+							if ( entry != null ){
+								
+									// don't trigger popout if double click is in close area or
+									// item has child and user is clicking in the expandy area 
+
+								Point cursorLocation = tree.toControl(event.display.getCursorLocation());
+								
+								if ( lastCloseAreaClicked == null || !lastCloseAreaClicked.contains(cursorLocation.x, cursorLocation.y)){
+																		
+									Integer offset = (Integer)treeItem.getData( "contentStartOffset" );
+									
+									if ( treeItem.getItemCount() == 0 || ( offset != null && event.x >= offset )){
+										
+										popoutEntry( entry, true );
+									}
+								}
+							}
+							
+							break;
+						}
 						case SWT.Dispose: {
 							fontHeader.dispose();
 							if (dropTarget != null && !dropTarget.isDisposed()) {
@@ -1014,6 +1068,7 @@ public class SideBar
 
 		tree.addListener(SWT.Selection, treeListener);
 		tree.addListener(SWT.Dispose, treeListener);
+		tree.addListener(SWT.MouseDoubleClick, treeListener);
 
 		// For icons
 		tree.addListener(SWT.MouseUp, treeListener);
@@ -1115,6 +1170,10 @@ public class SideBar
 
 		final Menu menuTree = new Menu(tree);
 		tree.setMenu(menuTree);
+		
+		tree.addMenuDetectListener(e -> {
+			menuTree.setData("MenuSource", e.detail);
+		});
 
 		menuTree.addMenuListener(new MenuListener() {
 			boolean bShown = false;
@@ -1146,14 +1205,23 @@ public class SideBar
 
 				bShown = true;
 
-				Point ptMouse = tree.toControl(e.display.getCursorLocation());
+				Object oMenuSource = menuTree.getData("MenuSource");
+				int menuSource = (oMenuSource instanceof Number)
+						? ((Number) oMenuSource).intValue() : SWT.MENU_MOUSE;
 
-				int indent = END_INDENT ? tree.getClientArea().width - 1 : 0;
-				TreeItem treeItem = tree.getItem(new Point(indent, ptMouse.y));
-				if (treeItem == null) {
-					return;
+				SideBarEntrySWT entry;
+				if (menuSource != SWT.MENU_KEYBOARD) {
+					Point ptMouse = tree.toControl(e.display.getCursorLocation());
+	
+					int indent = END_INDENT ? tree.getClientArea().width - 1 : 0;
+					TreeItem treeItem = tree.getItem(new Point(indent, ptMouse.y));
+					if (treeItem == null) {
+						return;
+					}
+					entry = (SideBarEntrySWT) treeItem.getData("MdiEntry");
+				} else {
+					entry = getCurrentEntry();
 				}
-				SideBarEntrySWT entry = (SideBarEntrySWT) treeItem.getData("MdiEntry");
 
 				fillMenu(menuTree, entry, "sidebar");
 
@@ -1626,10 +1694,17 @@ public class SideBar
 			entry.setImageLeftID("image.sidebar.plugin");
 		}
 
+		activityStart();
+		
 		Utils.execSWTThreadLater(0, new AERunnable() {
 			@Override
 			public void runSupport() {
-				_setupNewEntry(entry, id, expandParent, closeable);
+				try{
+					_setupNewEntry(entry, id, expandParent, closeable);
+				}finally{
+					
+					activityEnd();
+				}
 			}
 		});
 	}
@@ -1641,7 +1716,11 @@ public class SideBar
 			return;
 		}
 		
-		if ( getEntry( entry.getViewID()) != entry ){
+		String view_id = entry.getViewID();
+		
+		SideBarEntrySWT existing = getEntry( view_id );
+		
+		if ( existing != entry ){
 		
 				// entry has been deleted/replaced in the meantime
 			
@@ -2365,4 +2444,79 @@ public class SideBar
 	protected SideBarEntrySWT createEntryByCreationListener(String id, Map<?, ?> autoOpenInfo) {
 		return (SideBarEntrySWT) super.createEntryByCreationListener(id, autoOpenInfo);
 	}
+	
+	private List<Runnable> 	idle_pending = new ArrayList<>();	
+	private int				activity_count;
+	
+	private void
+	activityStart()
+	{
+		synchronized( idle_pending ){
+			
+			activity_count++;
+		}
+	}
+	
+	private void
+	activityEnd()
+	{
+		List<Runnable> to_run = null;
+		
+		synchronized( idle_pending ){
+			
+			activity_count--;
+			
+			if ( activity_count == 0 ){
+				
+				if ( !idle_pending.isEmpty()){
+					
+					to_run = new ArrayList<>( idle_pending );
+					
+					idle_pending.clear();
+				}
+			}
+		}
+		
+		if ( to_run != null ){
+			
+			List<Runnable> x = to_run;
+
+			Runnable doit = ()->{
+				for ( Runnable r: x ){
+					try{
+						r.run();
+					}catch( Throwable e ){
+						Debug.out(e);
+					}
+				}
+			};
+						
+			if ( Utils.isSWTThread()){
+			
+				Utils.getOffOfSWTThread( doit );
+				
+			}else{
+				
+				doit.run();
+			}
+		}
+	}
+	
+	@Override
+	public void 
+	runWhenIdle(
+		Runnable r )
+	{
+		synchronized( idle_pending ){
+			
+			if ( activity_count > 0 ){
+								
+				idle_pending.add( r );
+				
+				return;
+			}
+		}
+		
+		r.run();
+	}
 }
