@@ -406,6 +406,7 @@ public class SideBarEntrySWT
 
 		//System.out.println(id + " destroyEntry " + swtItem + "; " + Debug.getCompressedStackTrace());
 		if (swtItem == null) {
+			destroyEntryAlways();
 			return;
 		}
 
@@ -887,6 +888,8 @@ public class SideBarEntrySWT
 			imageLeft = getImageLeft(null);
 		}
 		
+		treeItem.setData("contentStartOffset", x0IndicatorOfs);
+		
 		if ( IMAGELEFT_HIDDEN ){
 			
 		}else if (imageLeft != null ) {
