@@ -55,10 +55,6 @@ public class VivaldiPanel extends BasePanel {
   Canvas canvas;
   Scale scale;
 
-  private boolean mouseLeftDown = false;
-  private boolean mouseRightDown = false;
-  private int xDown;
-  private int yDown;
 
 
   private boolean antiAliasingAvailable = true;
@@ -110,21 +106,12 @@ public class VivaldiPanel extends BasePanel {
 
       @Override
       public void mouseDown(MouseEvent event) {
-        if(event.button == 1) mouseLeftDown = true;
-        if(event.button == 3) mouseRightDown = true;
-        xDown = event.x;
-        yDown = event.y;
-        scale.saveMinX = scale.minX;
-        scale.saveMaxX = scale.maxX;
-        scale.saveMinY = scale.minY;
-        scale.saveMaxY = scale.maxY;
-        scale.saveRotation = scale.rotation;
+    	  scale.mouseDown( event );
       }
 
       @Override
       public void mouseUp(MouseEvent event) {
-        if(event.button == 1) mouseLeftDown = false;
-        if(event.button == 3) mouseRightDown = false;
+        scale.mouseUp( event );
         refreshContacts(lastContacts, lastSelf);
       }
 
@@ -204,102 +191,19 @@ public class VivaldiPanel extends BasePanel {
 			@Override
 			public void handleEvent(Event event) {
 				// System.out.println(event.count);
-        scale.saveMinX = scale.minX;
-        scale.saveMaxX = scale.maxX;
-        scale.saveMinY = scale.minY;
-        scale.saveMaxY = scale.maxY;
 
-        int deltaY = event.count * -5;
-        // scaleFactor>1 means zoom in, this happens when
-        // deltaY<0 which happens when the mouse is moved up.
-        float scaleFactor = 1 - (float) deltaY / 300;
-        if(scaleFactor <= 0) scaleFactor = 0.01f;
-
-        // Scalefactor of e.g. 3 makes elements 3 times larger
-        float moveFactor = 1 - 1/scaleFactor;
-
-				Canvas canvas = ((Canvas) event.widget);
-				Point canvasSize = canvas.getSize();
-				// event.x, event.y are relative to control
-				float mouseXpct = (event.x + 1) / (float) canvasSize.x;
-				float mouseYpct = (event.y + 1) / (float) canvasSize.y;
-				float xOfs = (mouseXpct - 0.5f) * (scale.saveMaxX - scale.saveMinX);
-				float yOfs = (mouseYpct - 0.5f) * (scale.saveMaxY - scale.saveMinY);
-
-				float centerX = ((scale.saveMinX + scale.saveMaxX)/2) + xOfs;
-        scale.minX = scale.saveMinX + moveFactor * (centerX - scale.saveMinX);
-        scale.maxX = scale.saveMaxX - moveFactor * (scale.saveMaxX - centerX);
-
-        float centerY = (scale.saveMinY + scale.saveMaxY)/2 + yOfs;
-        scale.minY = scale.saveMinY + moveFactor * (centerY - scale.saveMinY);
-        scale.maxY = scale.saveMaxY - moveFactor * (scale.saveMaxY - centerY);
-
-        scale.disableAutoScale = true;
-        refreshContacts(lastContacts, lastSelf);
+				scale.mouseWheel( event );
+				
+				refreshContacts(lastContacts, lastSelf);
 			}
 		});
 
     canvas.addMouseMoveListener(new MouseMoveListener() {
       @Override
       public void mouseMove(MouseEvent event) {
-        if(mouseLeftDown && (event.stateMask & SWT.MOD4) == 0) {
-          int deltaX = event.x - xDown;
-          int deltaY = event.y - yDown;
-          float width = scale.width;
-          float height = scale.height;
-          float ratioX = (scale.saveMaxX - scale.saveMinX) / width;
-          float ratioY = (scale.saveMaxY - scale.saveMinY) / height;
-          float realDeltaX = deltaX * ratioX;
-          float realDeltaY  = deltaY * ratioY;
-          scale.minX = scale.saveMinX - realDeltaX;
-          scale.maxX = scale.saveMaxX - realDeltaX;
-          scale.minY = scale.saveMinY - realDeltaY;
-          scale.maxY = scale.saveMaxY - realDeltaY;
-          scale.disableAutoScale = true;
-          refreshContacts(lastContacts, lastSelf);
-        }
-        if(mouseRightDown || (mouseLeftDown && (event.stateMask & SWT.MOD4) > 0)) {
-          int deltaX = event.x - xDown;
-	        int deltaY = event.y - yDown;
-	        int diffX = Math.abs(deltaX);
-	        int diffY = Math.abs(deltaY);
-	        // Don't start rotating until a few px movement.  Helps when
-	        // user just wants to zoom (move up/down) or rotate (move left/right) 
-	        // and doesn't have steady hand
-          if (diffY > diffX && diffX <= 3) {
-          	deltaX = 0;
-          }
-	        if (diffY > diffX && diffY <= 3) {
-		        deltaY = 0;
-	        }
-          scale.rotation = scale.saveRotation - (float) deltaX / 100;
-
-          // scaleFactor>1 means zoom in, this happens when
-          // deltaY<0 which happens when the mouse is moved up.
-          float scaleFactor = 1 - (float) deltaY / 300;
-          if(scaleFactor <= 0) scaleFactor = 0.01f;
-
-          // Scalefactor of e.g. 3 makes elements 3 times larger
-          float moveFactor = 1 - 1/scaleFactor;
-
-	        Canvas canvas = ((Canvas) event.widget);
-	        Point canvasSize = canvas.getSize();
-	        // event.x, event.y are relative to control
-	        float mouseXpct = (xDown + 1) / (float) canvasSize.x;
-	        float mouseYpct = (yDown + 1) / (float) canvasSize.y;
-	        float xOfs = (mouseXpct - 0.5f) * (scale.saveMaxX - scale.saveMinX);
-	        float yOfs = (mouseYpct - 0.5f) * (scale.saveMaxY - scale.saveMinY);
-
-	        float centerX = (scale.saveMinX + scale.saveMaxX)/2 + xOfs;
-          scale.minX = scale.saveMinX + moveFactor * (centerX - scale.saveMinX);
-          scale.maxX = scale.saveMaxX - moveFactor * (scale.saveMaxX - centerX);
-
-          float centerY = (scale.saveMinY + scale.saveMaxY)/2 + yOfs;
-          scale.minY = scale.saveMinY + moveFactor * (centerY - scale.saveMinY);
-          scale.maxY = scale.saveMaxY - moveFactor * (scale.saveMaxY - centerY);
-          scale.disableAutoScale = true;
-          refreshContacts(lastContacts, lastSelf);
-        }
+    	  if ( scale.mouseMove(event)){
+    		  refreshContacts(lastContacts, lastSelf);
+    	  }
       }
     });
 
@@ -385,8 +289,7 @@ public class VivaldiPanel extends BasePanel {
 	    	return;
 	    }
 
-	    scale.width = size.width;
-	    scale.height = size.height;
+	    scale.setSize( size );
 
 	    Color white = ColorCache.getColor(display,255,255,255);
 	    Color blue = ColorCache.getColor(display,66,87,104);
@@ -516,16 +419,7 @@ public class VivaldiPanel extends BasePanel {
 					float new_min_y = min_y - 50;
 					float new_max_y = max_y + 50;
 
-					if ( 	scale.minX != new_min_x ||
-							scale.maxX != new_max_x ||
-							scale.minY != new_min_y ||
-							scale.maxY != new_max_y ){
-
-						scale.minX = new_min_x;
-						scale.maxX = new_max_x;
-						scale.minY = new_min_y;
-						scale.maxY = new_max_y;
-					}
+					scale.setScale( new_min_x, new_max_x, new_min_y, new_max_y );
 
 					//System.out.println(scale.minX+","+ scale.maxX+","+scale.minY+","+scale.maxY+" -> " + new_min_x + "," +new_max_x+","+new_min_y+","+new_max_y);
 
@@ -546,8 +440,7 @@ public class VivaldiPanel extends BasePanel {
     if(canvas.isDisposed()) return;
     Rectangle size = canvas.getBounds();
 
-    scale.width = size.width;
-    scale.height = size.height;
+    scale.setSize( size );
 
     if (img != null && !img.isDisposed()) {
     	img.dispose();
@@ -592,7 +485,7 @@ public class VivaldiPanel extends BasePanel {
     int x0 = scale.getX(x,y);
     int y0 = scale.getY(x,y);
     gc.fillRectangle(x0-1,y0-1,3,3);
-    gc.drawLine(x0,y0,x0,(int)(y0-200*h/(scale.maxY-scale.minY)));
+    gc.drawLine(x0,y0,x0,(int)(y0-200*h/(scale.getMaxY()-scale.getMinY())));
   }
 
   private void draw(GC gc,float x,float y,float h,DHTControlContact contact,int distance,float error) {
