@@ -37,7 +37,6 @@ import com.biglybt.core.internat.MessageText.MessageTextListener;
 import com.biglybt.core.util.*;
 import com.biglybt.ui.common.table.*;
 import com.biglybt.ui.common.table.impl.TableColumnManager;
-import com.biglybt.ui.common.table.impl.TableRowCoreSorter;
 import com.biglybt.ui.common.table.impl.TableViewImpl;
 import com.biglybt.ui.selectedcontent.ISelectedContent;
 import com.biglybt.ui.selectedcontent.SelectedContentListener;
@@ -46,6 +45,8 @@ import com.biglybt.ui.swt.MenuBuildUtils;
 import com.biglybt.ui.swt.SimpleTextEntryWindow;
 import com.biglybt.ui.swt.TextWithHistory;
 import com.biglybt.ui.swt.Utils;
+import com.biglybt.ui.swt.components.BubbleTextBox;
+import com.biglybt.ui.swt.components.BubbleTextBox.BubbleTextBoxChangeListener;
 import com.biglybt.ui.swt.debug.ObfuscateImage;
 import com.biglybt.ui.swt.debug.UIDebugGenerator;
 import com.biglybt.ui.swt.mainwindow.Colors;
@@ -58,6 +59,8 @@ import com.biglybt.ui.swt.views.table.*;
 import com.biglybt.ui.swt.views.table.impl.TableTooltips;
 import com.biglybt.ui.swt.views.table.impl.TableViewSWT_Common;
 import com.biglybt.ui.swt.views.table.impl.TableViewSWT_TabsCommon;
+import com.biglybt.util.MapUtils;
+
 import com.biglybt.pif.download.Download;
 import com.biglybt.pif.download.DownloadTypeComplete;
 import com.biglybt.pif.download.DownloadTypeIncomplete;
@@ -77,7 +80,7 @@ import com.biglybt.pif.ui.tables.TableRowMouseListener;
 public class TableViewPainted
 	extends TableViewImpl<Object>
 	implements ParameterListener, TableViewSWT<Object>, ObfuscateImage,
-	MessageTextListener
+	MessageTextListener, BubbleTextBoxChangeListener
 {
 	private static final boolean hasGetScrollBarMode = SWT.getVersion() >= 3821;
 
@@ -112,6 +115,10 @@ public class TableViewPainted
 
 	private int defaultRowHeight = 17;
 
+	private int lineHeight;
+
+	protected float iHeightEM = -1;
+
 	/**
 	 * Rows visible to user.  We assume this list is always up to date
 	 */
@@ -182,6 +189,7 @@ public class TableViewPainted
 	private DragSource dragSource;
 	private DropTarget dropTarget;
 	private boolean destroying;
+	private int rowMinHeight;
 
 	private class
 	RefreshTableRunnable
@@ -261,8 +269,6 @@ public class TableViewPainted
 
 	protected boolean isFocused;
 
-	protected float iHeightEM = -1;
-
 	/**
 	 * Main Initializer
 	 * @param _sTableID Which table to handle (see
@@ -354,8 +360,8 @@ public class TableViewPainted
 			public void keyPressed(KeyEvent event) {
 				if ( event.keyCode == SWT.ESC ){
 					TableViewSWTFilter<?> filter = getSWTFilter();
-					if ( filter != null ){
-						filter.widget.setText( "" );
+					if ( filter != null && filter.filterBox != null ){
+						filter.filterBox.setText( "" );
 					}
 				}
 				if ( getComposite().isDisposed()){
@@ -377,7 +383,7 @@ public class TableViewPainted
 					if ((event.stateMask & SWT.SHIFT) != 0) {
 						if (rowToSelect != null && focusedRow != null) {
 							TableRowCore[] selectedRows = getSelectedRows();
-							Arrays.sort(selectedRows, new TableRowCoreSorter());
+							sortRowsByVisibilityIndex( selectedRows );
 							boolean select = selectedRows.length == 0
 									|| selectedRows[0] == focusedRow;
 //							System.out.println("i=" + selectedRows[0].getIndex() + ";"
@@ -459,26 +465,26 @@ public class TableViewPainted
 						}
 					} else {
 						TableRowCore rowToSelect = getNextRow(focusedRow);
-  					if (rowToSelect != null) {
-  						if ((event.stateMask & SWT.SHIFT) != 0) {
-  							TableRowCore[] selectedRows = getSelectedRows();
-  							Arrays.sort(selectedRows, new TableRowCoreSorter());
-  							boolean select = selectedRows.length == 0
-  									|| selectedRows[selectedRows.length - 1] == focusedRow;
-  							if (select) {
-  								rowToSelect.setSelected(select);
-  							} else {
-  								TableRowPainted rowToUnSelect = focusedRow;
-  								setFocusedRow(rowToSelect);
-  								rowToUnSelect.setSelected(false);
-  							}
-  						} else {
-  							setSelectedRows(new TableRowCore[] {
-  								rowToSelect
-  							});
-  						}
+						if (rowToSelect != null) {
+							if ((event.stateMask & SWT.SHIFT) != 0) {
+								TableRowCore[] selectedRows = getSelectedRows();
+								sortRowsByVisibilityIndex( selectedRows );
+								boolean select = selectedRows.length == 0
+										|| selectedRows[selectedRows.length - 1] == focusedRow;
+								if (select) {
+									rowToSelect.setSelected(select);
+								} else {
+									TableRowPainted rowToUnSelect = focusedRow;
+									setFocusedRow(rowToSelect);
+									rowToUnSelect.setSelected(false);
+								}
+							} else {
+								setSelectedRows(new TableRowCore[] {
+										rowToSelect
+								});
+							}
 							updateTable = true;
-  					}
+						}
 					}
 				} else if (event.keyCode == SWT.PAGE_DOWN) {
 					TableRowCore row = focusedRow;
@@ -502,15 +508,19 @@ public class TableViewPainted
 					}
 					updateTable = true;
 				} else if (event.keyCode == SWT.END) {
-					TableRowCore lastRow = getRow(getRowCount() - 1);
-					if ((event.stateMask & SWT.SHIFT) != 0) {
-						selectRowsTo(lastRow);
-					} else if (event.stateMask == 0) {
-  					setSelectedRows(new TableRowCore[] {
-  						lastRow
-  					});
+					//TableRowCore lastRow = getRow(getRowCount() - 1);
+					TableRowCore[] rows = getRowsAndSubRows( false );
+					if ( rows.length > 0 ){
+						TableRowCore lastRow = rows[rows.length-1];
+						if ((event.stateMask & SWT.SHIFT) != 0) {
+							selectRowsTo(lastRow);
+						} else if (event.stateMask == 0) {
+	  					setSelectedRows(new TableRowCore[] {
+	  						lastRow
+	  					});
+						}
+						updateTable = true;
 					}
-					updateTable = true;
 				} else if (event.keyCode == SWT.ARROW_RIGHT && event.stateMask == 0) {
 					if (event.stateMask == 0 && focusedRow != null && !focusedRow.isExpanded() && canHaveSubItems()) {
 						focusedRow.setExpanded(true);
@@ -549,6 +559,20 @@ public class TableViewPainted
 		};
 	}
 
+	private void
+	sortRowsByVisibilityIndex(
+		TableRowCore[] 	selectedRows )
+	{
+		Arrays.sort(
+			selectedRows,
+			new Comparator<TableRowCore>(){
+				@Override
+				public int compare(TableRowCore o1, TableRowCore o2){
+					return( o1.getVisibleRowIndex() - o2.getVisibleRowIndex());
+				}
+			});
+	}
+	
 	protected boolean isRowPartiallyVisible(TableRowPainted row) {
 		if (row == null) {
 			return false;
@@ -724,8 +748,10 @@ public class TableViewPainted
 			for ( int i=0;i<rows.length-1;i++){
 			
 				if ( rows[i] == relativeToRow ){
-											
-					return( rows[i+1] );
+						
+					TableRowCore next = rows[i+1];
+												
+					return( next );
 				}
 			}
 			
@@ -829,7 +855,7 @@ public class TableViewPainted
 			return( true );
 		}
 		
-		sortRows( true );
+		sortRows( true, false );
 		
 		if ( lastMC != mutationCount.get()){
 			
@@ -984,7 +1010,7 @@ public class TableViewPainted
 
 	@Override
 	public void setRowDefaultHeightPX(int iHeight) {
-		if (iHeight > defaultRowHeight) {
+		if (iHeight != defaultRowHeight) {
 			defaultRowHeight = iHeight;
 
 			Utils.execSWTThread(new AERunnable() {
@@ -998,6 +1024,16 @@ public class TableViewPainted
 		}
 	}
 
+	@Override
+	public int getLineHeight() {
+		return lineHeight;
+	}
+	
+	@Override
+	public int getRowMinHeight() {
+		return rowMinHeight;
+	}
+
 	/* (non-Javadoc)
 	 * @see TableView#getRow(int, int)
 	 */
@@ -1381,17 +1417,18 @@ public class TableViewPainted
 		cTable = new Canvas(cTableComposite, SWT.NO_BACKGROUND | SWT.H_SCROLL | SWT.V_SCROLL);
 		cTable.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
 
-		int minRowHeight = FontUtils.getFontHeightInPX(cTable.getFont());
+		lineHeight = FontUtils.getFontHeightInPX(cTable.getFont());
+		rowMinHeight = lineHeight;
 		if (iHeightEM > 0) {
-			defaultRowHeight = (int) ((minRowHeight * iHeightEM) + iHeightEM);
+			defaultRowHeight = (int) ((rowMinHeight * iHeightEM) + iHeightEM);
 			iHeightEM = -1;
 		}
 
 		// good test
 		//cTable.setFont(FontUtils.getFontPercentOf(cTable.getFont(), 1.50f));
-		minRowHeight += Math.ceil(minRowHeight * 2.0 / 16.0);
-		if (defaultRowHeight < minRowHeight) {
-			defaultRowHeight = minRowHeight;
+		rowMinHeight += Math.ceil(rowMinHeight * 2.0 / 16.0);
+		if (defaultRowHeight < rowMinHeight) {
+			defaultRowHeight = rowMinHeight;
 		}
 
 		cTable.setBackground(TablePaintedUtils.getColour(parent.getDisplay(), SWT.COLOR_LIST_BACKGROUND));
@@ -1492,6 +1529,7 @@ public class TableViewPainted
 		if (DEBUG_WITH_SHELL) {
 	  		Shell shell = new Shell();
 	  		sCanvasImage = new Canvas(shell, SWT.DOUBLE_BUFFERED);
+	  		shell.setText(tableID);
 	  		shell.setLayout(new FillLayout());
 	  		sCanvasImage.addPaintListener(new PaintListener() {
 	  			@Override
@@ -1512,6 +1550,7 @@ public class TableViewPainted
 	  			}
 	  		});
 	  		shell.setVisible(true);
+	  		forceDebugShellRefresh(null);
 		}
 
 
@@ -1519,6 +1558,21 @@ public class TableViewPainted
 		cTable.addMouseMoveListener(tvSWTCommon);
 		cTable.addKeyListener(tvSWTCommon);
 		//composite.addSelectionListener(tvSWTCommon);
+		
+		cTable.addListener(SWT.MouseVerticalWheel, e -> {
+			if  (e.stateMask == 0) {
+				return;
+			}
+			if (e.widget != e.display.getCursorControl()) {
+				return;
+			}
+			if ((e.stateMask & SWT.MOD1) > 0) {
+				int newHeight = Math.min(
+						Math.max(getRowMinHeight(), getRowDefaultHeight() - e.count), 300);
+				setRowHeight(newHeight);
+				e.doit = false;
+			}
+		});
 
 		cTable.addTraverseListener(new TraverseListener() {
 			@Override
@@ -1590,6 +1644,9 @@ public class TableViewPainted
 		}
 		setSortColumns(sortColumns, false);
 
+		Map tableConfigMap = tcManager.getTableConfigMap(tableID);
+		defaultRowHeight = MapUtils.getMapInt(tableConfigMap, "RowHeight", defaultRowHeight);
+
 		triggerLifeCycleListener(TableLifeCycleListener.EVENT_TABLELIFECYCLE_INITIALIZED);
 
 		configMan.addParameterListener("Graphics Update", this);
@@ -1608,7 +1665,7 @@ public class TableViewPainted
 			FrequencyLimitedDispatcher(
 				AERunnable.create(()->{
 					Utils.execSWTThread(()->{
-						if ( cTable.isDisposed()){
+						if ( cTable == null || cTable.isDisposed()){
 							return;
 						}
 						if (DEBUG_SELECTION) {
@@ -1685,6 +1742,14 @@ public class TableViewPainted
 					drawBounds = canvasImage.getBounds();
 				}
 			}
+			
+			synchronized( visibleRows_sync ){
+				
+				for ( TableColumnPainted tcp: paintedColumns.values()){
+					
+					tcp.sync();
+				}
+			}
 		}
 		
 		try{
@@ -1922,32 +1987,39 @@ public class TableViewPainted
 		return new Point(x, y);
 	}
 
+	@Override
+	public void enableFilterCheck(Text txtFilter, TableViewFilterCheck<Object> filterCheck) {
+		Object o = txtFilter.getData("BubbleTextBox");
+		if (o instanceof BubbleTextBox) {
+			enableFilterCheck((BubbleTextBox) o, filterCheck, false);
+		}
+	}
+
 	@Override
 	public void 
-	enableFilterCheck(Text txtFilter, TableViewFilterCheck<Object> filterCheck) {
+	enableFilterCheck(BubbleTextBox txtFilter, TableViewFilterCheck<Object> filterCheck) {
 		enableFilterCheck( txtFilter, filterCheck, false );
 	}
 	@Override
 	public void 
 	enableFilterCheck(
-		Text txtFilter,
-	    TableViewFilterCheck<Object> filterCheck,
-	    boolean						 filterSubRows ) 
+		BubbleTextBox filterBox,
+		TableViewFilterCheck<Object> filterCheck,
+		boolean						 filterSubRows ) 
 	{
 		this.filterSubRows = filterSubRows;
-		
+
 		TableViewSWTFilter<?> filter = getSWTFilter();
 		if (filter != null) {
-			if (filter.widget != null && !filter.widget.isDisposed()) {
-				filter.widget.removeKeyListener(tvSWTCommon);
-				filter.widget.removeKeyListener(filter.widgetKeyListener);
-				filter.widget.removeModifyListener(filter.widgetModifyListener);
+			if (filter.filterBox != null && !filter.filterBox.isDisposed()) {
+				filter.filterBox.setKeyListener(null);
+				filter.filterBox.removeBubbleTextBoxChangeListenener(this);
 			}
 		} else {
 			this.filter = filter = new TableViewSWTFilter();
 		}
-		filter.widget = txtFilter;
-		if (txtFilter != null) {
+		filter.filterBox = filterBox;
+		if (filterBox != null) {
 			
 			Class<?> cla = getDataSourceType();
 			
@@ -1963,61 +2035,29 @@ public class TableViewPainted
 			
 				// must create this before adding other listeners so it gets priority over key events
 			
-			TextWithHistory twh = new TextWithHistory( "tableviewpainted.search" + historyKey, "table.filter.history", txtFilter );
+			TextWithHistory twh = new TextWithHistory(
+					"tableviewpainted.search" + historyKey, "table.filter.history",
+					filterBox.getTextWidget());
 			
 				// disable as interferes with key-down into search results feature 
 			
 			twh.setKeDownShowsHistory( false );
 			
-			txtFilter.addListener( SWT.FocusOut, (ev)->{
-				String text = txtFilter.getText().trim();
+			filterBox.getTextWidget().addListener( SWT.FocusOut, (ev)->{
+				String text = filterBox.getText().trim();
 				if ( !text.isEmpty()){
 					twh.addHistory( text );
 				}
 			});
 			
-			txtFilter.addKeyListener(tvSWTCommon);
+			filterBox.setKeyListener(tvSWTCommon);
 
-			filter.widgetModifyListener = new ModifyListener() {
-				@Override
-				public void modifyText(ModifyEvent e) {
-					setFilterText(((Text) e.widget).getText());
-				}
-			};
-			txtFilter.addModifyListener(filter.widgetModifyListener);
-			
-			final TableViewSWTFilter f_filter = filter;
-			
-			filter.widgetKeyListener = new KeyAdapter(){
-				public void keyPressed(KeyEvent event){
-					int key = event.character;
-					if (key <= 26 && key > 0) {
-						key += 'a' - 1;
-					}
-
-					if (event.stateMask == SWT.MOD1) {
-						switch (key) {
-							case 'x': { // CTRL+X: RegEx search switch
-								f_filter.regex = !f_filter.regex;
-								tvSWTCommon.validateFilterRegex();
-								refilter();
-								event.doit = false;	// prevent sound from this key
-							}
-							break;
-						}
-					}
-				}};
-			
-			txtFilter.addKeyListener(filter.widgetKeyListener);
-			
-			if (txtFilter.getText().length() == 0) {
-				txtFilter.setText(filter.text);
-			} else {
-				filter.text = filter.nextText = txtFilter.getText();
+			if (filterBox.getText().length() == 0) {
+				filterBox.setText(filter.text);
 			}
-			
-
 
+			filterBox.setAllowRegex(true);
+			filterBox.addBubbleTextBoxChangeListener(this);
 		} else {
 			filter.text = filter.nextText = "";
 		}
@@ -2035,14 +2075,36 @@ public class TableViewPainted
 	}
 	
 	@Override
-	public Text
-	getFilterControl()
+	public boolean
+	hasFilterControl()
 	{
 		TableViewSWTFilter<?> filter = getSWTFilter();
 
-		return( filter==null?null:filter.widget );
+		return( filter != null && filter.filterBox != null && !filter.filterBox.isDisposed() );
 	}
 
+	private Map<TableColumnCore,TableColumnPainted>	paintedColumns = new HashMap<>();
+	
+	protected TableColumnPainted
+	getColumnPainted(
+		TableColumnCore		c )
+	{
+		TableColumnPainted tcp = paintedColumns.get( c );
+		
+		if ( tcp == null ){
+				
+				// force a sync next draw to pick up initial state
+			
+			mutationCount.incrementAndGet();
+			
+			tcp = new TableColumnPainted( c );
+			
+			paintedColumns.put( c, tcp );
+		}
+		
+		return( tcp );
+	}
+	
 	@Override
 	public void disableFilterCheck() {
 		TableViewSWTFilter<?> filter = getSWTFilter();
@@ -2050,39 +2112,53 @@ public class TableViewPainted
 			return;
 		}
 
-		if (filter.widget != null && !filter.widget.isDisposed()) {
-			filter.widget.removeKeyListener(tvSWTCommon);
-			filter.widget.removeKeyListener(filter.widgetKeyListener);
-			filter.widget.removeModifyListener(filter.widgetModifyListener);
+		if (filter.filterBox != null && !filter.filterBox.isDisposed()) {
+			filter.filterBox.setKeyListener(null);
+			filter.filterBox.removeBubbleTextBoxChangeListenener(this);
 		}
-		filter = null;
+		this.filter = null;
 	}
 
 	/* (non-Javadoc)
 	 * @see com.biglybt.ui.swt.views.table.TableViewSWT#setFilterText(java.lang.String)
 	 */
 	@Override
-	public void setFilterText(String s) {
+	public void setFilterText(String s, boolean force) {
 		if (tvSWTCommon != null) {
-			tvSWTCommon.setFilterText(s);
+			tvSWTCommon.setFilterText(s, force);
 		}
 	}
 
-	/* (non-Javadoc)
-	 * @see com.biglybt.ui.swt.views.table.TableViewSWT#enableSizeSlider(org.eclipse.swt.widgets.Composite, int, int)
-	 */
 	@Override
 	public boolean enableSizeSlider(Composite composite, int min, int max) {
-		// TODO
 		return false;
 	}
 
-	/* (non-Javadoc)
-	 * @see com.biglybt.ui.swt.views.table.TableViewSWT#disableSizeSlider()
-	 */
 	@Override
-	public void disableSizeSlider() {
-		// TODO
+	public void setRowHeight(int newHeight) {
+		if (newHeight == defaultRowHeight) {
+			return;
+		}
+
+		// Note: Various library views use the same tableID (tags torrent list)
+		// Changing one will affect the rest on table re-initialization (restart or closing&re-opening view)
+		// We'll need a sub-id at some point to store settings for specific tables
+		TableColumnManager tcManager = TableColumnManager.getInstance();
+		Map tableConfigMap = tcManager.getTableConfigMap(tableID);
+		tableConfigMap.put("RowHeight", newHeight);
+		tcManager.setTableConfigMap(tableID, tableConfigMap);
+
+		setRowDefaultHeightPX(newHeight);
+		TableRowCore[] rows = getRowsAndSubRows(true);
+		if (rows.length == 0) {
+			return;
+		}
+		for (TableRowCore row : rows) {
+			((TableRowPainted) row).setHeight(newHeight, true);
+		}
+
+		// Bug: First row isn't being painted or something
+		Utils.execSWTThreadLater(0, () -> rows[0].redraw());
 	}
 
 	/* (non-Javadoc)
@@ -2574,7 +2650,7 @@ public class TableViewPainted
 
   		// paint event will handle any changedX or changedW
   		if (changedH || canvasChanged || refreshTable) {
-  			//System.out.println(changedX + ";" + changedY + ";" + changedH + ";" + canvasChanged);
+  			//System.out.println(changedX + ";cY=" + changedY + ";cH=" + changedH + ";cC=" + canvasChanged + ";rT=" + refreshTable);
   			//System.out.println("Redraw " + Debug.getCompressedStackTrace());
 
   			// run refreshTable on SWT (this) thread to ensure rows have been
@@ -2597,7 +2673,12 @@ public class TableViewPainted
 	}
 
 	private void forceDebugShellRefresh(Rectangle bounds) {
-		if (sCanvasImage == null) {
+		if (sCanvasImage == null || sCanvasImage.isDisposed()) {
+			return;
+		}
+
+		if (canvasImage == null || canvasImage.isDisposed()) {
+			sCanvasImage.getShell().setSize(0, 0);
 			return;
 		}
 		Point size = sCanvasImage.getShell().computeSize(
@@ -2610,8 +2691,8 @@ public class TableViewPainted
 				true);
 		}
 		sCanvasImage.update();
-		while (sCanvasImage.getDisplay().readAndDispatch()) {
-		}
+		//while (sCanvasImage.getDisplay().readAndDispatch()) {
+		//}
 	}
 
 	public void swt_updateCanvasImage(boolean immediateRedraw) {
@@ -2901,7 +2982,7 @@ public class TableViewPainted
 					message = "";
 				}
 
-				setFilterText(message);
+				setFilterText(message, false);
 			}
 		});
 	}
@@ -3029,13 +3110,18 @@ public class TableViewPainted
 					menu.setData(MENUKEY_IS_HEADER, true);
 
 				} else {
-					TableRowCore row = getTableRowWithCursor();
-					boolean noRow = row == null;
-
-					// If shell is not active, right clicking on a row will
-					// result in a MenuDetect, but not a MouseDown or MouseUp
-					if (!isSelected(row)) {
-						setSelectedRows(new TableRowCore[] { row });
+					boolean noRow;
+					if (event.detail == SWT.MENU_KEYBOARD) {
+						noRow = getFocusedRow() == null;
+					} else {
+						TableRowCore row = getTableRowWithCursor();
+						noRow = row == null;
+	
+						// If shell is not active, right clicking on a row will
+						// result in a MenuDetect, but not a MouseDown or MouseUp
+						if (!isSelected(row)) {
+							setSelectedRows(new TableRowCore[] { row });
+						}
 					}
 
 					menu.setData(MENUKEY_IN_BLANK_AREA, noRow);
@@ -3076,13 +3162,6 @@ public class TableViewPainted
 		return menu;
 	}
 
-	@Override
-	public void showColumnEditor() {
-		if (tvSWTCommon != null) {
-			tvSWTCommon.showColumnEditor();
-		}
-	}
-
 	@Override
 	public TableRowCore getFocusedRow() {
 		return focusedRow;
@@ -3190,13 +3269,15 @@ public class TableViewPainted
 			int newHeight) {
 
 		synchronized (heightChangeSync) {
-  		totalHeight += (newHeight - oldHeight);
-  		//System.out.println("Height delta: " + (newHeight - oldHeight) + ";ttl=" + totalHeight);
-
-  		if (qdRowHeightChanged) {
-  			return;
-  		}
-  		qdRowHeightChanged = true;
+			totalHeight += (newHeight - oldHeight);
+			//System.out.println("Height delta: " + (newHeight - oldHeight) + ";ttl=" + totalHeight);
+			if ( totalHeight < 0 ){
+				Debug.out( "eh?" );
+			}
+			if (qdRowHeightChanged) {
+				return;
+			}
+			qdRowHeightChanged = true;
 		}
 		Utils.execSWTThreadLater(0, new SWTRunnable() {
 			@Override
@@ -3389,7 +3470,28 @@ public class TableViewPainted
 		return tv == this || (tv == null && isFocused) || (tv != this && tv != null && tv.getSelectedRowsSize() == 0);
 	}
 
+	@Override
+	protected boolean
+	isTableFocused()
+	{
+		return( isFocused );
+	}
+	
 	public boolean isEnabled() {
 		return cTable.isEnabled();
 	}
+
+	@Override
+	public void bubbleTextBoxChanged(BubbleTextBox bubbleTextBox) {
+		boolean changed;
+		if (filter != null) {
+			changed = bubbleTextBox.isRegexEnabled() != filter.regex;
+			if (changed) {
+				filter.regex = !filter.regex;
+			}
+		} else {
+			changed = false;
+		}
+		setFilterText(bubbleTextBox.getText(), changed);
+	}
 }
